/*
 * Copyright (C) 2021-2023 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "MessageNames.h"

namespace IPC::Detail {

const MessageDescription messageDescriptions[static_cast<size_t>(MessageName::Count) + 1] = {
    { "AcceleratedBackingStoreDMABuf_DidCreateBuffer"_s, ReceiverName::AcceleratedBackingStoreDMABuf, false, false },
    { "AcceleratedBackingStoreDMABuf_DidCreateBufferSHM"_s, ReceiverName::AcceleratedBackingStoreDMABuf, false, false },
    { "AcceleratedBackingStoreDMABuf_DidDestroyBuffer"_s, ReceiverName::AcceleratedBackingStoreDMABuf, false, false },
    { "AcceleratedBackingStoreDMABuf_Frame"_s, ReceiverName::AcceleratedBackingStoreDMABuf, false, false },
    { "AcceleratedSurfaceDMABuf_FrameDone"_s, ReceiverName::AcceleratedSurfaceDMABuf, false, false },
    { "AcceleratedSurfaceDMABuf_ReleaseBuffer"_s, ReceiverName::AcceleratedSurfaceDMABuf, false, false },
    { "AuthenticationManager_CompleteAuthenticationChallenge"_s, ReceiverName::AuthenticationManager, false, false },
#if OS(LINUX)
    { "AuxiliaryProcess_DidReceiveMemoryPressureEvent"_s, ReceiverName::AuxiliaryProcess, false, false },
#endif
    { "AuxiliaryProcess_MainThreadPing"_s, ReceiverName::AuxiliaryProcess, false, false },
#if ENABLE(CFPREFS_DIRECT_MODE)
    { "AuxiliaryProcess_PreferenceDidUpdate"_s, ReceiverName::AuxiliaryProcess, false, false },
    { "AuxiliaryProcess_PreferencesDidUpdate"_s, ReceiverName::AuxiliaryProcess, false, false },
#endif
    { "AuxiliaryProcess_SetProcessSuppressionEnabled"_s, ReceiverName::AuxiliaryProcess, false, false },
    { "AuxiliaryProcess_ShutDown"_s, ReceiverName::AuxiliaryProcess, false, false },
    { "DownloadProxy_DecideDestinationWithSuggestedFilename"_s, ReceiverName::DownloadProxy, false, false },
    { "DownloadProxy_DidCreateDestination"_s, ReceiverName::DownloadProxy, false, false },
    { "DownloadProxy_DidFail"_s, ReceiverName::DownloadProxy, false, false },
    { "DownloadProxy_DidFinish"_s, ReceiverName::DownloadProxy, false, false },
    { "DownloadProxy_DidReceiveAuthenticationChallenge"_s, ReceiverName::DownloadProxy, false, false },
    { "DownloadProxy_DidReceiveData"_s, ReceiverName::DownloadProxy, false, false },
    { "DownloadProxy_DidStart"_s, ReceiverName::DownloadProxy, false, false },
    { "DownloadProxy_WillSendRequest"_s, ReceiverName::DownloadProxy, false, false },
    { "DrawingAreaProxy_DidFirstLayerFlush"_s, ReceiverName::DrawingAreaProxy, false, false },
    { "DrawingAreaProxy_DispatchPresentationCallbacksAfterFlushingLayers"_s, ReceiverName::DrawingAreaProxy, false, false },
    { "DrawingAreaProxy_EnterAcceleratedCompositingMode"_s, ReceiverName::DrawingAreaProxy, false, false },
#if USE(COORDINATED_GRAPHICS) || USE(TEXTURE_MAPPER)
    { "DrawingAreaProxy_ExitAcceleratedCompositingMode"_s, ReceiverName::DrawingAreaProxy, false, false },
    { "DrawingAreaProxy_Update"_s, ReceiverName::DrawingAreaProxy, false, false },
#endif
    { "DrawingAreaProxy_UpdateAcceleratedCompositingMode"_s, ReceiverName::DrawingAreaProxy, false, false },
#if PLATFORM(COCOA)
    { "DrawingArea_AcceleratedAnimationDidEnd"_s, ReceiverName::DrawingArea, false, false },
    { "DrawingArea_AcceleratedAnimationDidStart"_s, ReceiverName::DrawingArea, false, false },
#endif
#if PLATFORM(COCOA) || PLATFORM(GTK)
    { "DrawingArea_AdjustTransientZoom"_s, ReceiverName::DrawingArea, false, false },
    { "DrawingArea_CommitTransientZoom"_s, ReceiverName::DrawingArea, false, false },
#endif
#if USE(COORDINATED_GRAPHICS) || USE(TEXTURE_MAPPER)
    { "DrawingArea_DidDiscardBackingStore"_s, ReceiverName::DrawingArea, false, false },
#endif
#if PLATFORM(COCOA)
    { "DrawingArea_DispatchAfterEnsuringDrawing"_s, ReceiverName::DrawingArea, false, false },
#endif
    { "DrawingArea_DisplayDidRefresh"_s, ReceiverName::DrawingArea, false, false },
#if USE(COORDINATED_GRAPHICS) || USE(TEXTURE_MAPPER)
    { "DrawingArea_ForceUpdate"_s, ReceiverName::DrawingArea, false, false },
#endif
#if PLATFORM(COCOA)
    { "DrawingArea_SetColorSpace"_s, ReceiverName::DrawingArea, false, false },
#endif
#if USE(COORDINATED_GRAPHICS) || USE(TEXTURE_MAPPER) || PLATFORM(COCOA)
    { "DrawingArea_SetDeviceScaleFactor"_s, ReceiverName::DrawingArea, false, false },
#endif
#if PLATFORM(COCOA)
    { "DrawingArea_SetViewExposedRect"_s, ReceiverName::DrawingArea, false, false },
#endif
#if USE(COORDINATED_GRAPHICS) || USE(TEXTURE_MAPPER) || PLATFORM(COCOA)
    { "DrawingArea_UpdateGeometry"_s, ReceiverName::DrawingArea, false, false },
#endif
#if USE(GRAPHICS_LAYER_WC)
    { "DrawingArea_UpdateGeometryWC"_s, ReceiverName::DrawingArea, false, false },
#endif
#if HAVE(DISPLAY_LINK)
    { "EventDispatcher_DisplayDidRefresh"_s, ReceiverName::EventDispatcher, false, false },
#endif
#if ENABLE(MAC_GESTURE_EVENTS)
    { "EventDispatcher_GestureEvent"_s, ReceiverName::EventDispatcher, false, false },
#endif
    { "EventDispatcher_PageScreenDidChange"_s, ReceiverName::EventDispatcher, false, false },
#if ENABLE(MOMENTUM_EVENT_DISPATCHER)
    { "EventDispatcher_SetScrollingAccelerationCurve"_s, ReceiverName::EventDispatcher, false, false },
#endif
#if ENABLE(IOS_TOUCH_EVENTS)
    { "EventDispatcher_TouchEvent"_s, ReceiverName::EventDispatcher, false, false },
#endif
    { "EventDispatcher_WheelEvent"_s, ReceiverName::EventDispatcher, false, false },
    { "GPUConnectionToWebProcess_ClearNowPlayingInfo"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
    { "GPUConnectionToWebProcess_ConfigureLoggingChannel"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
    { "GPUConnectionToWebProcess_CreateAudioHardwareListener"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
    { "GPUConnectionToWebProcess_CreateGPU"_s, ReceiverName::GPUConnectionToWebProcess, true, false },
#if ENABLE(WEBGL)
    { "GPUConnectionToWebProcess_CreateGraphicsContextGL"_s, ReceiverName::GPUConnectionToWebProcess, true, false },
#endif
    { "GPUConnectionToWebProcess_CreateRemoteCommandListener"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
    { "GPUConnectionToWebProcess_CreateRenderingBackend"_s, ReceiverName::GPUConnectionToWebProcess, true, false },
#if HAVE(VISIBILITY_PROPAGATION_VIEW)
    { "GPUConnectionToWebProcess_CreateVisibilityPropagationContextForPage"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
#endif
#if USE(GRAPHICS_LAYER_WC)
    { "GPUConnectionToWebProcess_CreateWCLayerTreeHost"_s, ReceiverName::GPUConnectionToWebProcess, true, false },
#endif
#if HAVE(VISIBILITY_PROPAGATION_VIEW)
    { "GPUConnectionToWebProcess_DestroyVisibilityPropagationContextForPage"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
#endif
#if ENABLE(MEDIA_SOURCE)
    { "GPUConnectionToWebProcess_EnableMockMediaSource"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "GPUConnectionToWebProcess_EnsureMediaSessionHelper"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
#endif
    { "GPUConnectionToWebProcess_ReleaseAudioHardwareListener"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
    { "GPUConnectionToWebProcess_ReleaseGPU"_s, ReceiverName::GPUConnectionToWebProcess, true, false },
#if ENABLE(WEBGL)
    { "GPUConnectionToWebProcess_ReleaseGraphicsContextGL"_s, ReceiverName::GPUConnectionToWebProcess, true, false },
#endif
    { "GPUConnectionToWebProcess_ReleaseRemoteCommandListener"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
    { "GPUConnectionToWebProcess_ReleaseRenderingBackend"_s, ReceiverName::GPUConnectionToWebProcess, true, false },
#if USE(GRAPHICS_LAYER_WC)
    { "GPUConnectionToWebProcess_ReleaseWCLayerTreeHost"_s, ReceiverName::GPUConnectionToWebProcess, true, false },
#endif
#if ENABLE(EXTENSION_CAPABILITIES)
    { "GPUConnectionToWebProcess_SetMediaEnvironment"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
#endif
    { "GPUConnectionToWebProcess_SetMediaOverridesForTesting"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
    { "GPUConnectionToWebProcess_SetNowPlayingInfo"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
#if PLATFORM(COCOA) && ENABLE(MEDIA_STREAM)
    { "GPUConnectionToWebProcess_UpdateSampleBufferDisplayLayerBoundsAndPosition"_s, ReceiverName::GPUConnectionToWebProcess, false, false },
#endif
#if ENABLE(ROUTING_ARBITRATION)
    { "GPUProcessConnection_BeginRoutingArbitrationWithCategory"_s, ReceiverName::GPUProcessConnection, false, false },
#endif
    { "GPUProcessConnection_DidInitialize"_s, ReceiverName::GPUProcessConnection, false, false },
    { "GPUProcessConnection_DidReceiveRemoteCommand"_s, ReceiverName::GPUProcessConnection, false, false },
#if ENABLE(ROUTING_ARBITRATION)
    { "GPUProcessConnection_EndRoutingArbitration"_s, ReceiverName::GPUProcessConnection, false, false },
#endif
#if ENABLE(MEDIA_STREAM) && PLATFORM(COCOA)
    { "GPUProcessConnection_ResetAudioMediaStreamTrackRendererInternalUnit"_s, ReceiverName::GPUProcessConnection, false, false },
#endif
#if HAVE(VISIBILITY_PROPAGATION_VIEW)
    { "GPUProcessProxy_DidCreateContextForVisibilityPropagation"_s, ReceiverName::GPUProcessProxy, false, false },
#endif
    { "GPUProcessProxy_ProcessIsReadyToExit"_s, ReceiverName::GPUProcessProxy, false, false },
#if ENABLE(AV1)
    { "GPUProcessProxy_SetHasAV1HardwareDecoder"_s, ReceiverName::GPUProcessProxy, false, false },
#endif
#if ENABLE(VP9)
    { "GPUProcessProxy_SetHasVP9HardwareDecoder"_s, ReceiverName::GPUProcessProxy, false, false },
#endif
#if ENABLE(MEDIA_STREAM) && PLATFORM(IOS_FAMILY)
    { "GPUProcessProxy_StatusBarWasTapped"_s, ReceiverName::GPUProcessProxy, false, false },
#endif
    { "GPUProcessProxy_TerminateWebProcess"_s, ReceiverName::GPUProcessProxy, false, false },
#if ENABLE(MEDIA_STREAM)
    { "GPUProcess_AddMockMediaDevice"_s, ReceiverName::GPUProcess, false, false },
#endif
    { "GPUProcess_AddSession"_s, ReceiverName::GPUProcess, false, false },
#if HAVE(SCREEN_CAPTURE_KIT)
    { "GPUProcess_CancelGetDisplayMediaPrompt"_s, ReceiverName::GPUProcess, false, false },
#endif
#if ENABLE(MEDIA_STREAM)
    { "GPUProcess_ClearMockMediaDevices"_s, ReceiverName::GPUProcess, false, false },
#endif
#if HAVE(AUDIO_COMPONENT_SERVER_REGISTRATIONS)
    { "GPUProcess_ConsumeAudioComponentRegistrations"_s, ReceiverName::GPUProcess, false, false },
#endif
    { "GPUProcess_CreateGPUConnectionToWebProcess"_s, ReceiverName::GPUProcess, true, false },
#if HAVE(POWERLOG_TASK_MODE_QUERY)
    { "GPUProcess_EnablePowerLogging"_s, ReceiverName::GPUProcess, false, false },
#endif
    { "GPUProcess_InitializeGPUProcess"_s, ReceiverName::GPUProcess, false, false },
#if PLATFORM(MAC)
    { "GPUProcess_OpenDirectoryCacheInvalidated"_s, ReceiverName::GPUProcess, false, false },
#endif
    { "GPUProcess_PrepareToSuspend"_s, ReceiverName::GPUProcess, false, false },
    { "GPUProcess_ProcessDidResume"_s, ReceiverName::GPUProcess, false, false },
#if HAVE(SCREEN_CAPTURE_KIT)
    { "GPUProcess_PromptForGetDisplayMedia"_s, ReceiverName::GPUProcess, false, false },
#endif
#if ENABLE(MEDIA_STREAM)
    { "GPUProcess_RemoveMockMediaDevice"_s, ReceiverName::GPUProcess, false, false },
#endif
    { "GPUProcess_RemoveSession"_s, ReceiverName::GPUProcess, false, false },
#if ENABLE(VIDEO)
    { "GPUProcess_RequestBitmapImageForCurrentTime"_s, ReceiverName::GPUProcess, false, false },
#endif
#if ENABLE(MEDIA_STREAM)
    { "GPUProcess_ResetMockMediaDevices"_s, ReceiverName::GPUProcess, false, false },
#endif
#if USE(EXTENSIONKIT)
    { "GPUProcess_ResolveBookmarkDataForCacheDirectory"_s, ReceiverName::GPUProcess, false, false },
#endif
#if ENABLE(MEDIA_STREAM)
    { "GPUProcess_SetMockCaptureDevicesEnabled"_s, ReceiverName::GPUProcess, false, false },
    { "GPUProcess_SetMockCaptureDevicesInterrupted"_s, ReceiverName::GPUProcess, false, false },
    { "GPUProcess_SetMockMediaDeviceIsEphemeral"_s, ReceiverName::GPUProcess, false, false },
    { "GPUProcess_SetOrientationForMediaCapture"_s, ReceiverName::GPUProcess, false, false },
#endif
#if PLATFORM(MAC)
    { "GPUProcess_SetScreenProperties"_s, ReceiverName::GPUProcess, false, false },
#endif
#if HAVE(SC_CONTENT_SHARING_PICKER)
    { "GPUProcess_SetUseSCContentSharingPicker"_s, ReceiverName::GPUProcess, false, false },
#endif
    { "GPUProcess_SharedPreferencesForWebProcessDidChange"_s, ReceiverName::GPUProcess, false, false },
#if ENABLE(MEDIA_STREAM)
    { "GPUProcess_TriggerMockCaptureConfigurationChange"_s, ReceiverName::GPUProcess, false, false },
    { "GPUProcess_UpdateCaptureAccess"_s, ReceiverName::GPUProcess, false, false },
    { "GPUProcess_UpdateCaptureOrigin"_s, ReceiverName::GPUProcess, false, false },
#endif
    { "GPUProcess_UpdateGPUProcessPreferences"_s, ReceiverName::GPUProcess, false, false },
    { "GPUProcess_UpdateSandboxAccess"_s, ReceiverName::GPUProcess, false, false },
    { "GPUProcess_UserPreferredLanguagesChanged"_s, ReceiverName::GPUProcess, false, false },
    { "GPUProcess_WebProcessConnectionCountForTesting"_s, ReceiverName::GPUProcess, false, false },
#if ENABLE(WEBXR)
    { "GPUProcess_WebXRPromptAccepted"_s, ReceiverName::GPUProcess, false, false },
#endif
    { "GtkSettingsManagerProxy_SettingsDidChange"_s, ReceiverName::GtkSettingsManagerProxy, false, false },
    { "IPCConnectionTester_AsyncMessage"_s, ReceiverName::IPCConnectionTester, false, false },
    { "IPCStreamTesterProxy_WasCreated"_s, ReceiverName::IPCStreamTesterProxy, false, false },
    { "IPCStreamTester_AsyncPing"_s, ReceiverName::IPCStreamTester, true, false },
    { "IPCTesterReceiver_AsyncMessage"_s, ReceiverName::IPCTesterReceiver, false, false },
    { "IPCTester_AsyncOptionalExceptionData"_s, ReceiverName::IPCTester, false, false },
    { "IPCTester_AsyncPing"_s, ReceiverName::IPCTester, false, false },
    { "IPCTester_CreateConnectionTester"_s, ReceiverName::IPCTester, false, false },
    { "IPCTester_CreateConnectionTesterAndSendAsyncMessages"_s, ReceiverName::IPCTester, false, false },
    { "IPCTester_CreateStreamTester"_s, ReceiverName::IPCTester, false, false },
    { "IPCTester_SendAsyncMessageToReceiver"_s, ReceiverName::IPCTester, false, false },
    { "IPCTester_SendSameSemaphoreBack"_s, ReceiverName::IPCTester, false, false },
    { "IPCTester_SendSemaphoreBackAndSignalProtocol"_s, ReceiverName::IPCTester, false, false },
    { "IPCTester_StartMessageTesting"_s, ReceiverName::IPCTester, false, false },
    { "LibWebRTCCodecsProxy_CreateDecoder"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_CreateEncoder"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_DecodeFrame"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_EncodeFrame"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_FlushDecoder"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_FlushEncoder"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_InitializeEncoder"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_ReleaseDecoder"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_ReleaseEncoder"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_SetDecoderFormatDescription"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_SetEncodeRates"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_SetFrameSize"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_SetRTCLoggingLevel"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_SetSharedVideoFrameMemory"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecsProxy_SetSharedVideoFrameSemaphore"_s, ReceiverName::LibWebRTCCodecsProxy, false, false },
    { "LibWebRTCCodecs_CompletedDecoding"_s, ReceiverName::LibWebRTCCodecs, false, false },
    { "LibWebRTCCodecs_CompletedDecodingCV"_s, ReceiverName::LibWebRTCCodecs, false, false },
    { "LibWebRTCCodecs_CompletedEncoding"_s, ReceiverName::LibWebRTCCodecs, false, false },
    { "LibWebRTCCodecs_FailedDecoding"_s, ReceiverName::LibWebRTCCodecs, false, false },
    { "LibWebRTCCodecs_FlushDecoderCompleted"_s, ReceiverName::LibWebRTCCodecs, false, false },
    { "LibWebRTCCodecs_SetEncodingConfiguration"_s, ReceiverName::LibWebRTCCodecs, false, false },
    { "LibWebRTCNetwork_SignalAddressReady"_s, ReceiverName::LibWebRTCNetwork, false, false },
    { "LibWebRTCNetwork_SignalClose"_s, ReceiverName::LibWebRTCNetwork, false, false },
    { "LibWebRTCNetwork_SignalConnect"_s, ReceiverName::LibWebRTCNetwork, false, false },
    { "LibWebRTCNetwork_SignalReadPacket"_s, ReceiverName::LibWebRTCNetwork, false, false },
    { "LibWebRTCNetwork_SignalSentPacket"_s, ReceiverName::LibWebRTCNetwork, false, false },
    { "LibWebRTCNetwork_SignalUsedInterface"_s, ReceiverName::LibWebRTCNetwork, false, false },
    { "MediaPlayerPrivateRemote_ActiveSourceBuffersChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_AddDataCue"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#if ENABLE(DATACUE_VALUE)
    { "MediaPlayerPrivateRemote_AddDataCueWithType"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#endif
    { "MediaPlayerPrivateRemote_AddGenericCue"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_AddRemoteAudioTrack"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_AddRemoteTextTrack"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_AddRemoteVideoTrack"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_CharacteristicChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_CommitAllTransactions"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#if ENABLE(WIRELESS_PLAYBACK_TARGET)
    { "MediaPlayerPrivateRemote_CurrentPlaybackTargetIsWirelessChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#endif
    { "MediaPlayerPrivateRemote_CurrentTimeChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_DurationChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_EngineFailedToLoad"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_FirstVideoFrameAvailable"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#if PLATFORM(IOS_FAMILY)
    { "MediaPlayerPrivateRemote_GetRawCookies"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#endif
#if ENABLE(ENCRYPTED_MEDIA)
    { "MediaPlayerPrivateRemote_InitializationDataEncountered"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#endif
#if PLATFORM(COCOA)
    { "MediaPlayerPrivateRemote_LayerHostingContextIdChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#endif
#if ENABLE(LEGACY_ENCRYPTED_MEDIA)
    { "MediaPlayerPrivateRemote_MediaPlayerKeyNeeded"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#endif
    { "MediaPlayerPrivateRemote_MuteChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_NetworkStateChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_ParseWebVTTCueData"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_ParseWebVTTCueDataStruct"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_ParseWebVTTFileHeader"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_PlaybackStateChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#if PLATFORM(COCOA)
    { "MediaPlayerPrivateRemote_PushVideoFrameMetadata"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#endif
    { "MediaPlayerPrivateRemote_RateChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_ReadyStateChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_RemoteAudioTrackConfigurationChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_RemoteTextTrackConfigurationChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_RemoteVideoTrackConfigurationChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#if ENABLE(DATACUE_VALUE)
    { "MediaPlayerPrivateRemote_RemoveDataCue"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#endif
    { "MediaPlayerPrivateRemote_RemoveGenericCue"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_RemoveRemoteAudioTrack"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_RemoveRemoteTextTrack"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_RemoveRemoteVideoTrack"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_RemoveResource"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_RenderingModeChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_RequestResource"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_ResourceNotSupported"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_Seeked"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_SendH2Ping"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_SizeChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_TimeChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_UpdateCachedState"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#if ENABLE(DATACUE_VALUE)
    { "MediaPlayerPrivateRemote_UpdateDataCue"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#endif
    { "MediaPlayerPrivateRemote_UpdateGenericCue"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_UpdatePlaybackQualityMetrics"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_VideoPlaybackConfigurationChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
    { "MediaPlayerPrivateRemote_VolumeChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#if ENABLE(ENCRYPTED_MEDIA)
    { "MediaPlayerPrivateRemote_WaitingForKeyChanged"_s, ReceiverName::MediaPlayerPrivateRemote, false, false },
#endif
    { "MediaSourcePrivateRemoteMessageReceiver_MediaSourcePrivateShuttingDown"_s, ReceiverName::MediaSourcePrivateRemoteMessageReceiver, false, false },
    { "MediaSourcePrivateRemoteMessageReceiver_ProxySeekToTime"_s, ReceiverName::MediaSourcePrivateRemoteMessageReceiver, false, false },
    { "MediaSourcePrivateRemoteMessageReceiver_ProxyWaitForTarget"_s, ReceiverName::MediaSourcePrivateRemoteMessageReceiver, false, false },
    { "ModelConnectionToWebProcess_ConfigureLoggingChannel"_s, ReceiverName::ModelConnectionToWebProcess, false, false },
#if HAVE(VISIBILITY_PROPAGATION_VIEW)
    { "ModelConnectionToWebProcess_CreateVisibilityPropagationContextForPage"_s, ReceiverName::ModelConnectionToWebProcess, false, false },
    { "ModelConnectionToWebProcess_DestroyVisibilityPropagationContextForPage"_s, ReceiverName::ModelConnectionToWebProcess, false, false },
#endif
    { "ModelProcessConnection_DidInitialize"_s, ReceiverName::ModelProcessConnection, false, false },
    { "ModelProcessModelPlayerManagerProxy_CreateModelPlayer"_s, ReceiverName::ModelProcessModelPlayerManagerProxy, false, false },
    { "ModelProcessModelPlayerManagerProxy_DeleteModelPlayer"_s, ReceiverName::ModelProcessModelPlayerManagerProxy, false, false },
    { "ModelProcessModelPlayer_DidCreateLayer"_s, ReceiverName::ModelProcessModelPlayer, false, false },
    { "ModelProcessModelPlayer_DidFinishLoading"_s, ReceiverName::ModelProcessModelPlayer, false, false },
    { "ModelProcessModelPlayer_DidUpdateEntityTransform"_s, ReceiverName::ModelProcessModelPlayer, false, false },
#if HAVE(VISIBILITY_PROPAGATION_VIEW)
    { "ModelProcessProxy_DidCreateContextForVisibilityPropagation"_s, ReceiverName::ModelProcessProxy, false, false },
#endif
    { "ModelProcessProxy_ProcessIsReadyToExit"_s, ReceiverName::ModelProcessProxy, false, false },
    { "ModelProcessProxy_TerminateWebProcess"_s, ReceiverName::ModelProcessProxy, false, false },
    { "ModelProcess_AddSession"_s, ReceiverName::ModelProcess, false, false },
    { "ModelProcess_CreateModelConnectionToWebProcess"_s, ReceiverName::ModelProcess, true, false },
    { "ModelProcess_InitializeModelProcess"_s, ReceiverName::ModelProcess, false, false },
    { "ModelProcess_PrepareToSuspend"_s, ReceiverName::ModelProcess, false, false },
    { "ModelProcess_ProcessDidResume"_s, ReceiverName::ModelProcess, false, false },
    { "ModelProcess_RemoveSession"_s, ReceiverName::ModelProcess, false, false },
    { "ModelProcess_WebProcessConnectionCountForTesting"_s, ReceiverName::ModelProcess, false, false },
    { "NetworkBroadcastChannelRegistry_PostMessage"_s, ReceiverName::NetworkBroadcastChannelRegistry, false, false },
    { "NetworkBroadcastChannelRegistry_RegisterChannel"_s, ReceiverName::NetworkBroadcastChannelRegistry, false, false },
    { "NetworkBroadcastChannelRegistry_UnregisterChannel"_s, ReceiverName::NetworkBroadcastChannelRegistry, false, false },
    { "NetworkConnectionToWebProcess_AddOriginAccessAllowListEntry"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_BrowsingContextRemoved"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_ClearFrameLoadRecordsForStorageAccess"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_ClearPageSpecificData"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_CloseSWContextConnection"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_CloseSharedWorkerContextConnection"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#if ENABLE(WEB_RTC)
    { "NetworkConnectionToWebProcess_ConnectToRTCDataChannelRemoteSource"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#endif
    { "NetworkConnectionToWebProcess_ConvertMainResourceLoadToDownload"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_CookiesEnabled"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_CookiesForDOMAsync"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_CreateNewMessagePortChannel"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_CreateRTCProvider"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_CreateSocketChannel"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_DeleteCookie"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_DestroyWebTransportSession"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_DidDeliverMessagePortMessages"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_EntangleLocalPortInThisProcessToRemote"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_EstablishSWContextConnection"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_EstablishSharedWorkerContextConnection"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#if PLATFORM(MAC)
    { "NetworkConnectionToWebProcess_GetProcessDisplayName"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#endif
    { "NetworkConnectionToWebProcess_HasStorageAccess"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_InitializeWebTransportSession"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#if ENABLE(CONTENT_FILTERING)
    { "NetworkConnectionToWebProcess_InstallMockContentFilter"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#endif
    { "NetworkConnectionToWebProcess_IsLoggedIn"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_IsResourceLoadFinished"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_LoadImageForDecoding"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_LoadPing"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#if ENABLE(LOGD_BLOCKING_IN_WEBCONTENT)
    { "NetworkConnectionToWebProcess_LogOnBehalfOfWebContent"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#endif
    { "NetworkConnectionToWebProcess_LogUserInteraction"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_MessagePortClosed"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_MessagePortDisentangled"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#if ENABLE(DECLARATIVE_WEB_PUSH)
    { "NetworkConnectionToWebProcess_NavigatorGetPushPermissionState"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_NavigatorGetPushSubscription"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_NavigatorSubscribeToPushService"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_NavigatorUnsubscribeFromPushService"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#endif
    { "NetworkConnectionToWebProcess_PageLoadCompleted"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_PostMessageToRemote"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_PreconnectTo"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_PrefetchDNS"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_PrioritizeResourceLoads"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RegisterBlobPathForTesting"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RegisterBlobURL"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RegisterBlobURLHandle"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RegisterInternalBlobURL"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RegisterInternalBlobURLForSlice"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RegisterInternalBlobURLOptionallyFileBacked"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RegisterInternalFileBlobURL"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RegisterURLSchemesAsCORSEnabled"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RemoveLoadIdentifier"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RemoveOriginAccessAllowListEntry"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RemoveStorageAccessForFrame"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RequestStorageAccess"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_RequestStorageAccessUnderOpener"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_ResetOriginAccessAllowLists"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_ResourceLoadStatisticsUpdated"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_ScheduleResourceLoad"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_SendH2Ping"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_SetCORSDisablingPatterns"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_SetCaptureExtraNetworkLoadMetricsEnabled"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_SetCookieFromDOMAsync"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_SetCookiesFromDOM"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_SetLoginStatus"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_SetRawCookie"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_SetResourceLoadSchedulingMode"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_StartDownload"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_StorageAccessQuirkForTopFrameDomain"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#if HAVE(COOKIE_CHANGE_LISTENER_API)
    { "NetworkConnectionToWebProcess_SubscribeToCookieChangeNotifications"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
#endif
    { "NetworkConnectionToWebProcess_TakeAllMessagesForPort"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_UnregisterBlobURL"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_UnregisterBlobURLHandle"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#if HAVE(COOKIE_CHANGE_LISTENER_API)
    { "NetworkConnectionToWebProcess_UnsubscribeFromCookieChangeNotifications"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
#endif
#if PLATFORM(MAC)
    { "NetworkConnectionToWebProcess_UpdateActivePages"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
#endif
    { "NetworkConnectionToWebProcess_UseRedirectionForCurrentNavigation"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkConnectionToWebProcess_WriteBlobsToTemporaryFilesForIndexedDB"_s, ReceiverName::NetworkConnectionToWebProcess, false, false },
    { "NetworkContentRuleListManager_AddContentRuleLists"_s, ReceiverName::NetworkContentRuleListManager, false, false },
    { "NetworkContentRuleListManager_Remove"_s, ReceiverName::NetworkContentRuleListManager, false, false },
    { "NetworkContentRuleListManager_RemoveAllContentRuleLists"_s, ReceiverName::NetworkContentRuleListManager, false, false },
    { "NetworkContentRuleListManager_RemoveContentRuleList"_s, ReceiverName::NetworkContentRuleListManager, false, false },
    { "NetworkMDNSRegister_RegisterMDNSName"_s, ReceiverName::NetworkMDNSRegister, false, false },
    { "NetworkMDNSRegister_UnregisterMDNSNames"_s, ReceiverName::NetworkMDNSRegister, false, false },
#if HAVE(COOKIE_CHANGE_LISTENER_API)
    { "NetworkProcessConnection_AllCookiesDeleted"_s, ReceiverName::NetworkProcessConnection, false, false },
#endif
    { "NetworkProcessConnection_BroadcastConsoleMessage"_s, ReceiverName::NetworkProcessConnection, false, false },
#if ENABLE(WEB_RTC)
    { "NetworkProcessConnection_ConnectToRTCDataChannelRemoteSource"_s, ReceiverName::NetworkProcessConnection, false, false },
#endif
    { "NetworkProcessConnection_CookieAcceptPolicyChanged"_s, ReceiverName::NetworkProcessConnection, false, false },
#if HAVE(COOKIE_CHANGE_LISTENER_API)
    { "NetworkProcessConnection_CookiesAdded"_s, ReceiverName::NetworkProcessConnection, false, false },
    { "NetworkProcessConnection_CookiesDeleted"_s, ReceiverName::NetworkProcessConnection, false, false },
#endif
#if ENABLE(SHAREABLE_RESOURCE)
    { "NetworkProcessConnection_DidCacheResource"_s, ReceiverName::NetworkProcessConnection, false, false },
#endif
    { "NetworkProcessConnection_DidFinishPingLoad"_s, ReceiverName::NetworkProcessConnection, false, false },
    { "NetworkProcessConnection_DidFinishPreconnection"_s, ReceiverName::NetworkProcessConnection, false, false },
    { "NetworkProcessConnection_LoadCancelledDownloadRedirectRequestInFrame"_s, ReceiverName::NetworkProcessConnection, false, false },
    { "NetworkProcessConnection_MessagesAvailableForPort"_s, ReceiverName::NetworkProcessConnection, false, false },
    { "NetworkProcessConnection_SetOnLineState"_s, ReceiverName::NetworkProcessConnection, false, false },
    { "NetworkProcessConnection_UpdateCachedCookiesEnabled"_s, ReceiverName::NetworkProcessConnection, false, false },
#if ENABLE(CONTENT_EXTENSIONS)
    { "NetworkProcessProxy_ContentExtensionRules"_s, ReceiverName::NetworkProcessProxy, false, false },
#endif
    { "NetworkProcessProxy_CookiesDidChange"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DataTaskDidCompleteWithError"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DataTaskDidReceiveData"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DataTaskDidReceiveResponse"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DataTaskReceivedChallenge"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DataTaskWillPerformHTTPRedirection"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DeleteWebsiteDataInUIProcessForRegistrableDomains"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DeleteWebsiteDataInWebProcessesForOrigin"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DidAllowPrivateTokenUsageByThirdPartyForTesting"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DidBlockLoadToKnownTracker"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DidCommitCrossSiteLoadWithDataTransferFromPrevalentResource"_s, ReceiverName::NetworkProcessProxy, false, false },
#if USE(SOUP)
    { "NetworkProcessProxy_DidExceedMemoryLimit"_s, ReceiverName::NetworkProcessProxy, false, false },
#endif
    { "NetworkProcessProxy_DidNegotiateModernTLS"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_DidReceiveAuthenticationChallenge"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_EndServiceWorkerBackgroundProcessing"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_EstablishRemoteWorkerContextConnectionToNetworkProcess"_s, ReceiverName::NetworkProcessProxy, false, false },
#if ENABLE(APP_BOUND_DOMAINS)
    { "NetworkProcessProxy_GetAppBoundDomains"_s, ReceiverName::NetworkProcessProxy, false, false },
#endif
#if ENABLE(APPLE_PAY_REMOTE_UI)
    { "NetworkProcessProxy_GetPaymentCoordinatorEmbeddingUserAgent"_s, ReceiverName::NetworkProcessProxy, false, false },
#endif
#if ENABLE(APPLE_PAY_REMOTE_UI_USES_SCENE)
    { "NetworkProcessProxy_GetWindowSceneAndBundleIdentifierForPaymentPresentation"_s, ReceiverName::NetworkProcessProxy, false, false },
#endif
    { "NetworkProcessProxy_IncreaseQuota"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_LogDiagnosticMessage"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_LogDiagnosticMessageWithResult"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_LogDiagnosticMessageWithValue"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_LogTestingEvent"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_NavigateServiceWorkerClient"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_NegotiatedLegacyTLS"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_NotifyBackgroundFetchChange"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_NotifyResourceLoadStatisticsProcessed"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_NotifyWebsiteDataDeletionForRegistrableDomainsFinished"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_NotifyWebsiteDataScanForRegistrableDomainsFinished"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_OpenWindowFromServiceWorker"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_RegisterRemoteWorkerClientProcess"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_ReloadExecutionContextsForOrigin"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_RemoteWorkerContextConnectionNoLongerNeeded"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_ReportConsoleMessage"_s, ReceiverName::NetworkProcessProxy, false, false },
#if ENABLE(NETWORK_ISSUE_REPORTING)
    { "NetworkProcessProxy_ReportNetworkIssue"_s, ReceiverName::NetworkProcessProxy, false, false },
#endif
    { "NetworkProcessProxy_RequestBackgroundFetchPermission"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_RequestStorageAccessConfirm"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_ResourceLoadDidCompleteWithError"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_ResourceLoadDidPerformHTTPRedirection"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_ResourceLoadDidReceiveChallenge"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_ResourceLoadDidReceiveResponse"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_ResourceLoadDidSendRequest"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_SetDomainsWithCrossPageStorageAccess"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_SetDomainsWithUserInteraction"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_SetWebProcessHasUploads"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_StartServiceWorkerBackgroundProcessing"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_TerminateUnresponsiveServiceWorkerProcesses"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_TerminateWebProcess"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_TriggerBrowsingContextGroupSwitchForNavigation"_s, ReceiverName::NetworkProcessProxy, false, false },
    { "NetworkProcessProxy_UnregisterRemoteWorkerClientProcess"_s, ReceiverName::NetworkProcessProxy, false, false },
#if USE(RUNNINGBOARD)
    { "NetworkProcessProxy_WakeUpWebProcessForIPC"_s, ReceiverName::NetworkProcessProxy, false, false },
#endif
    { "NetworkProcess_AbortBackgroundFetch"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_AddAllowedFirstPartyForCookies"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_AddWebPageNetworkParameters"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_AddWebsiteDataStore"_s, ReceiverName::NetworkProcess, true, false },
    { "NetworkProcess_AllowFileAccessFromWebProcess"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_AllowFilesAccessFromWebProcess"_s, ReceiverName::NetworkProcess, false, false },
#if !PLATFORM(COCOA)
    { "NetworkProcess_AllowSpecificHTTPSCertificateForHost"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_AllowTLSCertificateChainForLocalPCMTesting"_s, ReceiverName::NetworkProcess, false, false },
#if PLATFORM(COCOA)
    { "NetworkProcess_AppPrivacyReportTestingData"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_ApplicationDidEnterBackground"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ApplicationWillEnterForeground"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_CancelDataTask"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_CancelDownload"_s, ReceiverName::NetworkProcess, false, false },
#if ENABLE(APP_BOUND_DOMAINS)
    { "NetworkProcess_ClearAppBoundSession"_s, ReceiverName::NetworkProcess, false, false },
#endif
#if PLATFORM(COCOA)
    { "NetworkProcess_ClearAppPrivacyReportTestingData"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_ClearBundleIdentifier"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ClearPrevalentResource"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ClearPrivateClickMeasurement"_s, ReceiverName::NetworkProcess, false, false },
#if HAVE(NW_PROXY_CONFIG)
    { "NetworkProcess_ClearProxyConfigData"_s, ReceiverName::NetworkProcess, false, false },
#endif
#if PLATFORM(IOS_FAMILY) && !PLATFORM(MACCATALYST)
    { "NetworkProcess_ClearServiceWorkerEntitlementOverride"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_ClearUserInteraction"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ClickBackgroundFetch"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_CloneSessionStorageForWebPage"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_CloseITPDatabase"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ClosePCMDatabase"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_CountNonDefaultSessionSets"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_CreateNetworkConnectionToWebProcess"_s, ReceiverName::NetworkProcess, true, false },
    { "NetworkProcess_DataTaskWithRequest"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_DeleteCookiesForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_DeleteWebsiteData"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_DeleteWebsiteDataForOrigins"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_DestroySession"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_DidCommitCrossSiteLoadWithDataTransfer"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_DidIncreaseQuota"_s, ReceiverName::NetworkProcess, false, false },
#if PLATFORM(IOS_FAMILY) && !PLATFORM(MACCATALYST)
    { "NetworkProcess_DisableServiceWorkerEntitlement"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_DomainIDExistsInDatabase"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_DownloadRequest"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_DumpPrivateClickMeasurement"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_DumpResourceLoadStatistics"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_FetchWebsiteData"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_FlushCookies"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_GetAllBackgroundFetchIdentifiers"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_GetAllStorageAccessEntries"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_GetAppBadgeForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_GetBackgroundFetchState"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_GetPendingPushMessage"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_GetPendingPushMessages"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_GetResourceLoadStatisticsDataSummary"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_GrantStorageAccessForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_HadUserInteraction"_s, ReceiverName::NetworkProcess, false, false },
#if ENABLE(APP_BOUND_DOMAINS)
    { "NetworkProcess_HasAppBoundSession"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_HasIsolatedSession"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_HasLocalStorage"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_HasPushSubscriptionForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_InitializeNetworkProcess"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_InsertExpiredStatisticForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_IsGrandfathered"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_IsPrevalentResource"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_IsRegisteredAsRedirectingTo"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_IsRegisteredAsSubFrameUnder"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_IsRegisteredAsSubresourceUnder"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_IsRelationshipOnlyInDatabaseOnce"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_IsResourceLoadStatisticsEphemeral"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_IsVeryPrevalentResource"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_LogFrameNavigation"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_LogUserInteraction"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_MarkAttributedPrivateClickMeasurementsAsExpiredForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_MarkPrivateClickMeasurementsAsExpiredForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_MergeStatisticForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_NotifyMediaStreamingActivity"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_PauseBackgroundFetch"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_PreconnectTo"_s, ReceiverName::NetworkProcess, false, false },
#if USE(SOUP)
    { "NetworkProcess_PrefetchDNS"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_PrepareToSuspend"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ProcessDidResume"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ProcessNotificationEvent"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ProcessPushMessage"_s, ReceiverName::NetworkProcess, false, false },
#if HAVE(MODERN_DOWNLOADPROGRESS) || PLATFORM(COCOA) && !HAVE(MODERN_DOWNLOADPROGRESS)
    { "NetworkProcess_PublishDownloadProgress"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_RegisterURLSchemeAsBypassingContentSecurityPolicy"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_RegisterURLSchemeAsLocal"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_RegisterURLSchemeAsNoAccess"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_RegisterURLSchemeAsSecure"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_RemovePushSubscriptionsForOrigin"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_RemoveWebPageNetworkParameters"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_RenameOriginInWebsiteData"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ResetCacheMaxAgeCapForPrevalentResources"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ResetCrossSiteLoadsWithLinkDecorationForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ResetParametersToDefaultValues"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ResetQuota"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ResetStoragePersistedState"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ResumeBackgroundFetch"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ResumeDownload"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ScheduleClearInMemoryAndPersistent"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ScheduleCookieBlockingUpdate"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_ScheduleStatisticsAndDataRecordsProcessing"_s, ReceiverName::NetworkProcess, false, false },
#if ENABLE(APP_BOUND_DOMAINS)
    { "NetworkProcess_SetAppBoundDomainsForResourceLoadStatistics"_s, ReceiverName::NetworkProcess, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "NetworkProcess_SetBackupExclusionPeriodForTesting"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_SetBlobRegistryTopOriginPartitioningEnabled"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetCacheMaxAgeCapForPrevalentResources"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetCacheModel"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetCrossSiteLoadWithLinkDecorationForTesting"_s, ReceiverName::NetworkProcess, false, false },
#if ENABLE(INSPECTOR_NETWORK_THROTTLING)
    { "NetworkProcess_SetEmulatedConditions"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_SetFirstPartyHostCNAMEDomainForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetFirstPartyWebsiteDataRemovalModeForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetGrandfathered"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetGrandfatheringTime"_s, ReceiverName::NetworkProcess, false, false },
#if USE(SOUP)
    { "NetworkProcess_SetIgnoreTLSErrors"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_SetInspectionForServiceWorkersAllowed"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetIsRunningResourceLoadStatisticsTest"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetLastSeen"_s, ReceiverName::NetworkProcess, false, false },
#if ENABLE(MANAGED_DOMAINS)
    { "NetworkProcess_SetManagedDomainsForResourceLoadStatistics"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_SetMaxStatisticsEntries"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetMinimumTimeBetweenDataRecordsRemoval"_s, ReceiverName::NetworkProcess, false, false },
#if USE(SOUP) || USE(CURL)
    { "NetworkProcess_SetNetworkProxySettings"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_SetNotifyPagesWhenDataRecordsWereScanned"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetOriginQuotaRatioEnabledForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPCMFraudPreventionValuesForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPersistedDomains"_s, ReceiverName::NetworkProcess, false, false },
#if USE(SOUP)
    { "NetworkProcess_SetPersistentCredentialStorageEnabled"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_SetPrevalentResource"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPrevalentResourceForDebugMode"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementAppBundleIDForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementAttributionReportURLsForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementDebugMode"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementEphemeralMeasurementForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementOverrideTimerForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementTokenPublicKeyURLForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementTokenSignatureURLForTesting"_s, ReceiverName::NetworkProcess, false, false },
#if HAVE(NW_PROXY_CONFIG)
    { "NetworkProcess_SetProxyConfigData"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_SetPruneEntriesDownTo"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetPushAndNotificationsEnabledForOrigin"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetResourceLoadStatisticsDebugMode"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetResourceLoadStatisticsLogTestingEvent"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetResourceLoadStatisticsTimeAdvanceForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetSessionIsControlledByAutomation"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetShouldClassifyResourcesBeforeDataRecordsRemoval"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetShouldDowngradeReferrerForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetShouldEnbleSameSiteStrictEnforcementForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetShouldSendPrivateTokenIPCForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetStorageSiteValidationEnabled"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetSubframeUnderTopFrameDomain"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetSubresourceUnderTopFrameDomain"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetSubresourceUniqueRedirectFrom"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetSubresourceUniqueRedirectTo"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetThirdPartyCNAMEDomainForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetThirdPartyCookieBlockingMode"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetTimeToLiveUserInteraction"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetToSameSiteStrictCookiesForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetTopFrameUniqueRedirectFrom"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetTopFrameUniqueRedirectTo"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetTrackingPreventionEnabled"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SetVeryPrevalentResource"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SharedPreferencesForWebProcessDidChange"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SimulatePrivateClickMeasurementSessionRestart"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_StatisticsDatabaseHasAllTables"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_StorePrivateClickMeasurement"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_StoreServiceWorkerRegistrations"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_SyncLocalStorage"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_TerminateRemoteWorkerContextConnectionWhenPossible"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_UpdateBundleIdentifier"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_UpdatePrevalentDomainsToBlockCookiesFor"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkProcess_UpdateStorageAccessPromptQuirks"_s, ReceiverName::NetworkProcess, false, false },
#if USE(SOUP)
    { "NetworkProcess_UserPreferredLanguagesChanged"_s, ReceiverName::NetworkProcess, false, false },
#endif
    { "NetworkProcess_WebsiteDataOriginDirectoryForTesting"_s, ReceiverName::NetworkProcess, false, false },
    { "NetworkRTCMonitor_StartUpdatingIfNeeded"_s, ReceiverName::NetworkRTCMonitor, false, false },
    { "NetworkRTCMonitor_StopUpdating"_s, ReceiverName::NetworkRTCMonitor, false, false },
    { "NetworkRTCProvider_CloseSocket"_s, ReceiverName::NetworkRTCProvider, false, false },
    { "NetworkRTCProvider_CreateClientTCPSocket"_s, ReceiverName::NetworkRTCProvider, false, false },
    { "NetworkRTCProvider_CreateResolver"_s, ReceiverName::NetworkRTCProvider, false, false },
    { "NetworkRTCProvider_CreateUDPSocket"_s, ReceiverName::NetworkRTCProvider, false, false },
#if PLATFORM(COCOA)
    { "NetworkRTCProvider_GetInterfaceName"_s, ReceiverName::NetworkRTCProvider, false, false },
#endif
    { "NetworkRTCProvider_SendToSocket"_s, ReceiverName::NetworkRTCProvider, false, false },
    { "NetworkRTCProvider_SetSocketOption"_s, ReceiverName::NetworkRTCProvider, false, false },
    { "NetworkRTCProvider_StopResolver"_s, ReceiverName::NetworkRTCProvider, false, false },
    { "NetworkResourceLoader_ContinueDidReceiveResponse"_s, ReceiverName::NetworkResourceLoader, false, false },
    { "NetworkSocketChannel_Close"_s, ReceiverName::NetworkSocketChannel, false, false },
    { "NetworkSocketChannel_SendData"_s, ReceiverName::NetworkSocketChannel, false, false },
    { "NetworkSocketChannel_SendString"_s, ReceiverName::NetworkSocketChannel, false, false },
    { "NetworkStorageManager_AbortOpenAndUpgradeNeeded"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_AbortTransaction"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CacheStorageAllCaches"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CacheStorageClearMemoryRepresentation"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CacheStorageDereference"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CacheStorageOpenCache"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CacheStoragePutRecords"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CacheStorageReference"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CacheStorageRemoveCache"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CacheStorageRemoveRecords"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CacheStorageRepresentation"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CacheStorageRetrieveRecords"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CancelConnectToStorageArea"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_Clear"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_ClearObjectStore"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CloseHandle"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CloseSyncAccessHandle"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CommitTransaction"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_ConnectToStorageArea"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CreateIndex"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CreateObjectStore"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_CreateSyncAccessHandle"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_DatabaseConnectionClosed"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_DatabaseConnectionPendingClose"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_DeleteDatabase"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_DeleteIndex"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_DeleteObjectStore"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_DeleteRecord"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_DidFinishHandlingVersionChangeTransaction"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_DidFireVersionChangeEvent"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_DisconnectFromStorageArea"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_EstablishTransaction"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_Estimate"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_FileSystemGetDirectory"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_GetAllDatabaseNamesAndVersions"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_GetAllRecords"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_GetCount"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_GetDirectoryHandle"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_GetFile"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_GetFileHandle"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_GetHandle"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_GetHandleNames"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_GetRecord"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_IsSameEntry"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_IterateCursor"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_LockCacheStorage"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_Move"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_OpenCursor"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_OpenDBRequestCancelled"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_OpenDatabase"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_Persist"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_Persisted"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_PutOrAdd"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_RemoveEntry"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_RemoveItem"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_RenameIndex"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_RenameObjectStore"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_RequestNewCapacityForSyncAccessHandle"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_ResetQuotaUpdatedBasedOnUsageForTesting"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_Resolve"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_SetItem"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkStorageManager_UnlockCacheStorage"_s, ReceiverName::NetworkStorageManager, false, false },
    { "NetworkTransportSession_CreateBidirectionalStream"_s, ReceiverName::NetworkTransportSession, false, false },
    { "NetworkTransportSession_CreateOutgoingUnidirectionalStream"_s, ReceiverName::NetworkTransportSession, false, false },
    { "NetworkTransportSession_DestroyBidirectionalStream"_s, ReceiverName::NetworkTransportSession, false, false },
    { "NetworkTransportSession_DestroyOutgoingUnidirectionalStream"_s, ReceiverName::NetworkTransportSession, false, false },
    { "NetworkTransportSession_SendDatagram"_s, ReceiverName::NetworkTransportSession, false, false },
    { "NetworkTransportSession_SendStreamSendBytes"_s, ReceiverName::NetworkTransportSession, false, false },
    { "NetworkTransportSession_StreamSendBytes"_s, ReceiverName::NetworkTransportSession, false, false },
    { "NetworkTransportSession_Terminate"_s, ReceiverName::NetworkTransportSession, false, false },
    { "NotificationManagerMessageHandler_CancelNotification"_s, ReceiverName::NotificationManagerMessageHandler, false, false },
    { "NotificationManagerMessageHandler_ClearNotifications"_s, ReceiverName::NotificationManagerMessageHandler, false, false },
    { "NotificationManagerMessageHandler_DidDestroyNotification"_s, ReceiverName::NotificationManagerMessageHandler, false, false },
    { "NotificationManagerMessageHandler_GetPermissionState"_s, ReceiverName::NotificationManagerMessageHandler, false, false },
    { "NotificationManagerMessageHandler_PageWasNotifiedOfNotificationPermission"_s, ReceiverName::NotificationManagerMessageHandler, false, false },
    { "NotificationManagerMessageHandler_RequestPermission"_s, ReceiverName::NotificationManagerMessageHandler, false, false },
    { "NotificationManagerMessageHandler_SetAppBadge"_s, ReceiverName::NotificationManagerMessageHandler, false, false },
    { "NotificationManagerMessageHandler_ShowNotification"_s, ReceiverName::NotificationManagerMessageHandler, false, false },
    { "PlatformXRSystemProxy_SessionDidEnd"_s, ReceiverName::PlatformXRSystemProxy, false, false },
    { "PlatformXRSystemProxy_SessionDidUpdateVisibilityState"_s, ReceiverName::PlatformXRSystemProxy, false, false },
    { "PlatformXRSystem_DidCompleteShutdownTriggeredBySystem"_s, ReceiverName::PlatformXRSystem, false, false },
    { "PlatformXRSystem_EnumerateImmersiveXRDevices"_s, ReceiverName::PlatformXRSystem, false, false },
    { "PlatformXRSystem_InitializeTrackingAndRendering"_s, ReceiverName::PlatformXRSystem, false, false },
    { "PlatformXRSystem_RequestFrame"_s, ReceiverName::PlatformXRSystem, false, false },
    { "PlatformXRSystem_RequestPermissionOnSessionFeatures"_s, ReceiverName::PlatformXRSystem, false, false },
    { "PlatformXRSystem_ShutDownTrackingAndRendering"_s, ReceiverName::PlatformXRSystem, false, false },
    { "PlatformXRSystem_SubmitFrame"_s, ReceiverName::PlatformXRSystem, false, false },
    { "PushClientConnection_CancelNotification"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_GetAppBadgeForTesting"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_GetNotifications"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_GetPendingPushMessage"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_GetPendingPushMessages"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_GetPushPermissionState"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_GetPushSubscription"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_GetPushTopicsForTesting"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_IncrementSilentPushCount"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_InitializeConnection"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_InjectEncryptedPushMessageForTesting"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_InjectPushMessageForTesting"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_RemoveAllPushSubscriptions"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_RemovePushSubscriptionsForOrigin"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_RequestPushPermission"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_SetAppBadge"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_SetPublicTokenForTesting"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_SetPushAndNotificationsEnabledForOrigin"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_ShowNotification"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_SubscribeToPushService"_s, ReceiverName::PushClientConnection, false, false },
    { "PushClientConnection_UnsubscribeFromPushService"_s, ReceiverName::PushClientConnection, false, false },
    { "RTCDataChannelRemoteManagerProxy_BufferedAmountIsDecreasing"_s, ReceiverName::RTCDataChannelRemoteManagerProxy, false, false },
    { "RTCDataChannelRemoteManagerProxy_ChangeReadyState"_s, ReceiverName::RTCDataChannelRemoteManagerProxy, false, false },
    { "RTCDataChannelRemoteManagerProxy_Close"_s, ReceiverName::RTCDataChannelRemoteManagerProxy, false, false },
    { "RTCDataChannelRemoteManagerProxy_DetectError"_s, ReceiverName::RTCDataChannelRemoteManagerProxy, false, false },
    { "RTCDataChannelRemoteManagerProxy_ReceiveData"_s, ReceiverName::RTCDataChannelRemoteManagerProxy, false, false },
    { "RTCDataChannelRemoteManagerProxy_SendData"_s, ReceiverName::RTCDataChannelRemoteManagerProxy, false, false },
    { "RTCDataChannelRemoteManager_BufferedAmountIsDecreasing"_s, ReceiverName::RTCDataChannelRemoteManager, false, false },
    { "RTCDataChannelRemoteManager_ChangeReadyState"_s, ReceiverName::RTCDataChannelRemoteManager, false, false },
    { "RTCDataChannelRemoteManager_Close"_s, ReceiverName::RTCDataChannelRemoteManager, false, false },
    { "RTCDataChannelRemoteManager_DetectError"_s, ReceiverName::RTCDataChannelRemoteManager, false, false },
    { "RTCDataChannelRemoteManager_ReceiveData"_s, ReceiverName::RTCDataChannelRemoteManager, false, false },
    { "RTCDataChannelRemoteManager_SendData"_s, ReceiverName::RTCDataChannelRemoteManager, false, false },
    { "RemoteAdapter_Destruct"_s, ReceiverName::RemoteAdapter, true, false },
#if PLATFORM(COCOA)
    { "RemoteAudioDestinationManager_AudioSamplesStorageChanged"_s, ReceiverName::RemoteAudioDestinationManager, false, false },
#endif
    { "RemoteAudioDestinationManager_CreateAudioDestination"_s, ReceiverName::RemoteAudioDestinationManager, false, false },
    { "RemoteAudioDestinationManager_DeleteAudioDestination"_s, ReceiverName::RemoteAudioDestinationManager, false, false },
    { "RemoteAudioDestinationManager_StartAudioDestination"_s, ReceiverName::RemoteAudioDestinationManager, false, false },
    { "RemoteAudioDestinationManager_StopAudioDestination"_s, ReceiverName::RemoteAudioDestinationManager, false, false },
    { "RemoteAudioHardwareListener_AudioHardwareDidBecomeActive"_s, ReceiverName::RemoteAudioHardwareListener, false, false },
    { "RemoteAudioHardwareListener_AudioHardwareDidBecomeInactive"_s, ReceiverName::RemoteAudioHardwareListener, false, false },
    { "RemoteAudioHardwareListener_AudioOutputDeviceChanged"_s, ReceiverName::RemoteAudioHardwareListener, false, false },
    { "RemoteAudioMediaStreamTrackRendererInternalUnitManager_CreateUnit"_s, ReceiverName::RemoteAudioMediaStreamTrackRendererInternalUnitManager, false, false },
    { "RemoteAudioMediaStreamTrackRendererInternalUnitManager_DeleteUnit"_s, ReceiverName::RemoteAudioMediaStreamTrackRendererInternalUnitManager, false, false },
    { "RemoteAudioMediaStreamTrackRendererInternalUnitManager_SetAudioOutputDevice"_s, ReceiverName::RemoteAudioMediaStreamTrackRendererInternalUnitManager, false, false },
    { "RemoteAudioMediaStreamTrackRendererInternalUnitManager_StartUnit"_s, ReceiverName::RemoteAudioMediaStreamTrackRendererInternalUnitManager, false, false },
    { "RemoteAudioMediaStreamTrackRendererInternalUnitManager_StopUnit"_s, ReceiverName::RemoteAudioMediaStreamTrackRendererInternalUnitManager, false, false },
    { "RemoteAudioSessionProxy_BeginInterruptionRemote"_s, ReceiverName::RemoteAudioSessionProxy, false, false },
    { "RemoteAudioSessionProxy_EndInterruptionRemote"_s, ReceiverName::RemoteAudioSessionProxy, false, false },
    { "RemoteAudioSessionProxy_SetCategory"_s, ReceiverName::RemoteAudioSessionProxy, false, false },
    { "RemoteAudioSessionProxy_SetIsPlayingToBluetoothOverride"_s, ReceiverName::RemoteAudioSessionProxy, false, false },
    { "RemoteAudioSessionProxy_SetPreferredBufferSize"_s, ReceiverName::RemoteAudioSessionProxy, false, false },
    { "RemoteAudioSessionProxy_SetSceneIdentifier"_s, ReceiverName::RemoteAudioSessionProxy, false, false },
    { "RemoteAudioSessionProxy_SetSoundStageSize"_s, ReceiverName::RemoteAudioSessionProxy, false, false },
    { "RemoteAudioSessionProxy_TriggerBeginInterruptionForTesting"_s, ReceiverName::RemoteAudioSessionProxy, false, false },
    { "RemoteAudioSessionProxy_TriggerEndInterruptionForTesting"_s, ReceiverName::RemoteAudioSessionProxy, false, false },
    { "RemoteAudioSession_BeginInterruptionRemote"_s, ReceiverName::RemoteAudioSession, false, false },
    { "RemoteAudioSession_ConfigurationChanged"_s, ReceiverName::RemoteAudioSession, false, false },
    { "RemoteAudioSession_EndInterruptionRemote"_s, ReceiverName::RemoteAudioSession, false, false },
    { "RemoteAudioSourceProviderManager_AudioSamplesAvailable"_s, ReceiverName::RemoteAudioSourceProviderManager, false, false },
    { "RemoteAudioSourceProviderManager_AudioStorageChanged"_s, ReceiverName::RemoteAudioSourceProviderManager, false, false },
    { "RemoteBarcodeDetector_Detect"_s, ReceiverName::RemoteBarcodeDetector, true, false },
    { "RemoteBindGroupLayout_Destruct"_s, ReceiverName::RemoteBindGroupLayout, true, false },
    { "RemoteBindGroupLayout_SetLabel"_s, ReceiverName::RemoteBindGroupLayout, true, false },
    { "RemoteBindGroup_Destruct"_s, ReceiverName::RemoteBindGroup, true, false },
    { "RemoteBindGroup_SetLabel"_s, ReceiverName::RemoteBindGroup, true, false },
    { "RemoteBindGroup_UpdateExternalTextures"_s, ReceiverName::RemoteBindGroup, true, false },
    { "RemoteBuffer_Copy"_s, ReceiverName::RemoteBuffer, true, false },
    { "RemoteBuffer_Destroy"_s, ReceiverName::RemoteBuffer, true, false },
    { "RemoteBuffer_Destruct"_s, ReceiverName::RemoteBuffer, true, false },
    { "RemoteBuffer_MapAsync"_s, ReceiverName::RemoteBuffer, true, false },
    { "RemoteBuffer_SetLabel"_s, ReceiverName::RemoteBuffer, true, false },
    { "RemoteBuffer_Unmap"_s, ReceiverName::RemoteBuffer, true, false },
    { "RemoteCDMFactoryProxy_RemoveInstance"_s, ReceiverName::RemoteCDMFactoryProxy, false, false },
    { "RemoteCDMFactoryProxy_RemoveSession"_s, ReceiverName::RemoteCDMFactoryProxy, false, false },
    { "RemoteCDMInstanceProxy_InitializeWithConfiguration"_s, ReceiverName::RemoteCDMInstanceProxy, false, false },
    { "RemoteCDMInstanceProxy_SetServerCertificate"_s, ReceiverName::RemoteCDMInstanceProxy, false, false },
    { "RemoteCDMInstanceProxy_SetStorageDirectory"_s, ReceiverName::RemoteCDMInstanceProxy, false, false },
    { "RemoteCDMInstanceSessionProxy_CloseSession"_s, ReceiverName::RemoteCDMInstanceSessionProxy, false, false },
    { "RemoteCDMInstanceSessionProxy_LoadSession"_s, ReceiverName::RemoteCDMInstanceSessionProxy, false, false },
    { "RemoteCDMInstanceSessionProxy_RemoveSessionData"_s, ReceiverName::RemoteCDMInstanceSessionProxy, false, false },
    { "RemoteCDMInstanceSessionProxy_RequestLicense"_s, ReceiverName::RemoteCDMInstanceSessionProxy, false, false },
    { "RemoteCDMInstanceSessionProxy_SetLogIdentifier"_s, ReceiverName::RemoteCDMInstanceSessionProxy, false, false },
    { "RemoteCDMInstanceSessionProxy_StoreRecordOfKeyUsage"_s, ReceiverName::RemoteCDMInstanceSessionProxy, false, false },
    { "RemoteCDMInstanceSessionProxy_UpdateLicense"_s, ReceiverName::RemoteCDMInstanceSessionProxy, false, false },
    { "RemoteCDMInstanceSession_SendMessage"_s, ReceiverName::RemoteCDMInstanceSession, false, false },
    { "RemoteCDMInstanceSession_SessionIdChanged"_s, ReceiverName::RemoteCDMInstanceSession, false, false },
    { "RemoteCDMInstanceSession_UpdateKeyStatuses"_s, ReceiverName::RemoteCDMInstanceSession, false, false },
    { "RemoteCDMInstance_UnrequestedInitializationDataReceived"_s, ReceiverName::RemoteCDMInstance, false, false },
    { "RemoteCDMProxy_GetSupportedConfiguration"_s, ReceiverName::RemoteCDMProxy, false, false },
    { "RemoteCDMProxy_LoadAndInitialize"_s, ReceiverName::RemoteCDMProxy, false, false },
    { "RemoteCDMProxy_SetLogIdentifier"_s, ReceiverName::RemoteCDMProxy, false, false },
    { "RemoteCommandBuffer_Destruct"_s, ReceiverName::RemoteCommandBuffer, true, false },
    { "RemoteCommandBuffer_SetLabel"_s, ReceiverName::RemoteCommandBuffer, true, false },
    { "RemoteCommandEncoder_BeginComputePass"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_BeginRenderPass"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_ClearBuffer"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_CopyBufferToBuffer"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_CopyBufferToTexture"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_CopyTextureToBuffer"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_CopyTextureToTexture"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_Destruct"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_Finish"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_InsertDebugMarker"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_PopDebugGroup"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_PushDebugGroup"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_ResolveQuerySet"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_SetLabel"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCommandEncoder_WriteTimestamp"_s, ReceiverName::RemoteCommandEncoder, true, false },
    { "RemoteCompositorIntegration_Destruct"_s, ReceiverName::RemoteCompositorIntegration, true, false },
    { "RemoteComputePassEncoder_Destruct"_s, ReceiverName::RemoteComputePassEncoder, true, false },
    { "RemoteComputePassEncoder_Dispatch"_s, ReceiverName::RemoteComputePassEncoder, true, false },
    { "RemoteComputePassEncoder_DispatchIndirect"_s, ReceiverName::RemoteComputePassEncoder, true, false },
    { "RemoteComputePassEncoder_End"_s, ReceiverName::RemoteComputePassEncoder, true, false },
    { "RemoteComputePassEncoder_InsertDebugMarker"_s, ReceiverName::RemoteComputePassEncoder, true, false },
    { "RemoteComputePassEncoder_PopDebugGroup"_s, ReceiverName::RemoteComputePassEncoder, true, false },
    { "RemoteComputePassEncoder_PushDebugGroup"_s, ReceiverName::RemoteComputePassEncoder, true, false },
    { "RemoteComputePassEncoder_SetBindGroup"_s, ReceiverName::RemoteComputePassEncoder, true, false },
    { "RemoteComputePassEncoder_SetLabel"_s, ReceiverName::RemoteComputePassEncoder, true, false },
    { "RemoteComputePassEncoder_SetPipeline"_s, ReceiverName::RemoteComputePassEncoder, true, false },
    { "RemoteComputePipeline_Destruct"_s, ReceiverName::RemoteComputePipeline, true, false },
    { "RemoteComputePipeline_GetBindGroupLayout"_s, ReceiverName::RemoteComputePipeline, true, false },
    { "RemoteComputePipeline_SetLabel"_s, ReceiverName::RemoteComputePipeline, true, false },
    { "RemoteDevice_CreateBindGroup"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateBindGroupLayout"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateBuffer"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateCommandEncoder"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateComputePipeline"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateComputePipelineAsync"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreatePipelineLayout"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateQuerySet"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateRenderBundleEncoder"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateRenderPipeline"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateRenderPipelineAsync"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateSampler"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateShaderModule"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateTexture"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_CreateXRBinding"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_Destroy"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_Destruct"_s, ReceiverName::RemoteDevice, true, false },
#if PLATFORM(COCOA) && ENABLE(VIDEO)
    { "RemoteDevice_ImportExternalTextureFromVideoFrame"_s, ReceiverName::RemoteDevice, true, false },
#endif
    { "RemoteDevice_PopErrorScope"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_PushErrorScope"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_ResolveDeviceLostPromise"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_ResolveUncapturedErrorEvent"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_SetLabel"_s, ReceiverName::RemoteDevice, true, false },
#if PLATFORM(COCOA) && ENABLE(VIDEO)
    { "RemoteDevice_SetSharedVideoFrameMemory"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_SetSharedVideoFrameSemaphore"_s, ReceiverName::RemoteDevice, true, false },
    { "RemoteDevice_UpdateExternalTexture"_s, ReceiverName::RemoteDevice, true, false },
#endif
    { "RemoteDisplayListRecorder_ApplyDeviceScaleFactor"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#if USE(CG)
    { "RemoteDisplayListRecorder_ApplyFillPattern"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_ApplyStrokePattern"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#endif
    { "RemoteDisplayListRecorder_BeginTransparencyLayer"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_BeginTransparencyLayerWithCompositeMode"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_ClearDropShadow"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_ClearRect"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_Clip"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_ClipOut"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_ClipOutRoundedRect"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_ClipOutToPath"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_ClipPath"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_ClipRoundedRect"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_ClipToImageBuffer"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_ConcatenateCTM"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawControlPart"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawDecomposedGlyphs"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawDisplayListItems"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawDotsForDocumentMarker"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawEllipse"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawFilteredImageBuffer"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawFocusRingPath"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawFocusRingRects"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawGlyphs"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawImageBuffer"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawLine"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawLinesForText"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawNativeImage"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawPath"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawPattern"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawRect"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_DrawSystemImage"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_EndTransparencyLayer"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#if ENABLE(INLINE_PATH_DATA)
    { "RemoteDisplayListRecorder_FillArc"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_FillBezierCurve"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_FillClosedArc"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#endif
    { "RemoteDisplayListRecorder_FillCompositedRect"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_FillEllipse"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#if ENABLE(INLINE_PATH_DATA)
    { "RemoteDisplayListRecorder_FillLine"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#endif
    { "RemoteDisplayListRecorder_FillPath"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_FillPathSegment"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#if ENABLE(INLINE_PATH_DATA)
    { "RemoteDisplayListRecorder_FillQuadCurve"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#endif
    { "RemoteDisplayListRecorder_FillRect"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_FillRectWithColor"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_FillRectWithGradient"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_FillRectWithGradientAndSpaceTransform"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_FillRectWithRoundedHole"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_FillRoundedRect"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#if ENABLE(VIDEO)
    { "RemoteDisplayListRecorder_PaintFrameForMedia"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#endif
#if PLATFORM(COCOA) && ENABLE(VIDEO)
    { "RemoteDisplayListRecorder_PaintVideoFrame"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#endif
    { "RemoteDisplayListRecorder_ResetClip"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_Restore"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_Rotate"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_Save"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_Scale"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_SetCTM"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_SetInlineFillColor"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_SetInlineStroke"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_SetLineCap"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_SetLineDash"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_SetLineJoin"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_SetMiterLimit"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#if PLATFORM(COCOA) && ENABLE(VIDEO)
    { "RemoteDisplayListRecorder_SetSharedVideoFrameMemory"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_SetSharedVideoFrameSemaphore"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#endif
    { "RemoteDisplayListRecorder_SetState"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#if ENABLE(INLINE_PATH_DATA)
    { "RemoteDisplayListRecorder_StrokeArc"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_StrokeBezierCurve"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_StrokeClosedArc"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#endif
    { "RemoteDisplayListRecorder_StrokeEllipse"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#if ENABLE(INLINE_PATH_DATA)
    { "RemoteDisplayListRecorder_StrokeLine"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_StrokeLineWithColorAndThickness"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#endif
    { "RemoteDisplayListRecorder_StrokePath"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_StrokePathSegment"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#if ENABLE(INLINE_PATH_DATA)
    { "RemoteDisplayListRecorder_StrokeQuadCurve"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
#endif
    { "RemoteDisplayListRecorder_StrokeRect"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteDisplayListRecorder_Translate"_s, ReceiverName::RemoteDisplayListRecorder, true, false },
    { "RemoteExternalTexture_Destroy"_s, ReceiverName::RemoteExternalTexture, true, false },
    { "RemoteExternalTexture_Destruct"_s, ReceiverName::RemoteExternalTexture, true, false },
    { "RemoteExternalTexture_SetLabel"_s, ReceiverName::RemoteExternalTexture, true, false },
    { "RemoteExternalTexture_Undestroy"_s, ReceiverName::RemoteExternalTexture, true, false },
    { "RemoteFaceDetector_Detect"_s, ReceiverName::RemoteFaceDetector, true, false },
    { "RemoteGPUProxy_WasCreated"_s, ReceiverName::RemoteGPUProxy, false, false },
    { "RemoteGPU_CreateCompositorIntegration"_s, ReceiverName::RemoteGPU, true, false },
    { "RemoteGPU_CreatePresentationContext"_s, ReceiverName::RemoteGPU, true, false },
    { "RemoteGraphicsContextGLProxy_WasCreated"_s, ReceiverName::RemoteGraphicsContextGLProxy, false, false },
    { "RemoteGraphicsContextGLProxy_WasLost"_s, ReceiverName::RemoteGraphicsContextGLProxy, false, false },
    { "RemoteGraphicsContextGLProxy_addDebugMessage"_s, ReceiverName::RemoteGraphicsContextGLProxy, false, false },
    { "RemoteGraphicsContextGL_ActiveTexture"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_AttachShader"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BeginQuery"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BeginQueryEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BeginTransformFeedback"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BindAttribLocation"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BindBuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BindBufferBase"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BindBufferRange"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(WEBXR)
    { "RemoteGraphicsContextGL_BindExternalImage"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_BindFramebuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BindRenderbuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BindSampler"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BindTexture"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BindTransformFeedback"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BindVertexArray"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BlendColor"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BlendEquation"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BlendEquationSeparate"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BlendEquationSeparateiOES"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BlendEquationiOES"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BlendFunc"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BlendFuncSeparate"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BlendFuncSeparateiOES"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BlendFunciOES"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BlitFramebuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BufferData0"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BufferData1"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_BufferSubData"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Clear"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ClearBufferfi"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ClearBufferfv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ClearBufferiv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ClearBufferuiv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ClearColor"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ClearDepth"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ClearStencil"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ClipControlEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ColorMask"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ColorMaskiOES"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CompileShader"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CompressedTexImage2D0"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CompressedTexImage2D1"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CompressedTexImage3D0"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CompressedTexImage3D1"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CompressedTexSubImage2D0"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CompressedTexSubImage2D1"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CompressedTexSubImage3D0"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CompressedTexSubImage3D1"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CopyBufferSubData"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CopyTexImage2D"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CopyTexSubImage2D"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CopyTexSubImage3D"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateBuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(WEBXR)
    { "RemoteGraphicsContextGL_CreateExternalImage"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateExternalSync"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_CreateFramebuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateProgram"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateQuery"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateQueryEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateRenderbuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateSampler"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateShader"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateTexture"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateTransformFeedback"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CreateVertexArray"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_CullFace"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteBuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(WEBXR)
    { "RemoteGraphicsContextGL_DeleteExternalImage"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_DeleteExternalSync"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteFramebuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteProgram"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteQuery"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteQueryEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteRenderbuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteSampler"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteShader"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteSync"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteTexture"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteTransformFeedback"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DeleteVertexArray"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DepthFunc"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DepthMask"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DepthRange"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DetachShader"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Disable"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(WEBXR)
    { "RemoteGraphicsContextGL_DisableFoveation"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_DisableVertexAttribArray"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DisableiOES"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DrawArrays"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DrawArraysInstanced"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DrawArraysInstancedBaseInstanceANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DrawBuffers"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DrawBuffersEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DrawElements"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DrawElementsInstanced"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DrawElementsInstancedBaseVertexBaseInstanceANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_DrawRangeElements"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Enable"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(WEBXR)
    { "RemoteGraphicsContextGL_EnableFoveation"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_EnableVertexAttribArray"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_EnableiOES"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_EndQuery"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_EndQueryEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_EndTransformFeedback"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_EnsureExtensionEnabled"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Finish"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Flush"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(WEBXR)
    { "RemoteGraphicsContextGL_FramebufferDiscard"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_FramebufferRenderbuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(WEBXR)
    { "RemoteGraphicsContextGL_FramebufferResolveRenderbuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_FramebufferTexture2D"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_FramebufferTextureLayer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_FrontFace"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GenerateMipmap"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Hint"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_InvalidateFramebuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_InvalidateSubFramebuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_LineWidth"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_LinkProgram"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_MultiDrawArraysANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_MultiDrawArraysInstancedANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_MultiDrawArraysInstancedBaseInstanceANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_MultiDrawElementsANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_MultiDrawElementsInstancedANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_MultiDrawElementsInstancedBaseVertexBaseInstanceANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_PauseTransformFeedback"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_PixelStorei"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_PolygonModeANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_PolygonOffset"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_PolygonOffsetClampEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ProvokingVertexANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_QueryCounterEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ReadBuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ReadPixelsBufferObject"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_RenderbufferStorage"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_RenderbufferStorageMultisample"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_RenderbufferStorageMultisampleANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Reshape"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ResumeTransformFeedback"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_SampleCoverage"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_SamplerParameterf"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_SamplerParameteri"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Scissor"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_SetDrawingBufferColorSpace"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(VIDEO) && PLATFORM(COCOA)
    { "RemoteGraphicsContextGL_SetSharedVideoFrameMemory"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_SetSharedVideoFrameSemaphore"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_ShaderSource"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_SimulateEventForTesting"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_StencilFunc"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_StencilFuncSeparate"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_StencilMask"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_StencilMaskSeparate"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_StencilOp"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_StencilOpSeparate"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexImage2D0"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexImage2D1"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexImage3D0"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexImage3D1"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexParameterf"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexParameteri"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexStorage2D"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexStorage3D"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexSubImage2D0"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexSubImage2D1"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexSubImage3D0"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TexSubImage3D1"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_TransformFeedbackVaryings"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform1f"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform1fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform1i"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform1iv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform1ui"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform1uiv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform2f"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform2fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform2i"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform2iv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform2ui"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform2uiv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform3f"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform3fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform3i"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform3iv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform3ui"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform3uiv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform4f"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform4fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform4i"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform4iv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform4ui"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Uniform4uiv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UniformBlockBinding"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UniformMatrix2fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UniformMatrix2x3fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UniformMatrix2x4fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UniformMatrix3fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UniformMatrix3x2fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UniformMatrix3x4fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UniformMatrix4fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UniformMatrix4x2fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UniformMatrix4x3fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_UseProgram"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ValidateProgram"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttrib1f"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttrib1fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttrib2f"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttrib2fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttrib3f"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttrib3fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttrib4f"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttrib4fv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttribDivisor"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttribI4i"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttribI4iv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttribI4ui"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttribI4uiv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttribIPointer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_VertexAttribPointer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_Viewport"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_WaitSync"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteImageBufferProxy_DidCreateBackend"_s, ReceiverName::RemoteImageBufferProxy, false, false },
#if PLATFORM(COCOA)
    { "RemoteImageBufferSetProxy_DidPrepareForDisplay"_s, ReceiverName::RemoteImageBufferSetProxy, false, false },
#endif
    { "RemoteImageBufferSet_EndPrepareForDisplay"_s, ReceiverName::RemoteImageBufferSet, true, false },
    { "RemoteImageBufferSet_UpdateConfiguration"_s, ReceiverName::RemoteImageBufferSet, true, false },
    { "RemoteImageBuffer_ConvertToLuminanceMask"_s, ReceiverName::RemoteImageBuffer, true, false },
    { "RemoteImageBuffer_FlushContext"_s, ReceiverName::RemoteImageBuffer, true, false },
    { "RemoteImageBuffer_PutPixelBuffer"_s, ReceiverName::RemoteImageBuffer, true, false },
    { "RemoteImageBuffer_TransformToColorSpace"_s, ReceiverName::RemoteImageBuffer, true, false },
    { "RemoteLegacyCDMFactoryProxy_RemoveSession"_s, ReceiverName::RemoteLegacyCDMFactoryProxy, false, false },
    { "RemoteLegacyCDMProxy_SetPlayerId"_s, ReceiverName::RemoteLegacyCDMProxy, false, false },
    { "RemoteLegacyCDMSessionProxy_ReleaseKeys"_s, ReceiverName::RemoteLegacyCDMSessionProxy, false, false },
    { "RemoteLegacyCDMSession_SendError"_s, ReceiverName::RemoteLegacyCDMSession, false, false },
    { "RemoteLegacyCDMSession_SendMessage"_s, ReceiverName::RemoteLegacyCDMSession, false, false },
    { "RemoteMediaEngineConfigurationFactoryProxy_CreateDecodingConfiguration"_s, ReceiverName::RemoteMediaEngineConfigurationFactoryProxy, false, false },
    { "RemoteMediaEngineConfigurationFactoryProxy_CreateEncodingConfiguration"_s, ReceiverName::RemoteMediaEngineConfigurationFactoryProxy, false, false },
    { "RemoteMediaPlayerManagerProxy_CreateMediaPlayer"_s, ReceiverName::RemoteMediaPlayerManagerProxy, false, false },
    { "RemoteMediaPlayerManagerProxy_DeleteMediaPlayer"_s, ReceiverName::RemoteMediaPlayerManagerProxy, false, false },
    { "RemoteMediaPlayerProxy_AcceleratedRenderingStateChanged"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_ApplicationDidBecomeActive"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_ApplicationWillResignActive"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if ENABLE(ENCRYPTED_MEDIA)
    { "RemoteMediaPlayerProxy_AttemptToDecryptWithInstance"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaPlayerProxy_AudioTrackSetEnabled"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_BeginSimulatedHDCPError"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_CancelLoad"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if ENABLE(ENCRYPTED_MEDIA)
    { "RemoteMediaPlayerProxy_CdmInstanceAttached"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_CdmInstanceDetached"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
#if ENABLE(WEB_AUDIO)
    { "RemoteMediaPlayerProxy_CreateAudioSourceProvider"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaPlayerProxy_DidLoadingProgress"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_EndSimulatedHDCPError"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_IsInFullscreenOrPictureInPictureChanged"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if ENABLE(LEGACY_ENCRYPTED_MEDIA)
    { "RemoteMediaPlayerProxy_KeyAdded"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaPlayerProxy_Load"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if ENABLE(MEDIA_SOURCE)
    { "RemoteMediaPlayerProxy_LoadMediaSource"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaPlayerProxy_NotifyActiveSourceBuffersChanged"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_NotifyTrackModeChanged"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_Pause"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_PauseAtHostTime"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_PerformTaskAtTime"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_Play"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_PlayAtHostTime"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_PlayerContentBoxRectChanged"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_PrepareForPlayback"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_PrepareForRendering"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_PrepareToPlay"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_RequestHostingContextID"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SeekToTarget"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetBufferingPolicy"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if HAVE(SPATIAL_TRACKING_LABEL)
    { "RemoteMediaPlayerProxy_SetDefaultSpatialTrackingLabel"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
#if ENABLE(LEGACY_ENCRYPTED_MEDIA)
    { "RemoteMediaPlayerProxy_SetLegacyCDMSession"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaPlayerProxy_SetMuted"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetPageIsVisible"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetPitchCorrectionAlgorithm"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetPreferredDynamicRangeMode"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetPreload"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetPresentationSize"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetPreservesPitch"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetPrivateBrowsingMode"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetRate"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetShouldCheckHardwareSupport"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if ENABLE(LEGACY_ENCRYPTED_MEDIA) && ENABLE(ENCRYPTED_MEDIA)
    { "RemoteMediaPlayerProxy_SetShouldContinueAfterKeyNeeded"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaPlayerProxy_SetShouldDisableHDR"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetShouldDisableSleep"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if ENABLE(WEB_AUDIO)
    { "RemoteMediaPlayerProxy_SetShouldEnableAudioSourceProvider"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaPlayerProxy_SetShouldMaintainAspectRatio"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if ENABLE(WIRELESS_PLAYBACK_TARGET)
    { "RemoteMediaPlayerProxy_SetShouldPlayToPlaybackTarget"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
#if HAVE(SPATIAL_TRACKING_LABEL)
    { "RemoteMediaPlayerProxy_SetSpatialTrackingLabel"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
#if ENABLE(VIDEO_PRESENTATION_MODE)
    { "RemoteMediaPlayerProxy_SetVideoFullscreenGravity"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetVideoFullscreenMode"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
#if PLATFORM(COCOA)
    { "RemoteMediaPlayerProxy_SetVideoLayerSizeFenced"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaPlayerProxy_SetVideoPlaybackMetricsUpdateInterval"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetVolume"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if ENABLE(WIRELESS_PLAYBACK_TARGET)
    { "RemoteMediaPlayerProxy_SetWirelessPlaybackTarget"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_SetWirelessVideoPlaybackDisabled"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaPlayerProxy_StartVideoFrameMetadataGathering"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_StopVideoFrameMetadataGathering"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_TextTrackSetMode"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_TracksChanged"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if ENABLE(VIDEO_PRESENTATION_MODE)
    { "RemoteMediaPlayerProxy_UpdateVideoFullscreenInlineImage"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
    { "RemoteMediaPlayerProxy_VideoFullscreenStandbyChanged"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaPlayerProxy_VideoTrackSetSelected"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#if (PLATFORM(COCOA) && !HAVE(AVSAMPLEBUFFERDISPLAYLAYER_COPYDISPLAYEDPIXELBUFFER))
    { "RemoteMediaPlayerProxy_WillBeAskedToPaintGL"_s, ReceiverName::RemoteMediaPlayerProxy, false, false },
#endif
    { "RemoteMediaRecorderManager_CreateRecorder"_s, ReceiverName::RemoteMediaRecorderManager, false, false },
    { "RemoteMediaRecorderManager_ReleaseRecorder"_s, ReceiverName::RemoteMediaRecorderManager, false, false },
    { "RemoteMediaRecorder_AudioSamplesAvailable"_s, ReceiverName::RemoteMediaRecorder, false, false },
    { "RemoteMediaRecorder_AudioSamplesStorageChanged"_s, ReceiverName::RemoteMediaRecorder, false, false },
    { "RemoteMediaRecorder_FetchData"_s, ReceiverName::RemoteMediaRecorder, false, false },
    { "RemoteMediaRecorder_Pause"_s, ReceiverName::RemoteMediaRecorder, false, false },
    { "RemoteMediaRecorder_Resume"_s, ReceiverName::RemoteMediaRecorder, false, false },
    { "RemoteMediaRecorder_SetSharedVideoFrameMemory"_s, ReceiverName::RemoteMediaRecorder, false, false },
    { "RemoteMediaRecorder_SetSharedVideoFrameSemaphore"_s, ReceiverName::RemoteMediaRecorder, false, false },
    { "RemoteMediaRecorder_StopRecording"_s, ReceiverName::RemoteMediaRecorder, false, false },
    { "RemoteMediaRecorder_VideoFrameAvailable"_s, ReceiverName::RemoteMediaRecorder, false, false },
    { "RemoteMediaResourceManager_AccessControlCheckFailed"_s, ReceiverName::RemoteMediaResourceManager, false, false },
    { "RemoteMediaResourceManager_DataReceived"_s, ReceiverName::RemoteMediaResourceManager, false, false },
    { "RemoteMediaResourceManager_DataSent"_s, ReceiverName::RemoteMediaResourceManager, false, false },
    { "RemoteMediaResourceManager_LoadFailed"_s, ReceiverName::RemoteMediaResourceManager, false, false },
    { "RemoteMediaResourceManager_LoadFinished"_s, ReceiverName::RemoteMediaResourceManager, false, false },
    { "RemoteMediaResourceManager_RedirectReceived"_s, ReceiverName::RemoteMediaResourceManager, false, false },
    { "RemoteMediaResourceManager_ResponseReceived"_s, ReceiverName::RemoteMediaResourceManager, false, false },
    { "RemoteMediaSessionCoordinatorProxy_CoordinatePause"_s, ReceiverName::RemoteMediaSessionCoordinatorProxy, false, false },
    { "RemoteMediaSessionCoordinatorProxy_CoordinatePlay"_s, ReceiverName::RemoteMediaSessionCoordinatorProxy, false, false },
    { "RemoteMediaSessionCoordinatorProxy_CoordinateSeekTo"_s, ReceiverName::RemoteMediaSessionCoordinatorProxy, false, false },
    { "RemoteMediaSessionCoordinatorProxy_CoordinateSetTrack"_s, ReceiverName::RemoteMediaSessionCoordinatorProxy, false, false },
    { "RemoteMediaSessionCoordinatorProxy_Join"_s, ReceiverName::RemoteMediaSessionCoordinatorProxy, false, false },
    { "RemoteMediaSessionCoordinatorProxy_Leave"_s, ReceiverName::RemoteMediaSessionCoordinatorProxy, false, false },
    { "RemoteMediaSessionCoordinatorProxy_PlaybackStateChanged"_s, ReceiverName::RemoteMediaSessionCoordinatorProxy, false, false },
    { "RemoteMediaSessionCoordinatorProxy_PositionStateChanged"_s, ReceiverName::RemoteMediaSessionCoordinatorProxy, false, false },
    { "RemoteMediaSessionCoordinatorProxy_ReadyStateChanged"_s, ReceiverName::RemoteMediaSessionCoordinatorProxy, false, false },
    { "RemoteMediaSessionCoordinatorProxy_TrackIdentifierChanged"_s, ReceiverName::RemoteMediaSessionCoordinatorProxy, false, false },
    { "RemoteMediaSessionCoordinator_CoordinatorStateChanged"_s, ReceiverName::RemoteMediaSessionCoordinator, false, false },
    { "RemoteMediaSessionCoordinator_PauseSession"_s, ReceiverName::RemoteMediaSessionCoordinator, false, false },
    { "RemoteMediaSessionCoordinator_PlaySession"_s, ReceiverName::RemoteMediaSessionCoordinator, false, false },
    { "RemoteMediaSessionCoordinator_SeekSessionToTime"_s, ReceiverName::RemoteMediaSessionCoordinator, false, false },
    { "RemoteMediaSessionCoordinator_SetSessionTrack"_s, ReceiverName::RemoteMediaSessionCoordinator, false, false },
    { "RemoteMediaSourceProxy_BufferedChanged"_s, ReceiverName::RemoteMediaSourceProxy, false, false },
    { "RemoteMediaSourceProxy_DurationChanged"_s, ReceiverName::RemoteMediaSourceProxy, false, false },
    { "RemoteMediaSourceProxy_MarkEndOfStream"_s, ReceiverName::RemoteMediaSourceProxy, false, false },
    { "RemoteMediaSourceProxy_SetMediaPlayerReadyState"_s, ReceiverName::RemoteMediaSourceProxy, false, false },
    { "RemoteMediaSourceProxy_SetTimeFudgeFactor"_s, ReceiverName::RemoteMediaSourceProxy, false, false },
    { "RemoteMediaSourceProxy_UnmarkEndOfStream"_s, ReceiverName::RemoteMediaSourceProxy, false, false },
    { "RemotePipelineLayout_Destruct"_s, ReceiverName::RemotePipelineLayout, true, false },
    { "RemotePipelineLayout_SetLabel"_s, ReceiverName::RemotePipelineLayout, true, false },
    { "RemotePresentationContext_Configure"_s, ReceiverName::RemotePresentationContext, true, false },
    { "RemotePresentationContext_GetCurrentTexture"_s, ReceiverName::RemotePresentationContext, true, false },
    { "RemotePresentationContext_Present"_s, ReceiverName::RemotePresentationContext, true, false },
    { "RemotePresentationContext_Unconfigure"_s, ReceiverName::RemotePresentationContext, true, false },
    { "RemoteQuerySet_Destroy"_s, ReceiverName::RemoteQuerySet, true, false },
    { "RemoteQuerySet_Destruct"_s, ReceiverName::RemoteQuerySet, true, false },
    { "RemoteQuerySet_SetLabel"_s, ReceiverName::RemoteQuerySet, true, false },
    { "RemoteQueue_CopyExternalImageToTexture"_s, ReceiverName::RemoteQueue, true, false },
    { "RemoteQueue_Destruct"_s, ReceiverName::RemoteQueue, true, false },
    { "RemoteQueue_OnSubmittedWorkDone"_s, ReceiverName::RemoteQueue, true, false },
    { "RemoteQueue_SetLabel"_s, ReceiverName::RemoteQueue, true, false },
    { "RemoteQueue_Submit"_s, ReceiverName::RemoteQueue, true, false },
    { "RemoteQueue_WriteBuffer"_s, ReceiverName::RemoteQueue, true, false },
    { "RemoteQueue_WriteTexture"_s, ReceiverName::RemoteQueue, true, false },
    { "RemoteRemoteCommandListenerProxy_UpdateSupportedCommands"_s, ReceiverName::RemoteRemoteCommandListenerProxy, false, false },
    { "RemoteRemoteCommandListener_DidReceiveRemoteControlCommand"_s, ReceiverName::RemoteRemoteCommandListener, false, false },
    { "RemoteRenderBundleEncoder_Destruct"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_Draw"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_DrawIndexed"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_DrawIndexedIndirect"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_DrawIndirect"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_Finish"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_InsertDebugMarker"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_PopDebugGroup"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_PushDebugGroup"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_SetBindGroup"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_SetIndexBuffer"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_SetLabel"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_SetPipeline"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_SetVertexBuffer"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundleEncoder_UnsetVertexBuffer"_s, ReceiverName::RemoteRenderBundleEncoder, true, false },
    { "RemoteRenderBundle_Destruct"_s, ReceiverName::RemoteRenderBundle, true, false },
    { "RemoteRenderBundle_SetLabel"_s, ReceiverName::RemoteRenderBundle, true, false },
    { "RemoteRenderPassEncoder_BeginOcclusionQuery"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_Destruct"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_Draw"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_DrawIndexed"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_DrawIndexedIndirect"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_DrawIndirect"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_End"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_EndOcclusionQuery"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_ExecuteBundles"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_InsertDebugMarker"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_PopDebugGroup"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_PushDebugGroup"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_SetBindGroup"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_SetBlendConstant"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_SetIndexBuffer"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_SetLabel"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_SetPipeline"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_SetScissorRect"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_SetStencilReference"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_SetVertexBuffer"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_SetViewport"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPassEncoder_UnsetVertexBuffer"_s, ReceiverName::RemoteRenderPassEncoder, true, false },
    { "RemoteRenderPipeline_Destruct"_s, ReceiverName::RemoteRenderPipeline, true, false },
    { "RemoteRenderPipeline_GetBindGroupLayout"_s, ReceiverName::RemoteRenderPipeline, true, false },
    { "RemoteRenderPipeline_SetLabel"_s, ReceiverName::RemoteRenderPipeline, true, false },
    { "RemoteRenderingBackendProxy_DidFinalizeRenderingUpdate"_s, ReceiverName::RemoteRenderingBackendProxy, false, false },
    { "RemoteRenderingBackendProxy_DidInitialize"_s, ReceiverName::RemoteRenderingBackendProxy, false, false },
    { "RemoteRenderingBackendProxy_DidMarkLayersAsVolatile"_s, ReceiverName::RemoteRenderingBackendProxy, false, false },
    { "RemoteRenderingBackend_CacheDecomposedGlyphs"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_CacheFilter"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_CacheFont"_s, ReceiverName::RemoteRenderingBackend, true, false },
#if PLATFORM(COCOA) || !PLATFORM(COCOA)
    { "RemoteRenderingBackend_CacheFontCustomPlatformData"_s, ReceiverName::RemoteRenderingBackend, true, false },
#endif
    { "RemoteRenderingBackend_CacheGradient"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_CacheNativeImage"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_CreateImageBuffer"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_CreateRemoteBarcodeDetector"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_CreateRemoteFaceDetector"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_CreateRemoteImageBufferSet"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_CreateRemoteTextDetector"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_DestroyGetPixelBufferSharedMemory"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_FinalizeRenderingUpdate"_s, ReceiverName::RemoteRenderingBackend, true, false },
#if USE(GRAPHICS_LAYER_WC)
    { "RemoteRenderingBackend_Flush"_s, ReceiverName::RemoteRenderingBackend, true, false },
#endif
    { "RemoteRenderingBackend_GetImageBufferResourceLimitsForTesting"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_GetRemoteBarcodeDetectorSupportedFormats"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_MarkSurfacesVolatile"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_MoveToImageBuffer"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_MoveToSerializedBuffer"_s, ReceiverName::RemoteRenderingBackend, true, false },
#if PLATFORM(COCOA)
    { "RemoteRenderingBackend_PrepareImageBufferSetsForDisplay"_s, ReceiverName::RemoteRenderingBackend, true, false },
#endif
    { "RemoteRenderingBackend_ReleaseAllDrawingResources"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_ReleaseAllImageResources"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_ReleaseImageBuffer"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_ReleaseRemoteBarcodeDetector"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_ReleaseRemoteFaceDetector"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_ReleaseRemoteImageBufferSet"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_ReleaseRemoteTextDetector"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteRenderingBackend_ReleaseRenderingResource"_s, ReceiverName::RemoteRenderingBackend, true, false },
    { "RemoteSampleBufferDisplayLayerManager_CreateLayer"_s, ReceiverName::RemoteSampleBufferDisplayLayerManager, false, false },
    { "RemoteSampleBufferDisplayLayerManager_ReleaseLayer"_s, ReceiverName::RemoteSampleBufferDisplayLayerManager, false, false },
    { "RemoteSampleBufferDisplayLayer_ClearVideoFrames"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
    { "RemoteSampleBufferDisplayLayer_EnqueueVideoFrame"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
    { "RemoteSampleBufferDisplayLayer_Flush"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
    { "RemoteSampleBufferDisplayLayer_FlushAndRemoveImage"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
    { "RemoteSampleBufferDisplayLayer_Pause"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
    { "RemoteSampleBufferDisplayLayer_Play"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
#if !RELEASE_LOG_DISABLED
    { "RemoteSampleBufferDisplayLayer_SetLogIdentifier"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
#endif
    { "RemoteSampleBufferDisplayLayer_SetSharedVideoFrameMemory"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
    { "RemoteSampleBufferDisplayLayer_SetSharedVideoFrameSemaphore"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
    { "RemoteSampleBufferDisplayLayer_SetShouldMaintainAspectRatio"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
    { "RemoteSampleBufferDisplayLayer_UpdateDisplayMode"_s, ReceiverName::RemoteSampleBufferDisplayLayer, false, false },
    { "RemoteSampler_Destruct"_s, ReceiverName::RemoteSampler, true, false },
    { "RemoteSampler_SetLabel"_s, ReceiverName::RemoteSampler, true, false },
    { "RemoteShaderModule_CompilationInfo"_s, ReceiverName::RemoteShaderModule, true, false },
    { "RemoteShaderModule_Destruct"_s, ReceiverName::RemoteShaderModule, true, false },
    { "RemoteShaderModule_SetLabel"_s, ReceiverName::RemoteShaderModule, true, false },
    { "RemoteSharedResourceCache_ReleaseSerializedImageBuffer"_s, ReceiverName::RemoteSharedResourceCache, false, false },
    { "RemoteSourceBufferProxy_Abort"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_AddTrackBuffer"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_Append"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_AsyncEvictCodedFrames"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_BufferedSamplesForTrackId"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_ClearTrackBuffers"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_ComputeSeekTime"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_EnqueuedSamplesForTrackID"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_MemoryPressure"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_ReenqueueMediaIfNeeded"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_RemoveCodedFrames"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_RemovedFromMediaSource"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_ResetParserState"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_ResetTimestampOffsetInTrackBuffers"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_ResetTrackBuffers"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SeekToTime"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetActive"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetAllTrackBuffersNeedRandomAccess"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetAppendWindowEnd"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetAppendWindowStart"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetGroupStartTimestamp"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetGroupStartTimestampToEndTimestamp"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetMaximumBufferSize"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetMaximumQueueDepthForTrackID"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetMediaSourceEnded"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetMode"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetShouldGenerateTimestamps"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_SetTimestampOffset"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_StartChangingType"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteSourceBufferProxy_UpdateTrackIds"_s, ReceiverName::RemoteSourceBufferProxy, false, false },
    { "RemoteTextDetector_Detect"_s, ReceiverName::RemoteTextDetector, true, false },
    { "RemoteTextureView_Destruct"_s, ReceiverName::RemoteTextureView, true, false },
    { "RemoteTextureView_SetLabel"_s, ReceiverName::RemoteTextureView, true, false },
    { "RemoteTexture_CreateView"_s, ReceiverName::RemoteTexture, true, false },
    { "RemoteTexture_Destroy"_s, ReceiverName::RemoteTexture, true, false },
    { "RemoteTexture_Destruct"_s, ReceiverName::RemoteTexture, true, false },
    { "RemoteTexture_SetLabel"_s, ReceiverName::RemoteTexture, true, false },
    { "RemoteVideoFrameObjectHeapProxyProcessor_NewConvertedVideoFrameBuffer"_s, ReceiverName::RemoteVideoFrameObjectHeapProxyProcessor, false, false },
    { "RemoteVideoFrameObjectHeapProxyProcessor_NewVideoFrameBuffer"_s, ReceiverName::RemoteVideoFrameObjectHeapProxyProcessor, false, false },
    { "RemoteVideoFrameObjectHeapProxyProcessor_SetSharedVideoFrameMemory"_s, ReceiverName::RemoteVideoFrameObjectHeapProxyProcessor, false, false },
    { "RemoteVideoFrameObjectHeapProxyProcessor_SetSharedVideoFrameSemaphore"_s, ReceiverName::RemoteVideoFrameObjectHeapProxyProcessor, false, false },
#if PLATFORM(COCOA)
    { "RemoteVideoFrameObjectHeap_GetVideoFrameBuffer"_s, ReceiverName::RemoteVideoFrameObjectHeap, true, false },
#endif
    { "RemoteVideoFrameObjectHeap_ReleaseVideoFrame"_s, ReceiverName::RemoteVideoFrameObjectHeap, true, false },
#if PLATFORM(COCOA)
    { "RemoteVideoFrameObjectHeap_SetSharedVideoFrameMemory"_s, ReceiverName::RemoteVideoFrameObjectHeap, false, false },
    { "RemoteVideoFrameObjectHeap_SetSharedVideoFrameSemaphore"_s, ReceiverName::RemoteVideoFrameObjectHeap, false, false },
#endif
    { "RemoteWebInspectorUIProxy_BringToFront"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_FrontendDidClose"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_FrontendLoaded"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_Load"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_OpenURLExternally"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_PickColorFromScreen"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_Reopen"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_ResetState"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_RevealFileExternally"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_Save"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_SendMessageToBackend"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_SetForcedAppearance"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_SetInspectorPageDeveloperExtrasEnabled"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_SetSheetRect"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_ShowCertificate"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUIProxy_StartWindowDrag"_s, ReceiverName::RemoteWebInspectorUIProxy, false, false },
    { "RemoteWebInspectorUI_Initialize"_s, ReceiverName::RemoteWebInspectorUI, false, false },
    { "RemoteWebInspectorUI_SendMessageToFrontend"_s, ReceiverName::RemoteWebInspectorUI, false, false },
#if ENABLE(INSPECTOR_TELEMETRY)
    { "RemoteWebInspectorUI_SetDiagnosticLoggingAvailable"_s, ReceiverName::RemoteWebInspectorUI, false, false },
#endif
    { "RemoteWebInspectorUI_ShowConsole"_s, ReceiverName::RemoteWebInspectorUI, false, false },
    { "RemoteWebInspectorUI_ShowResources"_s, ReceiverName::RemoteWebInspectorUI, false, false },
    { "RemoteWebInspectorUI_UpdateFindString"_s, ReceiverName::RemoteWebInspectorUI, false, false },
    { "RemoteWebLockRegistry_DidCompleteLockRequest"_s, ReceiverName::RemoteWebLockRegistry, false, false },
    { "RemoteWebLockRegistry_DidStealLock"_s, ReceiverName::RemoteWebLockRegistry, false, false },
    { "RemoteXRBinding_CreateProjectionLayer"_s, ReceiverName::RemoteXRBinding, true, false },
    { "RemoteXRBinding_Destruct"_s, ReceiverName::RemoteXRBinding, true, false },
    { "RemoteXRBinding_GetViewSubImage"_s, ReceiverName::RemoteXRBinding, true, false },
    { "RemoteXRProjectionLayer_Destruct"_s, ReceiverName::RemoteXRProjectionLayer, true, false },
    { "RemoteXRProjectionLayer_EndFrame"_s, ReceiverName::RemoteXRProjectionLayer, true, false },
    { "RemoteXRProjectionLayer_StartFrame"_s, ReceiverName::RemoteXRProjectionLayer, true, false },
    { "RemoteXRSubImage_Destruct"_s, ReceiverName::RemoteXRSubImage, true, false },
    { "RemoteXRView_Destruct"_s, ReceiverName::RemoteXRView, true, false },
    { "SampleBufferDisplayLayer_SetDidFail"_s, ReceiverName::SampleBufferDisplayLayer, false, false },
    { "ServiceWorkerDownloadTask_DidFail"_s, ReceiverName::ServiceWorkerDownloadTask, false, false },
    { "ServiceWorkerDownloadTask_DidFinish"_s, ReceiverName::ServiceWorkerDownloadTask, false, false },
    { "ServiceWorkerDownloadTask_DidReceiveData"_s, ReceiverName::ServiceWorkerDownloadTask, false, false },
    { "ServiceWorkerDownloadTask_DidReceiveFormData"_s, ReceiverName::ServiceWorkerDownloadTask, false, false },
    { "ServiceWorkerFetchTask_DidFail"_s, ReceiverName::ServiceWorkerFetchTask, false, false },
    { "ServiceWorkerFetchTask_DidFinish"_s, ReceiverName::ServiceWorkerFetchTask, false, false },
    { "ServiceWorkerFetchTask_DidNotHandle"_s, ReceiverName::ServiceWorkerFetchTask, false, false },
    { "ServiceWorkerFetchTask_DidReceiveData"_s, ReceiverName::ServiceWorkerFetchTask, false, false },
    { "ServiceWorkerFetchTask_DidReceiveFormData"_s, ReceiverName::ServiceWorkerFetchTask, false, false },
    { "ServiceWorkerFetchTask_DidReceiveRedirectResponse"_s, ReceiverName::ServiceWorkerFetchTask, false, false },
    { "ServiceWorkerFetchTask_DidReceiveResponse"_s, ReceiverName::ServiceWorkerFetchTask, false, false },
    { "ServiceWorkerFetchTask_UsePreload"_s, ReceiverName::ServiceWorkerFetchTask, false, false },
    { "ServiceWorkerFetchTask_WorkerClosed"_s, ReceiverName::ServiceWorkerFetchTask, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateBufferedChanged"_s, ReceiverName::SourceBufferPrivateRemoteMessageReceiver, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateDidDropSample"_s, ReceiverName::SourceBufferPrivateRemoteMessageReceiver, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateDidReceiveInitializationSegment"_s, ReceiverName::SourceBufferPrivateRemoteMessageReceiver, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateDidReceiveRenderingError"_s, ReceiverName::SourceBufferPrivateRemoteMessageReceiver, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateDurationChanged"_s, ReceiverName::SourceBufferPrivateRemoteMessageReceiver, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateEvictionDataChanged"_s, ReceiverName::SourceBufferPrivateRemoteMessageReceiver, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateHighestPresentationTimestampChanged"_s, ReceiverName::SourceBufferPrivateRemoteMessageReceiver, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateShuttingDown"_s, ReceiverName::SourceBufferPrivateRemoteMessageReceiver, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_TakeOwnershipOfMemory"_s, ReceiverName::SourceBufferPrivateRemoteMessageReceiver, false, false },
    { "SpeechRecognitionRealtimeMediaSourceManager_CreateSource"_s, ReceiverName::SpeechRecognitionRealtimeMediaSourceManager, false, false },
    { "SpeechRecognitionRealtimeMediaSourceManager_DeleteSource"_s, ReceiverName::SpeechRecognitionRealtimeMediaSourceManager, false, false },
    { "SpeechRecognitionRealtimeMediaSourceManager_Start"_s, ReceiverName::SpeechRecognitionRealtimeMediaSourceManager, false, false },
    { "SpeechRecognitionRealtimeMediaSourceManager_Stop"_s, ReceiverName::SpeechRecognitionRealtimeMediaSourceManager, false, false },
    { "SpeechRecognitionRemoteRealtimeMediaSourceManager_RemoteAudioSamplesAvailable"_s, ReceiverName::SpeechRecognitionRemoteRealtimeMediaSourceManager, false, false },
    { "SpeechRecognitionRemoteRealtimeMediaSourceManager_RemoteCaptureFailed"_s, ReceiverName::SpeechRecognitionRemoteRealtimeMediaSourceManager, false, false },
    { "SpeechRecognitionRemoteRealtimeMediaSourceManager_RemoteSourceStopped"_s, ReceiverName::SpeechRecognitionRemoteRealtimeMediaSourceManager, false, false },
#if PLATFORM(COCOA)
    { "SpeechRecognitionRemoteRealtimeMediaSourceManager_SetStorage"_s, ReceiverName::SpeechRecognitionRemoteRealtimeMediaSourceManager, false, false },
#endif
    { "SpeechRecognitionServer_Abort"_s, ReceiverName::SpeechRecognitionServer, false, false },
    { "SpeechRecognitionServer_Invalidate"_s, ReceiverName::SpeechRecognitionServer, false, false },
    { "SpeechRecognitionServer_Start"_s, ReceiverName::SpeechRecognitionServer, false, false },
    { "SpeechRecognitionServer_Stop"_s, ReceiverName::SpeechRecognitionServer, false, false },
    { "StorageAreaMap_ClearCache"_s, ReceiverName::StorageAreaMap, false, false },
    { "StorageAreaMap_DispatchStorageEvent"_s, ReceiverName::StorageAreaMap, false, false },
    { "UserMediaCaptureManager_GetMediaStreamDevices"_s, ReceiverName::UserMediaCaptureManager, false, false },
    { "UserMediaCaptureManager_ValidateUserMediaRequestConstraints"_s, ReceiverName::UserMediaCaptureManager, false, false },
#if !PLATFORM(IOS_FAMILY)
    { "ViewGestureController_DidCollectGeometryForMagnificationGesture"_s, ReceiverName::ViewGestureController, false, false },
#endif
#if PLATFORM(MAC)
    { "ViewGestureController_DidCollectGeometryForSmartMagnificationGesture"_s, ReceiverName::ViewGestureController, false, false },
#endif
#if !PLATFORM(IOS_FAMILY)
    { "ViewGestureController_DidHitRenderTreeSizeThreshold"_s, ReceiverName::ViewGestureController, false, false },
    { "ViewGestureGeometryCollector_CollectGeometryForMagnificationGesture"_s, ReceiverName::ViewGestureGeometryCollector, false, false },
#endif
#if PLATFORM(COCOA)
    { "ViewGestureGeometryCollector_CollectGeometryForSmartMagnificationGesture"_s, ReceiverName::ViewGestureGeometryCollector, false, false },
#endif
#if !PLATFORM(IOS_FAMILY)
    { "ViewGestureGeometryCollector_SetRenderTreeSizeNotificationThreshold"_s, ReceiverName::ViewGestureGeometryCollector, false, false },
#endif
    { "VisitedLinkStore_AddVisitedLinkHashFromPage"_s, ReceiverName::VisitedLinkStore, false, false },
    { "VisitedLinkTableController_AllVisitedLinkStateChanged"_s, ReceiverName::VisitedLinkTableController, false, false },
    { "VisitedLinkTableController_RemoveAllVisitedLinks"_s, ReceiverName::VisitedLinkTableController, false, false },
    { "VisitedLinkTableController_SetVisitedLinkTable"_s, ReceiverName::VisitedLinkTableController, false, false },
    { "VisitedLinkTableController_VisitedLinkStateChanged"_s, ReceiverName::VisitedLinkTableController, false, false },
    { "WebAutomationSessionProxy_ComputeElementLayout"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_DeleteCookie"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_EvaluateJavaScriptFunction"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_GetComputedLabel"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_GetComputedRole"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_GetCookiesForFrame"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_ResolveChildFrameWithName"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_ResolveChildFrameWithNodeHandle"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_ResolveChildFrameWithOrdinal"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_ResolveParentFrame"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_SelectOptionElement"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_SetFilesForInputFileUpload"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_SnapshotRectForScreenshot"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSessionProxy_TakeScreenshot"_s, ReceiverName::WebAutomationSessionProxy, false, false },
    { "WebAutomationSession_DidEvaluateJavaScriptFunction"_s, ReceiverName::WebAutomationSession, false, false },
    { "WebAutomationSession_DidTakeScreenshot"_s, ReceiverName::WebAutomationSession, false, false },
    { "WebBroadcastChannelRegistry_PostMessageToRemote"_s, ReceiverName::WebBroadcastChannelRegistry, false, false },
    { "WebCookieManager_DeleteAllCookies"_s, ReceiverName::WebCookieManager, false, false },
    { "WebCookieManager_DeleteAllCookiesModifiedSince"_s, ReceiverName::WebCookieManager, false, false },
    { "WebCookieManager_DeleteCookie"_s, ReceiverName::WebCookieManager, false, false },
    { "WebCookieManager_DeleteCookiesForHostnames"_s, ReceiverName::WebCookieManager, false, false },
    { "WebCookieManager_GetAllCookies"_s, ReceiverName::WebCookieManager, false, false },
    { "WebCookieManager_GetCookies"_s, ReceiverName::WebCookieManager, false, false },
    { "WebCookieManager_GetHTTPCookieAcceptPolicy"_s, ReceiverName::WebCookieManager, false, false },
    { "WebCookieManager_GetHostnamesWithCookies"_s, ReceiverName::WebCookieManager, false, false },
#if USE(SOUP)
    { "WebCookieManager_ReplaceCookies"_s, ReceiverName::WebCookieManager, false, false },
#endif
    { "WebCookieManager_SetCookie"_s, ReceiverName::WebCookieManager, false, false },
#if USE(SOUP)
    { "WebCookieManager_SetCookiePersistentStorage"_s, ReceiverName::WebCookieManager, false, false },
#endif
    { "WebCookieManager_SetCookies"_s, ReceiverName::WebCookieManager, false, false },
    { "WebCookieManager_SetHTTPCookieAcceptPolicy"_s, ReceiverName::WebCookieManager, false, false },
    { "WebCookieManager_StartObservingCookieChanges"_s, ReceiverName::WebCookieManager, false, false },
    { "WebCookieManager_StopObservingCookieChanges"_s, ReceiverName::WebCookieManager, false, false },
    { "WebFileSystemStorageConnection_InvalidateAccessHandle"_s, ReceiverName::WebFileSystemStorageConnection, false, false },
    { "WebFullScreenManagerProxy_BeganEnterFullScreen"_s, ReceiverName::WebFullScreenManagerProxy, false, false },
    { "WebFullScreenManagerProxy_BeganExitFullScreen"_s, ReceiverName::WebFullScreenManagerProxy, false, false },
    { "WebFullScreenManagerProxy_Close"_s, ReceiverName::WebFullScreenManagerProxy, false, false },
    { "WebFullScreenManagerProxy_EnterFullScreen"_s, ReceiverName::WebFullScreenManagerProxy, false, false },
    { "WebFullScreenManagerProxy_ExitFullScreen"_s, ReceiverName::WebFullScreenManagerProxy, false, false },
    { "WebFullScreenManager_DidEnterFullScreen"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebFullScreenManager_DidExitFullScreen"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebFullScreenManager_RequestExitFullScreen"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebFullScreenManager_RequestRestoreFullScreen"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebFullScreenManager_RestoreScrollPosition"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebFullScreenManager_SaveScrollPosition"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebFullScreenManager_SetAnimatingFullScreen"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebFullScreenManager_SetFullscreenAutoHideDuration"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebFullScreenManager_SetFullscreenInsets"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebFullScreenManager_WillEnterFullScreen"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebFullScreenManager_WillExitFullScreen"_s, ReceiverName::WebFullScreenManager, false, false },
    { "WebGeolocationManagerProxy_SetEnableHighAccuracy"_s, ReceiverName::WebGeolocationManagerProxy, false, false },
    { "WebGeolocationManagerProxy_StartUpdating"_s, ReceiverName::WebGeolocationManagerProxy, false, false },
    { "WebGeolocationManagerProxy_StopUpdating"_s, ReceiverName::WebGeolocationManagerProxy, false, false },
    { "WebGeolocationManager_DidChangePosition"_s, ReceiverName::WebGeolocationManager, false, false },
    { "WebGeolocationManager_DidFailToDeterminePosition"_s, ReceiverName::WebGeolocationManager, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebGeolocationManager_ResetPermissions"_s, ReceiverName::WebGeolocationManager, false, false },
#endif
    { "WebIDBConnectionToServer_DidAbortTransaction"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidClearObjectStore"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidCloseFromServer"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidCommitTransaction"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidCreateIndex"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidCreateObjectStore"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidDeleteDatabase"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidDeleteIndex"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidDeleteObjectStore"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidDeleteRecord"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidGetAllDatabaseNamesAndVersions"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidGetAllRecords"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidGetCount"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidGetRecord"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidIterateCursor"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidOpenCursor"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidOpenDatabase"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidPutOrAdd"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidRenameIndex"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidRenameObjectStore"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_DidStartTransaction"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_FireVersionChangeEvent"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebIDBConnectionToServer_NotifyOpenDBRequestBlocked"_s, ReceiverName::WebIDBConnectionToServer, false, false },
    { "WebInspectorInterruptDispatcher_NotifyNeedDebuggerBreak"_s, ReceiverName::WebInspectorInterruptDispatcher, false, false },
    { "WebInspectorUIProxy_AttachAvailabilityChanged"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_AttachBottom"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_AttachLeft"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_AttachRight"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_BringToFront"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_Detach"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_DidClose"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_EffectiveAppearanceDidChange"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_ElementSelectionChanged"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_FrontendLoaded"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_InspectedURLChanged"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_Load"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_OpenLocalInspectorFrontend"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_OpenURLExternally"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_PickColorFromScreen"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_Reopen"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_ResetState"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_RevealFileExternally"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_Save"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_SendMessageToBackend"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_SetAttachedWindowHeight"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_SetAttachedWindowWidth"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_SetDeveloperPreferenceOverride"_s, ReceiverName::WebInspectorUIProxy, false, false },
#if ENABLE(INSPECTOR_NETWORK_THROTTLING)
    { "WebInspectorUIProxy_SetEmulatedConditions"_s, ReceiverName::WebInspectorUIProxy, false, false },
#endif
    { "WebInspectorUIProxy_SetForcedAppearance"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_SetFrontendConnection"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_SetInspectorPageDeveloperExtrasEnabled"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_SetSheetRect"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_ShowCertificate"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_StartWindowDrag"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUIProxy_TimelineRecordingChanged"_s, ReceiverName::WebInspectorUIProxy, false, false },
    { "WebInspectorUI_AttachedBottom"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_AttachedLeft"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_AttachedRight"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_Detached"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_EstablishConnection"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_EvaluateInFrontendForTesting"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_SendMessageToFrontend"_s, ReceiverName::WebInspectorUI, false, false },
#if ENABLE(INSPECTOR_TELEMETRY)
    { "WebInspectorUI_SetDiagnosticLoggingAvailable"_s, ReceiverName::WebInspectorUI, false, false },
#endif
    { "WebInspectorUI_SetDockingUnavailable"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_SetIsVisible"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_ShowConsole"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_ShowMainResourceForFrame"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_ShowResources"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_StartElementSelection"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_StartPageProfiling"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_StopElementSelection"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_StopPageProfiling"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_UpdateConnection"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspectorUI_UpdateFindString"_s, ReceiverName::WebInspectorUI, false, false },
    { "WebInspector_Close"_s, ReceiverName::WebInspector, false, false },
    { "WebInspector_SetAttached"_s, ReceiverName::WebInspector, false, false },
    { "WebInspector_SetFrontendConnection"_s, ReceiverName::WebInspector, false, false },
    { "WebInspector_Show"_s, ReceiverName::WebInspector, false, false },
    { "WebInspector_ShowConsole"_s, ReceiverName::WebInspector, false, false },
    { "WebInspector_ShowMainResourceForFrame"_s, ReceiverName::WebInspector, false, false },
    { "WebInspector_ShowResources"_s, ReceiverName::WebInspector, false, false },
    { "WebInspector_StartElementSelection"_s, ReceiverName::WebInspector, false, false },
    { "WebInspector_StartPageProfiling"_s, ReceiverName::WebInspector, false, false },
    { "WebInspector_StopElementSelection"_s, ReceiverName::WebInspector, false, false },
    { "WebInspector_StopPageProfiling"_s, ReceiverName::WebInspector, false, false },
    { "WebLockRegistryProxy_AbortLockRequest"_s, ReceiverName::WebLockRegistryProxy, false, false },
    { "WebLockRegistryProxy_ClientIsGoingAway"_s, ReceiverName::WebLockRegistryProxy, false, false },
    { "WebLockRegistryProxy_ReleaseLock"_s, ReceiverName::WebLockRegistryProxy, false, false },
    { "WebLockRegistryProxy_RequestLock"_s, ReceiverName::WebLockRegistryProxy, false, false },
    { "WebLockRegistryProxy_Snapshot"_s, ReceiverName::WebLockRegistryProxy, false, false },
    { "WebNotificationManager_DidClickNotification"_s, ReceiverName::WebNotificationManager, false, false },
    { "WebNotificationManager_DidCloseNotifications"_s, ReceiverName::WebNotificationManager, false, false },
    { "WebNotificationManager_DidRemoveNotificationDecisions"_s, ReceiverName::WebNotificationManager, false, false },
    { "WebNotificationManager_DidShowNotification"_s, ReceiverName::WebNotificationManager, false, false },
    { "WebNotificationManager_DidUpdateNotificationDecision"_s, ReceiverName::WebNotificationManager, false, false },
#if ENABLE(APPLE_PAY_AMS_UI)
    { "WebPageProxy_AbortApplePayAMSUISession"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(MEDIA_USAGE)
    { "WebPageProxy_AddMediaUsageManagerSession"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
    { "WebPageProxy_AddPlaybackTargetPickerClient"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(WRITING_TOOLS_UI)
    { "WebPageProxy_AddTextAnimationForAnimationID"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPageProxy_AssistiveTechnologyMakeFirstResponder"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_BackForwardAddItem"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_BackForwardClear"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(MEDIA_STREAM)
    { "WebPageProxy_BeginMonitoringCaptureDevices"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if USE(SYSTEM_PREVIEW)
    { "WebPageProxy_BeginSystemPreview"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(IMAGE_ANALYSIS) && ENABLE(VIDEO)
    { "WebPageProxy_BeginTextRecognitionForVideoInElementFullScreen"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPageProxy_BindAccessibilityTree"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_BroadcastMainFrameURLChangeToOtherProcesses"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(IMAGE_ANALYSIS) && ENABLE(VIDEO)
    { "WebPageProxy_CancelTextRecognitionForVideoInElementFullScreen"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPageProxy_ChangeUniversalAccessZoomFocus"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_ClearAllEditCommands"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ClearTextIndicator"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(ATTACHMENT_ELEMENT)
    { "WebPageProxy_CloneAttachmentData"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_ClosePage"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_CommitPotentialTapFailed"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_CompositionWasCanceled"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ConfigureLoggingChannel"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(CONTENT_FILTERING)
    { "WebPageProxy_ContentFilterDidBlockLoadForFrame"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(CONTENT_EXTENSIONS)
    { "WebPageProxy_ContentRuleListNotification"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_CouldNotRestorePageState"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_CreateInspectorTarget"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(PDF_HUD)
    { "WebPageProxy_CreatePDFHUD"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DecidePolicyForNavigationActionAsync"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DecidePolicyForNewWindowAction"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DecidePolicyForResponse"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DestroyInspectorTarget"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(WINDOW_PROXY_PROPERTY_ACCESS_NOTIFICATION)
    { "WebPageProxy_DidAccessWindowProxyPropertyViaOpenerForFrame"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidAdjustVisibilityWithSelectors"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidApplyLinkDecorationFiltering"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidCancelClientRedirectForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidChangeContentSize"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidChangeInspectorFrontendCount"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidChangeIntrinsicContentSize"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidChangeMainDocument"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidChangePageCount"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidChangeProgress"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidChangeProvisionalURLForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidChangeScrollOffsetPinningForMainFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidChangeScrollbarsForMainFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidChangeViewportProperties"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidCommitLoadForFrame"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_DidCompleteSyntheticClick"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if HAVE(VISIBILITY_PROPAGATION_VIEW)
    { "WebPageProxy_DidCreateContextInWebProcessForVisibilityPropagation"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidCreateSleepDisabler"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidCreateSubframe"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidDestroyNavigation"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidDestroySleepDisabler"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidDisplayInsecureContentForFrame"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(WRITING_TOOLS_UI)
    { "WebPageProxy_DidEndPartialIntelligenceTextPonderingAnimation"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidExplicitOpenForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidFailLoadForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidFailProvisionalLoadForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidFailToSuspendAfterProcessSwap"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidFindTextManipulationItems"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidFinishDocumentLoadForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidFinishLoadForFrame"_s, ReceiverName::WebPageProxy, false, false },
#if USE(QUICK_LOOK)
    { "WebPageProxy_DidFinishLoadForQuickLookDocumentInMainFrame"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPageProxy_DidFinishLoadForResource"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidFinishLoadingDataForCustomContentProvider"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidFinishProgress"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidFinishServiceWorkerPageRegistration"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidFirstLayoutForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidFirstVisuallyNonEmptyLayoutForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidGetImageForFindMatch"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_DidGetTapHighlightGeometries"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPageProxy_DidHandleAcceptedCandidate"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
    { "WebPageProxy_DidHandleAdditionalDragItemsRequest"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidHandleDragStartRequest"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_DidHandleTapAsHover"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPageProxy_DidInitiateLoadForResource"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ATTACHMENT_ELEMENT)
    { "WebPageProxy_DidInsertAttachmentWithIdentifier"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidNavigateWithNavigationData"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_DidNotHandleTapAsClick"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidPerformClientRedirect"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(COCOA)
    { "WebPageProxy_DidPerformDictionaryLookup"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPageProxy_DidPerformImmediateActionHitTest"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidPerformServerRedirect"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidReachLayoutMilestone"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
    { "WebPageProxy_DidReceiveEditDragSnapshot"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidReceiveEvent"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_DidReceivePositionInformation"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPageProxy_DidReceiveResponseForResource"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidReceiveServerRedirectForProvisionalLoadForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidReceiveTitleForFrame"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(ATTACHMENT_ELEMENT)
    { "WebPageProxy_DidRemoveAttachmentWithIdentifier"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidResignInputElementStrongPasswordAppearance"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidRestoreScrollPosition"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidRunInsecureContentForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidSameDocumentNavigationForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidSameDocumentNavigationForFrameViaJSHistoryAPI"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPageProxy_DidSendRequestForResource"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if USE(QUICK_LOOK)
    { "WebPageProxy_DidStartLoadForQuickLookDocumentInMainFrame"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DidStartProgress"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidStartProvisionalLoadForFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidSuspendAfterProcessSwap"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidUpdateActivityState"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidUpdateHistoryTitle"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DidUpdateRenderingAfterCommittingLoad"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_DisableDoubleTapGesturesDuringTapIfNecessary"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DisableInspectorNodeSearch"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPageProxy_DismissCorrectionPanel"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_DispatchLoadEventToFrameOwnerElement"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_DocumentURLForConsoleLog"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_EditorStateChanged"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_ElementDidBlur"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ElementDidFocus"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_EnableInspectorNodeSearch"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(INPUT_TYPE_COLOR)
    { "WebPageProxy_EndColorPicker"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(DATALIST_ELEMENT)
    { "WebPageProxy_EndDataListSuggestions"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(DATE_AND_TIME_INPUT_TYPES)
    { "WebPageProxy_EndDateTimePicker"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(MEDIA_STREAM)
    { "WebPageProxy_EnumerateMediaDevicesForFrame"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_FixedLayoutSizeDidChange"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_FocusFromServiceWorker"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_FocusRemoteFrame"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_FocusedElementDidChangeInputMode"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_FocusedFrameChanged"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_FrameNameChanged"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(GAMEPAD)
    { "WebPageProxy_GamepadsRecentlyAccessed"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_GetLoadDecisionForIcon"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(MAC)
    { "WebPageProxy_HandleAcceptsFirstMouse"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_HandleAutoFillButtonClick"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_HandleAutocorrectionContext"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_HandleAutoplayEvent"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(DATA_DETECTION)
    { "WebPageProxy_HandleClickForDataDetectionResult"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if HAVE(TRANSLATION_UI_SERVICES) && ENABLE(CONTEXT_MENUS)
    { "WebPageProxy_HandleContextMenuTranslation"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(DATALIST_ELEMENT)
    { "WebPageProxy_HandleKeydownInDataList"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_HandleMessage"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_HandleMessageWithAsyncReply"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
    { "WebPageProxy_HandleMouseDownForModelElement"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_HandleMouseMoveForModelElement"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_HandleMouseUpForModelElement"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_HandleSmartMagnificationInformationForPotentialTap"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_HasActiveNowPlayingSessionChanged"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_HideInspectorHighlight"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_HideInspectorIndication"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_HidePopupMenu"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(COCOA) || PLATFORM(GTK)
    { "WebPageProxy_HideValidationMessage"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_IgnoreWord"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ImageOrMediaDocumentSizeChanged"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
    { "WebPageProxy_IsAnyAnimationAllowedToPlayDidChange"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_IsPlayingMediaDidChange"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_LearnWord"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_LogDiagnosticMessageFromWebProcess"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_LogDiagnosticMessageWithDomainFromWebProcess"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_LogDiagnosticMessageWithEnhancedPrivacyFromWebProcess"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_LogDiagnosticMessageWithResultFromWebProcess"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_LogDiagnosticMessageWithValueDictionaryFromWebProcess"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_LogDiagnosticMessageWithValueFromWebProcess"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_LogScrollingEvent"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_MainFramePluginHandlesPageScaleGestureDidChange"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(COCOA)
    { "WebPageProxy_MakeFirstResponder"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
    { "WebPageProxy_MockMediaPlaybackTargetPickerDismissPopup"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW)
    { "WebPageProxy_ModelElementAnimationCurrentTime"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ModelElementAnimationDuration"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
    { "WebPageProxy_ModelElementCreateRemotePreview"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ModelElementDestroyRemotePreview"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW)
    { "WebPageProxy_ModelElementGetCamera"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ModelElementHasAudio"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ModelElementIsLoopingAnimation"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ModelElementIsMuted"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ModelElementIsPlayingAnimation"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
    { "WebPageProxy_ModelElementLoadRemotePreview"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW)
    { "WebPageProxy_ModelElementSetAnimationCurrentTime"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ModelElementSetAnimationIsPlaying"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ModelElementSetCamera"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW_IOS)
    { "WebPageProxy_ModelElementSetInteractionEnabled"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW)
    { "WebPageProxy_ModelElementSetIsLoopingAnimation"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ModelElementSetIsMuted"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
    { "WebPageProxy_ModelElementSizeDidChange"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ModelInlinePreviewUUIDs"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_MouseDidMoveOverElement"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_NotifyScrollerThumbIsVisibleInRect"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_NowPlayingMetadataChanged"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_PageDidScroll"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_PageExtendedBackgroundColorDidChange"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_PageScaleFactorDidChange"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_PerformSwitchHapticFeedback"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
    { "WebPageProxy_PlaybackTargetPickerClientStateDidChange"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_PluginScaleFactorDidChange"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_PluginZoomFactorDidChange"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_PostMessageToRemote"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(WRITING_TOOLS)
    { "WebPageProxy_ProofreadingSessionShowDetailsForSuggestionWithIDRelativeToRect"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ProofreadingSessionUpdateStateForSuggestionWithID"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_RecommendedScrollbarStyleDidChange"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(MAC)
    { "WebPageProxy_RecordAutocorrectionResponse"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ATTACHMENT_ELEMENT)
    { "WebPageProxy_RegisterAttachmentIdentifier"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_RegisterAttachmentIdentifierFromData"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_RegisterAttachmentIdentifierFromFilePath"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_RegisterAttachmentsFromSerializedData"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_RegisterEditCommandForUndo"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_RegisterInsertionUndoGrouping"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(COCOA)
    { "WebPageProxy_RegisterWebProcessAccessibilityToken"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_RelayAccessibilityNotification"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if USE(DICTATION_ALTERNATIVES)
    { "WebPageProxy_RemoveDictationAlternatives"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(MEDIA_USAGE)
    { "WebPageProxy_RemoveMediaUsageManagerSession"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(PDF_HUD)
    { "WebPageProxy_RemovePDFHUD"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
    { "WebPageProxy_RemovePlaybackTargetPickerClient"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(WRITING_TOOLS_UI)
    { "WebPageProxy_RemoveTextAnimationForAnimationID"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(ATTACHMENT_ELEMENT)
    { "WebPageProxy_RequestAttachmentIcon"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_RequestCheckingOfString"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_RequestCookieConsent"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_RequestGeolocationPermissionForFrame"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(ENCRYPTED_MEDIA)
    { "WebPageProxy_RequestMediaKeySystemPermissionForFrame"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_RequestNotificationPermission"_s, ReceiverName::WebPageProxy, false, false },
#if USE(QUICK_LOOK)
    { "WebPageProxy_RequestPasswordForQuickLookDocumentInMainFrame"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(POINTER_LOCK)
    { "WebPageProxy_RequestPointerLock"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_RequestPointerUnlock"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_RequestScrollToRect"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(IMAGE_ANALYSIS)
    { "WebPageProxy_RequestTextRecognition"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(MEDIA_STREAM)
    { "WebPageProxy_RequestUserMediaPermissionForFrame"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_RestorePageCenterAndScale"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_RestorePageState"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_RevokeGeolocationAuthorizationToken"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_RunModal"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_RunOpenPanel"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SampledPageTopColorChanged"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_SaveImageToLibrary"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_SaveRecentSearches"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_ScrollingNodeScrollDidEndScroll"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ScrollingNodeScrollWillStartScroll"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPageProxy_SearchTheWeb"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_SendMessageToInspectorFrontend"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPageProxy_SendMessageToWebView"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SendMessageToWebViewWithReply"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_SetAllowsLayoutViewportHeightExpansion"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetCanShortCircuitHorizontalWheelEvents"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(INPUT_TYPE_COLOR)
    { "WebPageProxy_SetColorPickerColor"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_SetCursor"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetCursorHiddenUntilMouseMoves"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(DATA_DETECTION)
    { "WebPageProxy_SetDataDetectionResult"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPageProxy_SetEditableElementIsFocused"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_SetFocus"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetHasActiveAnimatedScrolls"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetHasExecutedAppBoundBehaviorBeforeNavigation"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetHasHadSelectionChangesFromUserInteraction"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPageProxy_SetInputMethodState"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if HAVE(TOUCH_BAR)
    { "WebPageProxy_SetIsNeverRichlyEditableForTouchBar"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_SetIsResizable"_s, ReceiverName::WebPageProxy, false, false },
#if HAVE(TOUCH_BAR)
    { "WebPageProxy_SetIsTouchBarUpdateSuppressedForHiddenContentEditable"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_SetMenuBarIsVisible"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
    { "WebPageProxy_SetMockMediaPlaybackTargetPickerEnabled"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetMockMediaPlaybackTargetPickerState"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(VIDEO_PRESENTATION_MODE)
    { "WebPageProxy_SetMockVideoPresentationModeEnabled"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(WEB_AUTHN)
    { "WebPageProxy_SetMockWebAuthenticationConfiguration"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_SetNetworkRequestsInProgress"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(COCOA) && ENABLE(DRAG_SUPPORT)
    { "WebPageProxy_SetPromisedDataForImage"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_SetRenderTreeSize"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetStatusBarIsVisible"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetStatusText"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetTextIndicatorFromFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetToolbarsAreVisible"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SetWindowFrame"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(DEVICE_ORIENTATION)
    { "WebPageProxy_ShouldAllowDeviceOrientationAndMotionAccess"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(INPUT_TYPE_COLOR)
    { "WebPageProxy_ShowColorPicker"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_ShowContactPicker"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(CONTEXT_MENUS)
    { "WebPageProxy_ShowContextMenu"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ShowContextMenuFromFrame"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPageProxy_ShowCorrectionPanel"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_ShowDataDetectorsUIForPositionInformation"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(DATALIST_ELEMENT)
    { "WebPageProxy_ShowDataListSuggestions"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(DATE_AND_TIME_INPUT_TYPES)
    { "WebPageProxy_ShowDateTimePicker"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if USE(DICTATION_ALTERNATIVES)
    { "WebPageProxy_ShowDictationAlternativeUI"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(GTK)
    { "WebPageProxy_ShowEmojiPicker"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_ShowInspectorHighlight"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ShowInspectorIndication"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(MEDIA_CONTROLS_CONTEXT_MENUS) && USE(UICONTEXTMENU)
    { "WebPageProxy_ShowMediaControlsContextMenu"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(PDF_PLUGIN) && PLATFORM(MAC)
    { "WebPageProxy_ShowPDFContextMenu"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_ShowPage"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY) || ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
    { "WebPageProxy_ShowPlaybackTargetPicker"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_ShowPopupMenuFromFrame"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_ShowShareSheet"_s, ReceiverName::WebPageProxy, false, false },
#if (ENABLE(TELEPHONE_NUMBER_DETECTION) && PLATFORM(MAC))
    { "WebPageProxy_ShowTelephoneNumberMenu"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(COCOA) || PLATFORM(GTK)
    { "WebPageProxy_ShowValidationMessage"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(SPEECH_SYNTHESIS)
    { "WebPageProxy_SpeechSynthesisCancel"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SpeechSynthesisPause"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SpeechSynthesisResetState"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SpeechSynthesisResume"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SpeechSynthesisSetFinishedCallback"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_SpeechSynthesisSpeak"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(APPLE_PAY_AMS_UI)
    { "WebPageProxy_StartApplePayAMSUISession"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if PLATFORM(COCOA) && ENABLE(DRAG_SUPPORT) || PLATFORM(GTK) && ENABLE(DRAG_SUPPORT)
    { "WebPageProxy_StartDrag"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_StartURLSchemeTask"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_StopURLSchemeTask"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_TakeFocus"_s, ReceiverName::WebPageProxy, false, false },
#if ENABLE(ARKIT_INLINE_PREVIEW_IOS)
    { "WebPageProxy_TakeModelElementFullscreen"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_ThemeColorChanged"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_UpdateFocusedElementInformation"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_UpdateInputContextAfterBlurringAndRefocusingElement"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(MEDIA_USAGE)
    { "WebPageProxy_UpdateMediaUsageManagerSessionState"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(PDF_HUD)
    { "WebPageProxy_UpdatePDFHUDLocation"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_UpdateRemoteFrameAccessibilityOffset"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_UpdateRemoteFrameSize"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_UpdateSpellingUIWithGrammarString"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_UpdateSpellingUIWithMisspelledWord"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_UpdateStringForFind"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_UseFixedLayoutDidChange"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_WillGoToBackForwardListItem"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_WillPerformClientRedirectForFrame"_s, ReceiverName::WebPageProxy, false, false },
#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
    { "WebPageProxy_WillReceiveEditDragSnapshot"_s, ReceiverName::WebPageProxy, false, false },
#endif
    { "WebPageProxy_WillSubmitForm"_s, ReceiverName::WebPageProxy, false, false },
#if (ENABLE(ATTACHMENT_ELEMENT) && PLATFORM(IOS_FAMILY))
    { "WebPageProxy_WritePromisedAttachmentToPasteboard"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if USE(AUTOMATIC_TEXT_REPLACEMENT)
    { "WebPageProxy_toggleAutomaticDashSubstitution"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_toggleAutomaticLinkDetection"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_toggleAutomaticQuoteSubstitution"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_toggleAutomaticTextReplacement"_s, ReceiverName::WebPageProxy, false, false },
    { "WebPageProxy_toggleSmartInsertDelete"_s, ReceiverName::WebPageProxy, false, false },
#endif
#if ENABLE(NOTIFICATIONS)
    { "WebPageTesting_ClearNotificationPermissionState"_s, ReceiverName::WebPageTesting, false, false },
#endif
    { "WebPageTesting_ClearWheelEventTestMonitor"_s, ReceiverName::WebPageTesting, false, false },
    { "WebPageTesting_IsLayerTreeFrozen"_s, ReceiverName::WebPageTesting, false, false },
    { "WebPageTesting_SetDefersLoading"_s, ReceiverName::WebPageTesting, false, false },
    { "WebPageTesting_SetPageScaleFactor"_s, ReceiverName::WebPageTesting, false, false },
    { "WebPageTesting_SetPermissionLevel"_s, ReceiverName::WebPageTesting, false, false },
    { "WebPageTesting_SetTopContentInset"_s, ReceiverName::WebPageTesting, false, false },
    { "WebPage_AccessibilitySettingsDidChange"_s, ReceiverName::WebPage, false, false },
    { "WebPage_AddConsoleMessage"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_AddDictationAlternative"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_AddLayerForFindOverlay"_s, ReceiverName::WebPage, false, false },
    { "WebPage_AdjustVisibilityForTargetedElements"_s, ReceiverName::WebPage, false, false },
    { "WebPage_AdvanceToNextMisspelling"_s, ReceiverName::WebPage, false, false },
#if (ENABLE(GAMEPAD) && PLATFORM(VISION))
    { "WebPage_AllowGamepadAccess"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ApplicationDidBecomeActive"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ApplicationDidEnterBackground"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ApplicationDidEnterBackgroundForMedia"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ApplicationDidFinishSnapshottingAfterEnteringBackground"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ApplicationWillEnterForeground"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ApplicationWillEnterForegroundForMedia"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ApplicationWillResignActive"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ApplyAutocorrection"_s, ReceiverName::WebPage, false, false },
    { "WebPage_AttemptSyntheticClick"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPage_AttributedSubstringForCharacterRangeAsync"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_AutofillLoginCredentials"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_BeginPrinting"_s, ReceiverName::WebPage, false, false },
    { "WebPage_BeginPrintingDuringDOMPrintOperation"_s, ReceiverName::WebPage, false, true },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_BeginSelectionInDirection"_s, ReceiverName::WebPage, false, false },
    { "WebPage_BlurFocusedElement"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(SPEECH_SYNTHESIS)
    { "WebPage_BoundaryEventOccurred"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_CancelAutoscroll"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPage_CancelComposition"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_CancelPointer"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_CancelPotentialTap"_s, ReceiverName::WebPage, false, false },
#endif
#if USE(APPKIT)
    { "WebPage_CapitalizeWord"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(MEDIA_STREAM)
    { "WebPage_CaptureDevicesChanged"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_CenterSelectionInVisibleArea"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ChangeFont"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ChangeFontAttributes"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ChangeListType"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ChangeSpellingToWord"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_CharacterIndexForPointAsync"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_ClearAllDecoratedFoundText"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_ClearDictationAlternatives"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_ClearLoadedSubresourceDomains"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ClearSelection"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ClearServiceWorkerEntitlementOverride"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_Close"_s, ReceiverName::WebPage, false, false },
    { "WebPage_CloseCurrentTypingCommand"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(GTK)
    { "WebPage_CollapseSelectionInFrame"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_CommitPotentialTap"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_CompleteTextManipulation"_s, ReceiverName::WebPage, false, false },
#if ENABLE(WRITING_TOOLS)
    { "WebPage_CompositionSessionDidReceiveTextWithReplacementRange"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_ComputePagesForPrinting"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ComputePagesForPrintingDuringDOMPrintOperation"_s, ReceiverName::WebPage, false, true },
#if PLATFORM(COCOA)
    { "WebPage_ConfirmCompositionAsync"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_ConnectInspector"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ContentSizeCategoryDidChange"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(CONTEXT_MENUS)
    { "WebPage_ContextMenuForKeyEvent"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_CopyLinkToHighlight"_s, ReceiverName::WebPage, false, false },
    { "WebPage_CountStringMatches"_s, ReceiverName::WebPage, false, false },
#if ENABLE(APP_HIGHLIGHTS)
    { "WebPage_CreateAppHighlightInSelectedRange"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(MEDIA_SESSION_COORDINATOR)
    { "WebPage_CreateMediaSessionCoordinator"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_CreateProvisionalFrame"_s, ReceiverName::WebPage, false, false },
    { "WebPage_CreateRemoteSubframe"_s, ReceiverName::WebPage, false, false },
#if ENABLE(WRITING_TOOLS_UI)
    { "WebPage_CreateTextIndicatorForTextAnimationID"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPage_DataDetectorsDidChangeUI"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DataDetectorsDidHideUI"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DataDetectorsDidPresentUI"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_DecorateTextRangeWithStyle"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DecreaseListLevel"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPage_DeleteSurrounding"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_DestroyProvisionalFrame"_s, ReceiverName::WebPage, false, false },
#if ENABLE(DATA_DETECTION)
    { "WebPage_DetectDataInAllFrames"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_DictationAlternativesAtSelection"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(POINTER_LOCK)
    { "WebPage_DidAcquirePointerLock"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPage_DidBeginMagnificationGesture"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_DidBeginTextSearchOperation"_s, ReceiverName::WebPage, false, false },
#if ENABLE(WRITING_TOOLS)
    { "WebPage_DidBeginWritingToolsSession"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_DidCancelCheckingText"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DidCancelForOpenPanel"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DidChangeSelectedIndexForActivePopupMenu"_s, ReceiverName::WebPage, false, false },
#if ENABLE(INPUT_TYPE_COLOR)
    { "WebPage_DidChooseColor"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(DATE_AND_TIME_INPUT_TYPES)
    { "WebPage_DidChooseDate"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_DidChooseFilesForOpenPanel"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_DidChooseFilesForOpenPanelWithDisplayStringAndIcon"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(DATALIST_ELEMENT)
    { "WebPage_DidCloseSuggestions"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
    { "WebPage_DidConcludeDrop"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(CONTEXT_MENUS)
    { "WebPage_DidDismissContextMenu"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(INPUT_TYPE_COLOR)
    { "WebPage_DidEndColorPicker"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(DATE_AND_TIME_INPUT_TYPES)
    { "WebPage_DidEndDateTimePicker"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPage_DidEndMagnificationGesture"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(WRITING_TOOLS)
    { "WebPage_DidEndWritingToolsSession"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_DidFinishCheckingText"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DidFinishLoadInAnotherProcess"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DidGetLoadDecisionForIcon"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_DidInsertFinalDictationResult"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(POINTER_LOCK)
    { "WebPage_DidLosePointerLock"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DidNotAcquirePointerLock"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(GEOLOCATION)
    { "WebPage_DidReceiveGeolocationPermissionDecision"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_DidRecognizeLongPress"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_DidRemoveBackForwardItem"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DidRemoveEditCommand"_s, ReceiverName::WebPage, false, false },
#if ENABLE(DATALIST_ELEMENT)
    { "WebPage_DidSelectDataListOption"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(CONTEXT_MENUS)
    { "WebPage_DidSelectItemFromActiveContextMenu"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(DRAG_SUPPORT)
    { "WebPage_DidStartDrag"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_DisconnectInspector"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DispatchLoadEventToFrameOwnerElement"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_DispatchWheelEventWithoutScrolling"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(DRAG_SUPPORT)
    { "WebPage_DragCancelled"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DragEnded"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(GTK)
    { "WebPage_DrawPagesForPrinting"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DrawPagesForPrintingDuringDOMPrintOperation"_s, ReceiverName::WebPage, false, true },
#endif
#if PLATFORM(COCOA)
    { "WebPage_DrawPagesToPDF"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DrawPagesToPDFDuringDOMPrintOperation"_s, ReceiverName::WebPage, false, true },
    { "WebPage_DrawRectToImage"_s, ReceiverName::WebPage, false, false },
    { "WebPage_DrawRectToImageDuringDOMPrintOperation"_s, ReceiverName::WebPage, false, true },
#endif
#if (PLATFORM(COCOA) && PLATFORM(IOS_FAMILY))
    { "WebPage_DrawToImage"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_DrawToPDF"_s, ReceiverName::WebPage, false, false },
#endif
#if (PLATFORM(COCOA) && PLATFORM(IOS_FAMILY))
    { "WebPage_DrawToPDFiOS"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_DynamicViewportSizeUpdate"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_EffectiveAppearanceDidChange"_s, ReceiverName::WebPage, false, false },
#if ENABLE(WRITING_TOOLS_UI)
    { "WebPage_EnableSourceTextAnimationAfterElementWithID"_s, ReceiverName::WebPage, false, false },
    { "WebPage_EnableTextAnimationTypeForElementWithID"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_EndPrinting"_s, ReceiverName::WebPage, false, false },
    { "WebPage_EndPrintingDuringDOMPrintOperation"_s, ReceiverName::WebPage, false, true },
    { "WebPage_EnqueueSecurityPolicyViolationEvent"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ExecuteEditCommand"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ExecuteEditCommandWithCallback"_s, ReceiverName::WebPage, false, false },
#if ENABLE(SANDBOX_EXTENSIONS)
    { "WebPage_ExtendSandboxForFilesFromOpenPanel"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ExtendSelection"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ExtendSelectionForReplacement"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(GTK)
    { "WebPage_FailedToShowPopupMenu"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_FindRectsForStringMatches"_s, ReceiverName::WebPage, false, false },
    { "WebPage_FindString"_s, ReceiverName::WebPage, false, false },
#if ENABLE(IMAGE_ANALYSIS)
    { "WebPage_FindStringIncludingImages"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_FindStringMatches"_s, ReceiverName::WebPage, false, false },
    { "WebPage_FindTextRangesForStringMatches"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_FirstRectForCharacterRangeAsync"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_FlushDeferredDidReceiveMouseEvent"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_FocusNextFocusedElement"_s, ReceiverName::WebPage, false, false },
    { "WebPage_FocusTextInputContextAndPlaceCaret"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_FrameNameWasChangedInAnotherProcess"_s, ReceiverName::WebPage, false, false },
    { "WebPage_FrameWasFocusedInAnotherProcess"_s, ReceiverName::WebPage, false, false },
    { "WebPage_FrameWasRemovedInAnotherProcess"_s, ReceiverName::WebPage, false, false },
#if ENABLE(GAMEPAD)
    { "WebPage_GamepadActivity"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_GenerateSyntheticEditingCommand"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_GenerateTestReport"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetAccessibilityTreeData"_s, ReceiverName::WebPage, false, false },
#if ENABLE(APPLICATION_MANIFEST)
    { "WebPage_GetApplicationManifest"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_GetBytecodeProfile"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_GetContentsAsAttributedString"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(MHTML)
    { "WebPage_GetContentsAsMHTMLData"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_GetContentsAsString"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetFrameInfo"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetFrameTree"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetImageForFindMatch"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetLoadedSubresourceDomains"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetMainResourceDataOfFrame"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_GetMarkedRangeAsync"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_GetPDFFirstPageSize"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetProcessDisplayName"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_GetRectsAtSelectionOffsetWithText"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetRectsForGranularityWithSelectionOffset"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_GetRenderTreeExternalRepresentation"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetResourceDataFromFrame"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetSamplingProfilerOutput"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_GetSelectedRangeAsync"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_GetSelectionAsWebArchiveData"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_GetSelectionContext"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_GetSelectionOrContentsAsString"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetSourceForFrame"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetTextFragmentMatch"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetWebArchiveOfFrame"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GetWebArchiveOfFrameWithFileName"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GoToBackForwardItem"_s, ReceiverName::WebPage, false, false },
    { "WebPage_GoToBackForwardItemWaitingForProcessLaunch"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(MAC)
    { "WebPage_HandleAcceptedCandidate"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_HandleAlternativeTextUIResult"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_HandleAutocorrectionContextRequest"_s, ReceiverName::WebPage, false, false },
    { "WebPage_HandleDoubleTapForDoubleClickAtPoint"_s, ReceiverName::WebPage, false, false },
    { "WebPage_HandleTwoFingerTapAtPoint"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_HandleWheelEvent"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_HardwareKeyboardAvailabilityChanged"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_HasMarkedText"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_HideFindIndicator"_s, ReceiverName::WebPage, false, false },
    { "WebPage_HideFindUI"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(MAC)
    { "WebPage_ImmediateActionDidCancel"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ImmediateActionDidComplete"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ImmediateActionDidUpdate"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_IncreaseListLevel"_s, ReceiverName::WebPage, false, false },
    { "WebPage_IndicateFindMatch"_s, ReceiverName::WebPage, false, false },
#if ENABLE(ATTACHMENT_ELEMENT)
    { "WebPage_InsertAttachment"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_InsertDictatedTextAsync"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
    { "WebPage_InsertDroppedImagePlaceholders"_s, ReceiverName::WebPage, false, false },
#endif
#if (PLATFORM(COCOA) && ENABLE(MULTI_REPRESENTATION_HEIC))
    { "WebPage_InsertMultiRepresentationHEIC"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_InsertTextAsync"_s, ReceiverName::WebPage, false, false },
    { "WebPage_InsertTextPlaceholder"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_InspectorNodeSearchEndedAtPosition"_s, ReceiverName::WebPage, false, false },
    { "WebPage_InspectorNodeSearchMovedToPosition"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_KeyEvent"_s, ReceiverName::WebPage, false, false },
    { "WebPage_LastNavigationWasAppInitiated"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ListenForLayoutMilestones"_s, ReceiverName::WebPage, false, false },
    { "WebPage_LoadAlternateHTML"_s, ReceiverName::WebPage, false, false },
    { "WebPage_LoadAndDecodeImage"_s, ReceiverName::WebPage, false, false },
    { "WebPage_LoadData"_s, ReceiverName::WebPage, false, false },
    { "WebPage_LoadDataInFrame"_s, ReceiverName::WebPage, false, false },
    { "WebPage_LoadDidCommitInAnotherProcess"_s, ReceiverName::WebPage, false, false },
    { "WebPage_LoadRequest"_s, ReceiverName::WebPage, false, false },
    { "WebPage_LoadRequestWaitingForProcessLaunch"_s, ReceiverName::WebPage, false, false },
    { "WebPage_LoadSimulatedRequestAndResponse"_s, ReceiverName::WebPage, false, false },
    { "WebPage_LoadURLInFrame"_s, ReceiverName::WebPage, false, false },
#if USE(APPKIT)
    { "WebPage_LowercaseWord"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_MainFrameURLChangedInAnotherProcess"_s, ReceiverName::WebPage, false, false },
#if ENABLE(ENCRYPTED_MEDIA)
    { "WebPage_MediaKeySystemWasDenied"_s, ReceiverName::WebPage, false, false },
    { "WebPage_MediaKeySystemWasGranted"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW_IOS)
    { "WebPage_ModelInlinePreviewDidFailToLoad"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ModelInlinePreviewDidLoad"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_MouseEvent"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_MoveSelectionAtBoundaryWithDirection"_s, ReceiverName::WebPage, false, false },
    { "WebPage_MoveSelectionByOffset"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_NavigateServiceWorkerClient"_s, ReceiverName::WebPage, false, false },
    { "WebPage_NavigateToPDFLinkWithSimulatedClick"_s, ReceiverName::WebPage, false, false },
    { "WebPage_NotifyReportObservers"_s, ReceiverName::WebPage, false, false },
    { "WebPage_NotifyUserScripts"_s, ReceiverName::WebPage, false, false },
    { "WebPage_NumberOfVisibilityAdjustmentRects"_s, ReceiverName::WebPage, false, false },
#if ENABLE(PDF_PLUGIN) && PLATFORM(MAC)
    { "WebPage_OpenPDFWithPreview"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
    { "WebPage_PauseAllAnimations"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_PauseAllMediaPlayback"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_PerformActionOnElement"_s, ReceiverName::WebPage, false, false },
    { "WebPage_PerformActionOnElements"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_PerformDictionaryLookupAtLocation"_s, ReceiverName::WebPage, false, false },
    { "WebPage_PerformDictionaryLookupOfCurrentSelection"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(GTK) && ENABLE(DRAG_SUPPORT) || !PLATFORM(GTK) && ENABLE(DRAG_SUPPORT)
    { "WebPage_PerformDragControllerAction"_s, ReceiverName::WebPage, false, false },
#endif
#if !PLATFORM(GTK) && ENABLE(DRAG_SUPPORT)
    { "WebPage_PerformDragOperation"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_PerformHitTestForMouseEvent"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(MAC)
    { "WebPage_PerformImmediateActionHitTestAtLocation"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
    { "WebPage_PlayAllAnimations"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
    { "WebPage_PlaybackTargetAvailabilityDidChange"_s, ReceiverName::WebPage, false, false },
    { "WebPage_PlaybackTargetPickerWasDismissed"_s, ReceiverName::WebPage, false, false },
    { "WebPage_PlaybackTargetSelected"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_PostInjectedBundleMessage"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_PotentialTapAtPosition"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_PreferencesDidChange"_s, ReceiverName::WebPage, false, false },
    { "WebPage_PreferencesDidChangeDuringDOMPrintOperation"_s, ReceiverName::WebPage, false, true },
#if PLATFORM(WPE) && USE(GBM) && ENABLE(WPE_PLATFORM)
    { "WebPage_PreferredBufferFormatsDidChange"_s, ReceiverName::WebPage, false, false },
#endif
#if (PLATFORM(IOS_FAMILY) && ENABLE(REVEAL))
    { "WebPage_PrepareSelectionForContextMenuWithLocationInView"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(WRITING_TOOLS)
    { "WebPage_ProofreadingSessionDidReceiveSuggestions"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ProofreadingSessionDidUpdateStateForSuggestion"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_ReapplyEditCommand"_s, ReceiverName::WebPage, true, false },
#if PLATFORM(COCOA)
    { "WebPage_RegisterUIProcessAccessibilityTokens"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_RegisterURLSchemeHandler"_s, ReceiverName::WebPage, false, false },
    { "WebPage_Reload"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RemoteDictionaryPopupInfoToRootView"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RemotePostMessage"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RemoteViewPointToRootView"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RemoteViewRectToRootView"_s, ReceiverName::WebPage, false, false },
#if ENABLE(DATA_DETECTION)
    { "WebPage_RemoveDataDetectedLinks"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_RemoveLayerForFindOverlay"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_RemoveTextPlaceholder"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ReplaceDictatedText"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_ReplaceFoundTextRangeWithString"_s, ReceiverName::WebPage, false, false },
#if ENABLE(IMAGE_ANALYSIS_ENHANCEMENTS)
    { "WebPage_ReplaceImageForRemoveBackground"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_ReplaceMatches"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ReplaceSelectedText"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_ReplaceSelectionWithPasteboardData"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPage_RequestAcceptsFirstMouse"_s, ReceiverName::WebPage, false, true },
#endif
#if PLATFORM(COCOA)
    { "WebPage_RequestActiveNowPlayingSessionInfo"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
    { "WebPage_RequestAdditionalItemsForDragSession"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_RequestAutocorrectionData"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RequestDocumentEditingContext"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
    { "WebPage_RequestDragStart"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_RequestEvasionRectsAboveSelection"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RequestFocusedElementInformation"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_RequestFontAttributesAtSelectionStart"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RequestImageBitmap"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RequestMediaPlaybackState"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_RequestPositionInformation"_s, ReceiverName::WebPage, false, false },
#endif
#if (PLATFORM(IOS_FAMILY) && ENABLE(REVEAL))
    { "WebPage_RequestRVItemInCurrentSelectedRange"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_RequestRectForFoundTextRange"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RequestTargetedElement"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RequestTextExtraction"_s, ReceiverName::WebPage, false, false },
#if ENABLE(IOS_TOUCH_EVENTS)
    { "WebPage_ResetPotentialTapSecurityOrigin"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_ResetVisibilityAdjustmentsForTargetedElements"_s, ReceiverName::WebPage, false, false },
#if ENABLE(APP_HIGHLIGHTS)
    { "WebPage_RestoreAppHighlightsAndScrollToIndex"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_RestoreSelectionInFocusedEditableElement"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RestoreSession"_s, ReceiverName::WebPage, false, false },
    { "WebPage_Resume"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ResumeActiveDOMObjectsAndAnimations"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ResumeAllMediaPlayback"_s, ReceiverName::WebPage, false, false },
    { "WebPage_RunJavaScriptInFrameInScriptWorld"_s, ReceiverName::WebPage, false, false },
#if ENABLE(PDF_PLUGIN) && PLATFORM(MAC)
    { "WebPage_SavePDF"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_ScalePage"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ScalePageInViewCoordinates"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ScaleView"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ScheduleFullEditorStateUpdate"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ScrollBy"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ScrollTextRangeToVisible"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ScrollToRect"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SelectAll"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SelectFindMatch"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SelectPositionAtBoundaryWithDirection"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SelectPositionAtPoint"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SelectTextWithGranularityAtPoint"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SelectWithGesture"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SelectWithTwoTouches"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SelectWordBackward"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPage_SemanticContextDidChange"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SendMessageToTargetBackend"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPage_SendMessageToWebProcessExtension"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SendMessageToWebProcessExtensionWithReply"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SendReportToEndpoints"_s, ReceiverName::WebPage, false, false },
#if HAVE(APP_ACCENT_COLORS)
    { "WebPage_SetAccentColor"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetActivityState"_s, ReceiverName::WebPage, false, false },
#if ENABLE(ADVANCED_PRIVACY_PROTECTIONS)
    { "WebPage_SetAllowedQueryParametersForAdvancedPrivacyProtections"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SetAllowsMediaDocumentInlinePlayback"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetAlwaysShowsHorizontalScroller"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetAlwaysShowsVerticalScroller"_s, ReceiverName::WebPage, false, false },
#if ENABLE(APP_HIGHLIGHTS)
    { "WebPage_SetAppHighlightsVisibility"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetAutoSizingShouldExpandToViewHeight"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetBackgroundColor"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetBackgroundExtendsBeyondPage"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetBaseWritingDirection"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetCanRunBeforeUnloadConfirmPanel"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetCanRunModal"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetCanShowPlaceholder"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetCanUseCredentialStorage"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(MAC)
    { "WebPage_SetCaretAnimatorType"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetCaretBlinkingSuspended"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_SetCompositionAsync"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetControlledByAutomation"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetCurrentHistoryItemForReattach"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetCustomTextEncodingName"_s, ReceiverName::WebPage, false, false },
#if HAVE(SPATIAL_TRACKING_LABEL)
    { "WebPage_SetDefaultSpatialTrackingLabel"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetDefaultUnobscuredSize"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SetDeviceOrientation"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetEditable"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetEnableHorizontalRubberBanding"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetEnableVerticalRubberBanding"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetFixedLayoutSize"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SetFocusedElementSelectedIndex"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetFocusedElementValue"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPage_SetFooterBannerHeight"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SetForceAlwaysUserScalable"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetGapBetweenPages"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetHasCustomUserAgent"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetHasResourceLoadClient"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(MAC)
    { "WebPage_SetHeaderBannerHeight"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(REMOTE_INSPECTOR)
    { "WebPage_SetIndicating"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetInitialFocus"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SetInsertionPointColor"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(INTERACTION_REGIONS_IN_EVENT_REGION)
    { "WebPage_SetInteractionRegionsEnabled"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SetIsShowingInputViewForFocusedElement"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetIsSuspended"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetIsTakingSnapshotsForApplicationSuspension"_s, ReceiverName::WebPage, false, false },
#if HAVE(UIKIT_RESIZABLE_WINDOWS)
    { "WebPage_SetIsWindowResizingEnabled"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetLastKnownMousePosition"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetLayerHostingMode"_s, ReceiverName::WebPage, false, false },
#if ENABLE(ADVANCED_PRIVACY_PROTECTIONS)
    { "WebPage_SetLinkDecorationFilteringData"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_SetMainFrameIsScrollable"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetMaximumUnobscuredSize"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetMayStartMediaWhenInWindow"_s, ReceiverName::WebPage, false, false },
#if ENABLE(EXTENSION_CAPABILITIES)
    { "WebPage_SetMediaEnvironment"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetMediaVolume"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetMinimumSizeForAutoLayout"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetMinimumUnobscuredSize"_s, ReceiverName::WebPage, false, false },
#if (ENABLE(MEDIA_STREAM) && USE(GSTREAMER))
    { "WebPage_SetMockCaptureDevicesInterrupted"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetMuted"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetNeedsDOMWindowResizeEvent"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetNeedsFontAttributes"_s, ReceiverName::WebPage, false, false },
#if (ENABLE(MEDIA_STREAM) && USE(GSTREAMER))
    { "WebPage_SetOrientationForMediaCapture"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetOverriddenMediaType"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SetOverrideViewportArguments"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetPageAndTextZoomFactors"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetPageLength"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetPageZoomFactor"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetPaginationBehavesLikeColumns"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetPaginationMode"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SetSceneIdentifier"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetScreenIsBeingCaptured"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetScrollPinningBehavior"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetScrollbarOverlayStyle"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetShouldDispatchFakeMouseMoveEvents"_s, ReceiverName::WebPage, false, false },
#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
    { "WebPage_SetShouldPlayToPlaybackTarget"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SetShouldRevealCurrentSelectionAfterInsertion"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetShouldScaleViewToFitDocument"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetSizeToContentAutoSizeMaximumSize"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_SetSmartInsertDeleteEnabled"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetSuppressScrollbarAnimations"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_SetTextAsync"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetTextForActivePopupMenu"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetTextZoomFactor"_s, ReceiverName::WebPage, false, false },
#if !PLATFORM(COCOA)
    { "WebPage_SetTopContentInset"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_SetTopContentInsetFenced"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetUnderPageBackgroundColorOverride"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetUnderlayColor"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetUseFixedLayout"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetUseIconLoadingClient"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(MAC)
    { "WebPage_SetUseSystemAppearance"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetUserAgent"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SetUserInterfaceLayoutDirection"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SetViewportConfigurationViewLayoutSize"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SetViewportSizeForCSSViewportUnits"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(COCOA)
    { "WebPage_SetWritingSuggestion"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(IMAGE_ANALYSIS_ENHANCEMENTS)
    { "WebPage_ShouldAllowRemoveBackground"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ShouldDismissKeyboardAfterTapAtPoint"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(WRITING_TOOLS_UI)
    { "WebPage_ShowSelectionForActiveWritingToolsSession"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_SimulateClickOverFirstMatchingTextInViewportWithUserInteraction"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SimulateDeviceOrientationChange"_s, ReceiverName::WebPage, false, false },
#if ENABLE(SPEECH_SYNTHESIS)
    { "WebPage_SpeakingErrorOccurred"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_StartAutoscrollAtPosition"_s, ReceiverName::WebPage, false, false },
    { "WebPage_StartInteractionWithElementContextOrPosition"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_StartObservingNowPlayingMetadata"_s, ReceiverName::WebPage, false, false },
#if ENABLE(VIDEO_PRESENTATION_MODE)
    { "WebPage_StartPlayingPredominantVideo"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_StartTextManipulations"_s, ReceiverName::WebPage, false, false },
#if ENABLE(IMAGE_ANALYSIS)
    { "WebPage_StartVisualTranslation"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_StopInteraction"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_StopLoading"_s, ReceiverName::WebPage, false, false },
    { "WebPage_StopLoadingDueToProcessSwap"_s, ReceiverName::WebPage, false, false },
    { "WebPage_StopMediaCapture"_s, ReceiverName::WebPage, false, false },
    { "WebPage_StopObservingNowPlayingMetadata"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_StoreSelectionForAccessibility"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_Suspend"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SuspendActiveDOMObjectsAndAnimations"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SuspendAllMediaPlayback"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SwipeAnimationDidEnd"_s, ReceiverName::WebPage, false, false },
    { "WebPage_SwipeAnimationDidStart"_s, ReceiverName::WebPage, false, false },
#if USE(SYSTEM_PREVIEW)
    { "WebPage_SystemPreviewActionTriggered"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_TakeSnapshot"_s, ReceiverName::WebPage, false, false },
    { "WebPage_TakeSnapshotForTargetedElement"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_TapHighlightAtPosition"_s, ReceiverName::WebPage, false, false },
    { "WebPage_TextInputContextsInRect"_s, ReceiverName::WebPage, false, false },
#endif
#if !ENABLE(IOS_TOUCH_EVENTS) && ENABLE(TOUCH_EVENTS)
    { "WebPage_TouchEvent"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_TouchWithIdentifierWasRemoved"_s, ReceiverName::WebPage, false, false },
    { "WebPage_TryClose"_s, ReceiverName::WebPage, false, false },
    { "WebPage_TryRestoreScrollPosition"_s, ReceiverName::WebPage, false, false },
    { "WebPage_URLSchemeTaskDidComplete"_s, ReceiverName::WebPage, false, false },
    { "WebPage_URLSchemeTaskDidPerformRedirection"_s, ReceiverName::WebPage, false, false },
    { "WebPage_URLSchemeTaskDidReceiveData"_s, ReceiverName::WebPage, false, false },
    { "WebPage_URLSchemeTaskDidReceiveResponse"_s, ReceiverName::WebPage, false, false },
    { "WebPage_URLSchemeTaskWillPerformRedirection"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UnapplyEditCommand"_s, ReceiverName::WebPage, true, false },
#if ENABLE(ATTACHMENT_ELEMENT)
    { "WebPage_UpdateAttachmentAttributes"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UpdateAttachmentIcon"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UpdateAttachmentThumbnail"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_UpdateBackForwardListForReattach"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UpdateCORSDisablingPatterns"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UpdateCurrentModifierState"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UpdateFrameSize"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UpdateRemotePageAccessibilityOffset"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UpdateRenderingWithForcedRepaint"_s, ReceiverName::WebPage, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_UpdateSelectionWithDelta"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UpdateSelectionWithExtentPoint"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UpdateSelectionWithExtentPointAndBoundary"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UpdateSelectionWithTouches"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(WRITING_TOOLS_UI)
    { "WebPage_UpdateUnderlyingTextVisibilityForTextAnimationID"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_UpdateWebsitePolicies"_s, ReceiverName::WebPage, false, false },
#if ENABLE(IMAGE_ANALYSIS)
    { "WebPage_UpdateWithTextRecognitionResult"_s, ReceiverName::WebPage, false, false },
#endif
#if USE(APPKIT)
    { "WebPage_UppercaseWord"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_UseRedirectionForCurrentNavigation"_s, ReceiverName::WebPage, false, false },
#if ENABLE(MEDIA_STREAM)
    { "WebPage_UserMediaAccessWasDenied"_s, ReceiverName::WebPage, false, false },
    { "WebPage_UserMediaAccessWasGranted"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_ValidateCommand"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ViewWillEndLiveResize"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ViewWillStartLiveResize"_s, ReceiverName::WebPage, false, false },
#if ENABLE(SPEECH_SYNTHESIS)
    { "WebPage_VoicesDidChange"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_WasLoadedWithDataTransferFromPrevalentResource"_s, ReceiverName::WebPage, false, false },
#if ENABLE(WRITING_TOOLS)
    { "WebPage_WillBeginWritingToolsSession"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_WillInsertFinalDictationResult"_s, ReceiverName::WebPage, false, false },
    { "WebPage_WillStartUserTriggeredZooming"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_WindowAndViewFramesChanged"_s, ReceiverName::WebPage, false, false },
#endif
    { "WebPage_WindowScreenDidChange"_s, ReceiverName::WebPage, false, false },
#if ENABLE(WRITING_TOOLS)
    { "WebPage_WritingToolsSessionDidReceiveAction"_s, ReceiverName::WebPage, false, false },
#endif
#if ENABLE(PDF_PLUGIN) && PLATFORM(MAC)
    { "WebPage_ZoomPDFIn"_s, ReceiverName::WebPage, false, false },
    { "WebPage_ZoomPDFOut"_s, ReceiverName::WebPage, false, false },
#endif
#if PLATFORM(GTK)
    { "WebPasteboardProxy_ClearClipboard"_s, ReceiverName::WebPasteboardProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPasteboardProxy_UpdateSupportedTypeIdentifiers"_s, ReceiverName::WebPasteboardProxy, false, false },
    { "WebPasteboardProxy_WriteImageToPasteboard"_s, ReceiverName::WebPasteboardProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY) || USE(LIBWPE)
    { "WebPasteboardProxy_WriteStringToPasteboard"_s, ReceiverName::WebPasteboardProxy, false, false },
#endif
#if PLATFORM(GTK)
    { "WebPasteboardProxy_WriteToClipboard"_s, ReceiverName::WebPasteboardProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPasteboardProxy_WriteURLToPasteboard"_s, ReceiverName::WebPasteboardProxy, false, false },
#endif
#if PLATFORM(IOS_FAMILY) || USE(LIBWPE)
    { "WebPasteboardProxy_WriteWebContentToPasteboard"_s, ReceiverName::WebPasteboardProxy, false, false },
#endif
    { "WebPermissionControllerProxy_Query"_s, ReceiverName::WebPermissionControllerProxy, false, false },
    { "WebPermissionController_permissionChanged"_s, ReceiverName::WebPermissionController, false, false },
    { "WebProcessPool_HandleMessage"_s, ReceiverName::WebProcessPool, false, false },
#if ENABLE(GAMEPAD)
    { "WebProcessPool_PlayGamepadEffect"_s, ReceiverName::WebProcessPool, false, false },
#endif
    { "WebProcessPool_ReportWebContentCPUTime"_s, ReceiverName::WebProcessPool, false, false },
#if ENABLE(GAMEPAD)
    { "WebProcessPool_StartedUsingGamepads"_s, ReceiverName::WebProcessPool, false, false },
    { "WebProcessPool_StopGamepadEffects"_s, ReceiverName::WebProcessPool, false, false },
    { "WebProcessPool_StoppedUsingGamepads"_s, ReceiverName::WebProcessPool, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebProcessProxy_CacheMediaMIMETypes"_s, ReceiverName::WebProcessProxy, false, false },
#endif
#if ENABLE(GPU_PROCESS)
    { "WebProcessProxy_CreateGPUProcessConnection"_s, ReceiverName::WebProcessProxy, true, false },
#endif
#if ENABLE(MODEL_PROCESS)
    { "WebProcessProxy_CreateModelProcessConnection"_s, ReceiverName::WebProcessProxy, true, false },
#endif
    { "WebProcessProxy_CreateSpeechRecognitionServer"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_DestroySpeechRecognitionServer"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_DidCollectPrewarmInformation"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_DidDestroyFrame"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_DidDestroyUserGestureToken"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_DidExceedActiveMemoryLimit"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_DidExceedCPULimit"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_DidExceedInactiveMemoryLimit"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_DidExceedMemoryFootprintThreshold"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_DidReceiveBackgroundResponsivenessPing"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_DisableSuddenTermination"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_EnableSuddenTermination"_s, ReceiverName::WebProcessProxy, false, false },
#if ENABLE(GPU_PROCESS)
    { "WebProcessProxy_GPUProcessConnectionDidBecomeUnresponsive"_s, ReceiverName::WebProcessProxy, false, false },
#endif
    { "WebProcessProxy_GetNotifications"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_MemoryPressureStatusChanged"_s, ReceiverName::WebProcessProxy, false, false },
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebProcessProxy_SendMessageToWebContext"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_SendMessageToWebContextWithReply"_s, ReceiverName::WebProcessProxy, false, false },
#endif
    { "WebProcessProxy_SetAppBadge"_s, ReceiverName::WebProcessProxy, false, false },
#if HAVE(MEDIA_ACCESSIBILITY_FRAMEWORK)
    { "WebProcessProxy_SetCaptionDisplayMode"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_SetCaptionLanguage"_s, ReceiverName::WebProcessProxy, false, false },
#endif
    { "WebProcessProxy_SetClientBadge"_s, ReceiverName::WebProcessProxy, false, false },
#if HAVE(DISPLAY_LINK)
    { "WebProcessProxy_SetDisplayLinkPreferredFramesPerSecond"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_StartDisplayLink"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_StopDisplayLink"_s, ReceiverName::WebProcessProxy, false, false },
#endif
    { "WebProcessProxy_StopResponsivenessTimer"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_SystemBeep"_s, ReceiverName::WebProcessProxy, false, false },
    { "WebProcessProxy_UpdateBackForwardItem"_s, ReceiverName::WebProcessProxy, false, false },
#if PLATFORM(COCOA)
    { "WebProcess_AccessibilityPreferencesDidChange"_s, ReceiverName::WebProcess, false, false },
#endif
#if ENABLE(MEDIA_STREAM)
    { "WebProcess_AddMockMediaDevice"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_BackgroundResponsivenessPing"_s, ReceiverName::WebProcess, false, false },
#if PLATFORM(IOS_FAMILY) && !PLATFORM(MACCATALYST)
    { "WebProcess_BacklightLevelDidChange"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_BindAccessibilityFrameWithData"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_ClearCachedPage"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_ClearCurrentModifierStateForTesting"_s, ReceiverName::WebProcess, false, false },
#if ENABLE(MEDIA_STREAM)
    { "WebProcess_ClearMockMediaDevices"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_ClearResourceLoadStatistics"_s, ReceiverName::WebProcess, false, false },
#if PLATFORM(MAC) || PLATFORM(MACCATALYST)
    { "WebProcess_ColorPreferencesDidChange"_s, ReceiverName::WebProcess, false, false },
#endif
#if HAVE(AUDIO_COMPONENT_SERVER_REGISTRATIONS)
    { "WebProcess_ConsumeAudioComponentRegistrations"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_CreateWebPage"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_DeleteAllCookies"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_DeleteWebsiteData"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_DeleteWebsiteDataForOrigin"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_DeleteWebsiteDataForOrigins"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_DestroyAutomationSessionProxy"_s, ReceiverName::WebProcess, false, false },
#if PLATFORM(COCOA)
    { "WebProcess_DidWriteToPasteboardAsynchronously"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_DisableURLSchemeCheckInDataDetectors"_s, ReceiverName::WebProcess, false, false },
#endif
#if HAVE(DISPLAY_LINK)
    { "WebProcess_DisplayDidRefresh"_s, ReceiverName::WebProcess, false, false },
#endif
#if (PLATFORM(COCOA) && ENABLE(REMOTE_INSPECTOR))
    { "WebProcess_EnableRemoteWebInspector"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_EnsureAutomationSessionProxy"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_EstablishRemoteWorkerContextConnectionToNetworkProcess"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_FetchWebsiteData"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_FullKeyboardAccessModeChanged"_s, ReceiverName::WebProcess, false, false },
#if ENABLE(GAMEPAD)
    { "WebProcess_GamepadConnected"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_GamepadDisconnected"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_GarbageCollectJavaScriptObjects"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_GetActivePagesOriginsForTesting"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_GrantAccessToAssetServices"_s, ReceiverName::WebProcess, false, false },
#if (ENABLE(MEDIA_STREAM) && ENABLE(SANDBOX_EXTENSIONS))
    { "WebProcess_GrantUserMediaDeviceSandboxExtensions"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_HandleInjectedBundleMessage"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_InitializeWebProcess"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_IsJITEnabled"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_MarkIsNoLongerPrewarmed"_s, ReceiverName::WebProcess, false, false },
#if PLATFORM(MAC)
    { "WebProcess_OpenDirectoryCacheInvalidated"_s, ReceiverName::WebProcess, false, false },
#endif
#if ENABLE(NOTIFY_BLOCKING)
    { "WebProcess_PostNotification"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_PostObserverNotification"_s, ReceiverName::WebProcess, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebProcess_PowerSourceDidChange"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_PrepareToSuspend"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_PrewarmWithDomainInformation"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_ProcessDidResume"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_RegisterURLSchemeAsBypassingContentSecurityPolicy"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_RegisterURLSchemeAsCORSEnabled"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_RegisterURLSchemeAsCachePartitioned"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_RegisterURLSchemeAsCanDisplayOnlyIfCanRequest"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_RegisterURLSchemeAsDisplayIsolated"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_RegisterURLSchemeAsEmptyDocument"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_RegisterURLSchemeAsLocal"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_RegisterURLSchemeAsNoAccess"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_RegisterURLSchemeAsSecure"_s, ReceiverName::WebProcess, false, false },
#if ENABLE(WK_WEB_EXTENSIONS)
    { "WebProcess_RegisterURLSchemeAsWebExtension"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_ReleaseMemory"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_ReloadExecutionContextsForOrigin"_s, ReceiverName::WebProcess, false, false },
#if ENABLE(MEDIA_STREAM)
    { "WebProcess_RemoveMockMediaDevice"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_ResetMockMediaDevices"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_RevokeAccessToAssetServices"_s, ReceiverName::WebProcess, false, false },
#if (ENABLE(MEDIA_STREAM) && ENABLE(SANDBOX_EXTENSIONS))
    { "WebProcess_RevokeUserMediaDeviceSandboxExtensions"_s, ReceiverName::WebProcess, false, false },
#endif
#if PLATFORM(MAC)
    { "WebProcess_ScrollerStylePreferenceChanged"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_SeedResourceLoadStatisticsForTesting"_s, ReceiverName::WebProcess, false, false },
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebProcess_SendMessageToWebProcessExtension"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_SendResourceLoadStatisticsDataImmediately"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetAlwaysUsesComplexTextCodePath"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetBackForwardCacheCapacity"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetCacheModel"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetDefaultRequestTimeoutInterval"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetDisableFontSubpixelAntialiasingForTesting"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetDomainRelaxationForbiddenForURLScheme"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetDomainsWithCrossPageStorageAccess"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetDomainsWithUserInteraction"_s, ReceiverName::WebProcess, false, false },
#if ENABLE(SERVICE_CONTROLS)
    { "WebProcess_SetEnabledServices"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_SetEnhancedAccessibility"_s, ReceiverName::WebProcess, false, false },
#if HAVE(MOUSE_DEVICE_OBSERVATION)
    { "WebProcess_SetHasMouseDevice"_s, ReceiverName::WebProcess, false, false },
#endif
#if HAVE(STYLUS_DEVICE_OBSERVATION)
    { "WebProcess_SetHasStylusDevice"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_SetHasSuspendedPageProxy"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetHiddenPageDOMTimerThrottlingIncreaseLimit"_s, ReceiverName::WebProcess, false, false },
#if ENABLE(GAMEPAD)
    { "WebProcess_SetInitialGamepads"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_SetInjectedBundleParameter"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetInjectedBundleParameters"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetIsInProcessCache"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetJavaScriptGarbageCollectorTimerEnabled"_s, ReceiverName::WebProcess, false, false },
#if HAVE(MEDIA_ACCESSIBILITY_FRAMEWORK)
    { "WebProcess_SetMediaAccessibilityPreferences"_s, ReceiverName::WebProcess, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebProcess_SetMediaMIMETypes"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_SetMemoryCacheDisabled"_s, ReceiverName::WebProcess, false, false },
#if ENABLE(MEDIA_STREAM)
    { "WebProcess_SetMockMediaDeviceIsEphemeral"_s, ReceiverName::WebProcess, false, false },
#endif
#if PLATFORM(COCOA) || PLATFORM(GTK) || PLATFORM(WPE)
    { "WebProcess_SetScreenProperties"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_SetTextCheckerState"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetThirdPartyCookieBlockingMode"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SetTrackingPreventionEnabled"_s, ReceiverName::WebProcess, false, false },
#if PLATFORM(GTK) && !USE(GTK4) && USE(CAIRO)
    { "WebProcess_SetUseSystemAppearanceForScrollbars"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_SetWebsiteDataStoreParameters"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_StartMemorySampler"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_StopMemorySampler"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SwitchFromStaticFontRegistryToUserFontRegistry"_s, ReceiverName::WebProcess, false, false },
#if PLATFORM(MAC)
    { "WebProcess_SystemDidWake"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SystemWillPowerOn"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_SystemWillSleep"_s, ReceiverName::WebProcess, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebProcess_UnblockServicesRequiredByAccessibility"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_UpdateDomainsWithStorageAccessQuirks"_s, ReceiverName::WebProcess, false, false },
    { "WebProcess_UpdateStorageAccessUserAgentStringQuirks"_s, ReceiverName::WebProcess, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebProcess_UserInterfaceIdiomDidChange"_s, ReceiverName::WebProcess, false, false },
#endif
    { "WebProcess_UserPreferredLanguagesChanged"_s, ReceiverName::WebProcess, false, false },
    { "WebRTCMonitor_NetworksChanged"_s, ReceiverName::WebRTCMonitor, false, false },
    { "WebRTCResolver_ResolvedAddressError"_s, ReceiverName::WebRTCResolver, false, false },
    { "WebRTCResolver_SetResolvedAddress"_s, ReceiverName::WebRTCResolver, false, false },
#if ENABLE(CONTENT_FILTERING)
    { "WebResourceLoader_ContentFilterDidBlockLoad"_s, ReceiverName::WebResourceLoader, false, false },
#endif
    { "WebResourceLoader_DidBlockAuthenticationChallenge"_s, ReceiverName::WebResourceLoader, false, false },
    { "WebResourceLoader_DidFailResourceLoad"_s, ReceiverName::WebResourceLoader, false, false },
    { "WebResourceLoader_DidFailServiceWorkerLoad"_s, ReceiverName::WebResourceLoader, false, false },
    { "WebResourceLoader_DidFinishResourceLoad"_s, ReceiverName::WebResourceLoader, false, false },
    { "WebResourceLoader_DidReceiveData"_s, ReceiverName::WebResourceLoader, false, false },
#if ENABLE(SHAREABLE_RESOURCE)
    { "WebResourceLoader_DidReceiveResource"_s, ReceiverName::WebResourceLoader, false, false },
#endif
    { "WebResourceLoader_DidReceiveResponse"_s, ReceiverName::WebResourceLoader, false, false },
    { "WebResourceLoader_DidSendData"_s, ReceiverName::WebResourceLoader, false, false },
    { "WebResourceLoader_ServiceWorkerDidNotHandle"_s, ReceiverName::WebResourceLoader, false, false },
    { "WebResourceLoader_SetWorkerStart"_s, ReceiverName::WebResourceLoader, false, false },
    { "WebResourceLoader_StopLoadingAfterXFrameOptionsOrContentSecurityPolicyDenied"_s, ReceiverName::WebResourceLoader, false, false },
    { "WebResourceLoader_WillSendRequest"_s, ReceiverName::WebResourceLoader, false, false },
    { "WebSWClientConnection_FireUpdateFoundEvent"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_FocusServiceWorkerClient"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_JobRejectedInServer"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_NotifyClientsOfControllerChange"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_NotifyRecordResponseBodyChunk"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_NotifyRecordResponseBodyEnd"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_PostMessageToServiceWorkerClient"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_RegistrationJobResolvedInServer"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_SetRegistrationLastUpdateTime"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_SetRegistrationUpdateViaCache"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_SetSWOriginTableIsImported"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_SetSWOriginTableSharedMemory"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_SetServiceWorkerClientIsControlled"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_StartScriptFetchForServer"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_UpdateBackgroundFetchRegistration"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_UpdateRegistrationState"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWClientConnection_UpdateWorkerState"_s, ReceiverName::WebSWClientConnection, false, false },
    { "WebSWContextManagerConnection_CancelFetch"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_Close"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_ContinueDidReceiveFetchResponse"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_ConvertFetchToDownload"_s, ReceiverName::WebSWContextManagerConnection, false, false },
#if ENABLE(SHAREABLE_RESOURCE) && PLATFORM(COCOA)
    { "WebSWContextManagerConnection_DidSaveScriptsToDisk"_s, ReceiverName::WebSWContextManagerConnection, false, false },
#endif
    { "WebSWContextManagerConnection_FireActivateEvent"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_FireBackgroundFetchClickEvent"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_FireBackgroundFetchEvent"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_FireInstallEvent"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_FireNotificationEvent"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_FirePushEvent"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_FireUpdateFoundEvent"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_InstallServiceWorker"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_MatchAllCompleted"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_NavigationPreloadFailed"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_NavigationPreloadIsReady"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_PostMessageToServiceWorker"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_SetInspectable"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_SetRegistrationLastUpdateTime"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_SetRegistrationUpdateViaCache"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_SetThrottleState"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_SetUserAgent"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_SkipWaitingCompleted"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_StartFetch"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_TerminateWorker"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_UpdateAppInitiatedValue"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_UpdatePreferencesStore"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_UpdateRegistrationState"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWContextManagerConnection_UpdateWorkerState"_s, ReceiverName::WebSWContextManagerConnection, false, false },
    { "WebSWServerConnection_AbortBackgroundFetch"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_AddServiceWorkerRegistrationInServer"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_BackgroundFetchIdentifiers"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_BackgroundFetchInformation"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_DidResolveRegistrationPromise"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_DisableNavigationPreload"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_EnableNavigationPreload"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_FinishFetchingScriptInServer"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_GetNavigationPreloadState"_s, ReceiverName::WebSWServerConnection, false, false },
#if ENABLE(WEB_PUSH_NOTIFICATIONS)
    { "WebSWServerConnection_GetNotifications"_s, ReceiverName::WebSWServerConnection, false, false },
#endif
    { "WebSWServerConnection_GetPushPermissionState"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_GetPushSubscription"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_GetRegistrations"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_MatchBackgroundFetch"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_MatchRegistration"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_PostMessageToServiceWorker"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_RegisterServiceWorkerClient"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_RemoveServiceWorkerRegistrationInServer"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_RetrieveRecordResponse"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_RetrieveRecordResponseBody"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_ScheduleJobInServer"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_ScheduleUnregisterJobInServer"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_SetNavigationPreloadHeaderValue"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_SetThrottleState"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_StartBackgroundFetch"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_StoreRegistrationsOnDisk"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_SubscribeToPushService"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_TerminateWorkerFromClient"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_UnregisterServiceWorkerClient"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_UnsubscribeFromPushService"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_WhenRegistrationReady"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerConnection_WhenServiceWorkerIsTerminatedForTesting"_s, ReceiverName::WebSWServerConnection, false, false },
    { "WebSWServerToContextConnection_Claim"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_DidFailHeartBeatCheck"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_DidFinishActivation"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_DidFinishInstall"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_FindClientByVisibleIdentifier"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_Focus"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_MatchAll"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_Navigate"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_OpenWindow"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_PostMessageToServiceWorkerClient"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_ReportConsoleMessage"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_ScriptContextFailedToStart"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_ScriptContextStarted"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_SetAsInspected"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_SetScriptResource"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_SetServiceWorkerHasPendingEvents"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_SkipWaiting"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebSWServerToContextConnection_WorkerTerminated"_s, ReceiverName::WebSWServerToContextConnection, false, false },
    { "WebScreenOrientationManagerProxy_Lock"_s, ReceiverName::WebScreenOrientationManagerProxy, false, false },
    { "WebScreenOrientationManagerProxy_SetShouldSendChangeNotification"_s, ReceiverName::WebScreenOrientationManagerProxy, false, false },
    { "WebScreenOrientationManagerProxy_Unlock"_s, ReceiverName::WebScreenOrientationManagerProxy, false, false },
    { "WebScreenOrientationManager_OrientationDidChange"_s, ReceiverName::WebScreenOrientationManager, false, false },
    { "WebSharedWorkerContextManagerConnection_Close"_s, ReceiverName::WebSharedWorkerContextManagerConnection, false, false },
    { "WebSharedWorkerContextManagerConnection_LaunchSharedWorker"_s, ReceiverName::WebSharedWorkerContextManagerConnection, false, false },
    { "WebSharedWorkerContextManagerConnection_PostConnectEvent"_s, ReceiverName::WebSharedWorkerContextManagerConnection, false, false },
    { "WebSharedWorkerContextManagerConnection_ResumeSharedWorker"_s, ReceiverName::WebSharedWorkerContextManagerConnection, false, false },
    { "WebSharedWorkerContextManagerConnection_SetUserAgent"_s, ReceiverName::WebSharedWorkerContextManagerConnection, false, false },
    { "WebSharedWorkerContextManagerConnection_SuspendSharedWorker"_s, ReceiverName::WebSharedWorkerContextManagerConnection, false, false },
    { "WebSharedWorkerContextManagerConnection_TerminateSharedWorker"_s, ReceiverName::WebSharedWorkerContextManagerConnection, false, false },
    { "WebSharedWorkerContextManagerConnection_UpdatePreferencesStore"_s, ReceiverName::WebSharedWorkerContextManagerConnection, false, false },
    { "WebSharedWorkerObjectConnection_FetchScriptInClient"_s, ReceiverName::WebSharedWorkerObjectConnection, false, false },
    { "WebSharedWorkerObjectConnection_NotifyWorkerObjectOfLoadCompletion"_s, ReceiverName::WebSharedWorkerObjectConnection, false, false },
    { "WebSharedWorkerObjectConnection_PostErrorToWorkerObject"_s, ReceiverName::WebSharedWorkerObjectConnection, false, false },
    { "WebSharedWorkerServerConnection_RequestSharedWorker"_s, ReceiverName::WebSharedWorkerServerConnection, false, false },
    { "WebSharedWorkerServerConnection_ResumeForBackForwardCache"_s, ReceiverName::WebSharedWorkerServerConnection, false, false },
    { "WebSharedWorkerServerConnection_SharedWorkerObjectIsGoingAway"_s, ReceiverName::WebSharedWorkerServerConnection, false, false },
    { "WebSharedWorkerServerConnection_SuspendForBackForwardCache"_s, ReceiverName::WebSharedWorkerServerConnection, false, false },
    { "WebSharedWorkerServerToContextConnection_PostErrorToWorkerObject"_s, ReceiverName::WebSharedWorkerServerToContextConnection, false, false },
    { "WebSharedWorkerServerToContextConnection_SharedWorkerTerminated"_s, ReceiverName::WebSharedWorkerServerToContextConnection, false, false },
    { "WebSocketChannel_DidClose"_s, ReceiverName::WebSocketChannel, false, false },
    { "WebSocketChannel_DidConnect"_s, ReceiverName::WebSocketChannel, false, false },
    { "WebSocketChannel_DidReceiveBinaryData"_s, ReceiverName::WebSocketChannel, false, false },
    { "WebSocketChannel_DidReceiveHandshakeResponse"_s, ReceiverName::WebSocketChannel, false, false },
    { "WebSocketChannel_DidReceiveMessageError"_s, ReceiverName::WebSocketChannel, false, false },
    { "WebSocketChannel_DidReceiveText"_s, ReceiverName::WebSocketChannel, false, false },
    { "WebSocketChannel_DidSendHandshakeRequest"_s, ReceiverName::WebSocketChannel, false, false },
    { "WebSpeechRecognitionConnection_DidReceiveUpdate"_s, ReceiverName::WebSpeechRecognitionConnection, false, false },
    { "WebTransportSession_ReceiveBidirectionalStream"_s, ReceiverName::WebTransportSession, false, false },
    { "WebTransportSession_ReceiveDatagram"_s, ReceiverName::WebTransportSession, false, false },
    { "WebTransportSession_ReceiveIncomingUnidirectionalStream"_s, ReceiverName::WebTransportSession, false, false },
    { "WebTransportSession_StreamReceiveBytes"_s, ReceiverName::WebTransportSession, false, false },
    { "WebUserContentControllerProxy_DidPostMessage"_s, ReceiverName::WebUserContentControllerProxy, false, false },
#if ENABLE(CONTENT_EXTENSIONS)
    { "WebUserContentController_AddContentRuleLists"_s, ReceiverName::WebUserContentController, false, false },
#endif
    { "WebUserContentController_AddContentWorlds"_s, ReceiverName::WebUserContentController, false, false },
    { "WebUserContentController_AddUserScriptMessageHandlers"_s, ReceiverName::WebUserContentController, false, false },
    { "WebUserContentController_AddUserScripts"_s, ReceiverName::WebUserContentController, false, false },
    { "WebUserContentController_AddUserStyleSheets"_s, ReceiverName::WebUserContentController, false, false },
#if ENABLE(CONTENT_EXTENSIONS)
    { "WebUserContentController_RemoveAllContentRuleLists"_s, ReceiverName::WebUserContentController, false, false },
#endif
    { "WebUserContentController_RemoveAllUserScriptMessageHandlers"_s, ReceiverName::WebUserContentController, false, false },
    { "WebUserContentController_RemoveAllUserScriptMessageHandlersForWorlds"_s, ReceiverName::WebUserContentController, false, false },
    { "WebUserContentController_RemoveAllUserScripts"_s, ReceiverName::WebUserContentController, false, false },
    { "WebUserContentController_RemoveAllUserStyleSheets"_s, ReceiverName::WebUserContentController, false, false },
#if ENABLE(CONTENT_EXTENSIONS)
    { "WebUserContentController_RemoveContentRuleList"_s, ReceiverName::WebUserContentController, false, false },
#endif
    { "WebUserContentController_RemoveContentWorlds"_s, ReceiverName::WebUserContentController, false, false },
    { "WebUserContentController_RemoveUserScript"_s, ReceiverName::WebUserContentController, false, false },
    { "WebUserContentController_RemoveUserScriptMessageHandler"_s, ReceiverName::WebUserContentController, false, false },
    { "WebUserContentController_RemoveUserStyleSheet"_s, ReceiverName::WebUserContentController, false, false },
#if PLATFORM(COCOA)
    { "InitializeConnection"_s, ReceiverName::IPC, false, false },
#endif
    { "LegacySessionState"_s, ReceiverName::IPC, false, false },
    { "ProcessOutOfStreamMessage"_s, ReceiverName::IPC, false, false },
    { "SetStreamDestinationID"_s, ReceiverName::IPC, false, false },
    { "SyncMessageReply"_s, ReceiverName::IPC, false, false },
    { "AuxiliaryProcess_MainThreadPingReply"_s, ReceiverName::AsyncReply, false, false },
    { "DownloadProxy_DecideDestinationWithSuggestedFilenameReply"_s, ReceiverName::AsyncReply, false, false },
    { "DownloadProxy_WillSendRequestReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA) || PLATFORM(GTK)
    { "DrawingArea_CommitTransientZoomReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(COCOA)
    { "DrawingArea_DispatchAfterEnsuringDrawingReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if USE(COORDINATED_GRAPHICS) || USE(TEXTURE_MAPPER) || PLATFORM(COCOA)
    { "DrawingArea_UpdateGeometryReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(MAC_GESTURE_EVENTS)
    { "EventDispatcher_GestureEventReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(IOS_TOUCH_EVENTS)
    { "EventDispatcher_TouchEventReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(ROUTING_ARBITRATION)
    { "GPUProcessConnection_BeginRoutingArbitrationWithCategoryReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(MEDIA_STREAM) && PLATFORM(IOS_FAMILY)
    { "GPUProcessProxy_StatusBarWasTappedReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "GPUProcess_CreateGPUConnectionToWebProcessReply"_s, ReceiverName::AsyncReply, false, false },
    { "GPUProcess_InitializeGPUProcessReply"_s, ReceiverName::AsyncReply, false, false },
    { "GPUProcess_PrepareToSuspendReply"_s, ReceiverName::AsyncReply, false, false },
#if HAVE(SCREEN_CAPTURE_KIT)
    { "GPUProcess_PromptForGetDisplayMediaReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(VIDEO)
    { "GPUProcess_RequestBitmapImageForCurrentTimeReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "GPUProcess_SharedPreferencesForWebProcessDidChangeReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(MEDIA_STREAM)
    { "GPUProcess_UpdateCaptureAccessReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "GPUProcess_WebProcessConnectionCountForTestingReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(WEBXR)
    { "GPUProcess_WebXRPromptAcceptedReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "IPCStreamTester_AsyncPingReply"_s, ReceiverName::AsyncReply, false, false },
    { "IPCTesterReceiver_AsyncMessageReply"_s, ReceiverName::AsyncReply, false, false },
    { "IPCTester_AsyncOptionalExceptionDataReply"_s, ReceiverName::AsyncReply, false, false },
    { "IPCTester_AsyncPingReply"_s, ReceiverName::AsyncReply, false, false },
    { "LibWebRTCCodecsProxy_CreateDecoderReply"_s, ReceiverName::AsyncReply, false, false },
    { "LibWebRTCCodecsProxy_CreateEncoderReply"_s, ReceiverName::AsyncReply, false, false },
    { "LibWebRTCCodecsProxy_EncodeFrameReply"_s, ReceiverName::AsyncReply, false, false },
    { "LibWebRTCCodecsProxy_FlushEncoderReply"_s, ReceiverName::AsyncReply, false, false },
    { "MediaPlayerPrivateRemote_CommitAllTransactionsReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY)
    { "MediaPlayerPrivateRemote_GetRawCookiesReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "MediaPlayerPrivateRemote_SendH2PingReply"_s, ReceiverName::AsyncReply, false, false },
    { "MediaSourcePrivateRemoteMessageReceiver_MediaSourcePrivateShuttingDownReply"_s, ReceiverName::AsyncReply, false, false },
    { "MediaSourcePrivateRemoteMessageReceiver_ProxySeekToTimeReply"_s, ReceiverName::AsyncReply, false, false },
    { "MediaSourcePrivateRemoteMessageReceiver_ProxyWaitForTargetReply"_s, ReceiverName::AsyncReply, false, false },
    { "ModelProcess_CreateModelConnectionToWebProcessReply"_s, ReceiverName::AsyncReply, false, false },
    { "ModelProcess_InitializeModelProcessReply"_s, ReceiverName::AsyncReply, false, false },
    { "ModelProcess_PrepareToSuspendReply"_s, ReceiverName::AsyncReply, false, false },
    { "ModelProcess_WebProcessConnectionCountForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkBroadcastChannelRegistry_PostMessageReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(WEB_RTC)
    { "NetworkConnectionToWebProcess_ConnectToRTCDataChannelRemoteSourceReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkConnectionToWebProcess_CookiesEnabledReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_CookiesForDOMAsyncReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_CreateRTCProviderReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_DeleteCookieReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_EstablishSWContextConnectionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_EstablishSharedWorkerContextConnectionReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(MAC)
    { "NetworkConnectionToWebProcess_GetProcessDisplayNameReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkConnectionToWebProcess_HasStorageAccessReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_InitializeWebTransportSessionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_IsLoggedInReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_IsResourceLoadFinishedReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_LoadImageForDecodingReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(DECLARATIVE_WEB_PUSH)
    { "NetworkConnectionToWebProcess_NavigatorGetPushPermissionStateReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_NavigatorGetPushSubscriptionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_NavigatorSubscribeToPushServiceReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_NavigatorUnsubscribeFromPushServiceReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkConnectionToWebProcess_RegisterBlobPathForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_RequestStorageAccessReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_ResourceLoadStatisticsUpdatedReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_SendH2PingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_SetCookieFromDOMAsyncReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_SetLoginStatusReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_StorageAccessQuirkForTopFrameDomainReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_TakeAllMessagesForPortReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkConnectionToWebProcess_WriteBlobsToTemporaryFilesForIndexedDBReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkMDNSRegister_RegisterMDNSNameReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(WEB_RTC)
    { "NetworkProcessConnection_ConnectToRTCDataChannelRemoteSourceReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkProcessProxy_DataTaskDidReceiveResponseReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_DataTaskReceivedChallengeReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_DataTaskWillPerformHTTPRedirectionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_DeleteWebsiteDataInUIProcessForRegistrableDomainsReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_DeleteWebsiteDataInWebProcessesForOriginReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_EstablishRemoteWorkerContextConnectionToNetworkProcessReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(APP_BOUND_DOMAINS)
    { "NetworkProcessProxy_GetAppBoundDomainsReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(APPLE_PAY_REMOTE_UI)
    { "NetworkProcessProxy_GetPaymentCoordinatorEmbeddingUserAgentReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(APPLE_PAY_REMOTE_UI_USES_SCENE)
    { "NetworkProcessProxy_GetWindowSceneAndBundleIdentifierForPaymentPresentationReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkProcessProxy_NavigateServiceWorkerClientReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_OpenWindowFromServiceWorkerReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_ReloadExecutionContextsForOriginReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_RequestBackgroundFetchPermissionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_RequestStorageAccessConfirmReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_SetDomainsWithCrossPageStorageAccessReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcessProxy_TriggerBrowsingContextGroupSwitchForNavigationReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_AbortBackgroundFetchReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_AddAllowedFirstPartyForCookiesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_AllowFileAccessFromWebProcessReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_AllowFilesAccessFromWebProcessReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA)
    { "NetworkProcess_AppPrivacyReportTestingDataReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkProcess_CancelDataTaskReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_CancelDownloadReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(APP_BOUND_DOMAINS)
    { "NetworkProcess_ClearAppBoundSessionReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(COCOA)
    { "NetworkProcess_ClearAppPrivacyReportTestingDataReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkProcess_ClearBundleIdentifierReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ClearPrevalentResourceReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ClearPrivateClickMeasurementReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY) && !PLATFORM(MACCATALYST)
    { "NetworkProcess_ClearServiceWorkerEntitlementOverrideReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkProcess_ClearUserInteractionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ClickBackgroundFetchReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_CloseITPDatabaseReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ClosePCMDatabaseReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_CountNonDefaultSessionSetsReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_CreateNetworkConnectionToWebProcessReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_DataTaskWithRequestReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_DeleteCookiesForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_DeleteWebsiteDataForOriginsReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_DeleteWebsiteDataReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_DestroySessionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_DomainIDExistsInDatabaseReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_DumpPrivateClickMeasurementReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_DumpResourceLoadStatisticsReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_FetchWebsiteDataReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_FlushCookiesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_GetAllBackgroundFetchIdentifiersReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_GetAllStorageAccessEntriesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_GetAppBadgeForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_GetBackgroundFetchStateReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_GetPendingPushMessageReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_GetPendingPushMessagesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_GetResourceLoadStatisticsDataSummaryReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_GrantStorageAccessForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_HadUserInteractionReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(APP_BOUND_DOMAINS)
    { "NetworkProcess_HasAppBoundSessionReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkProcess_HasIsolatedSessionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_HasLocalStorageReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_HasPushSubscriptionForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_InitializeNetworkProcessReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_InsertExpiredStatisticForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_IsGrandfatheredReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_IsPrevalentResourceReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_IsRegisteredAsRedirectingToReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_IsRegisteredAsSubFrameUnderReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_IsRegisteredAsSubresourceUnderReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_IsRelationshipOnlyInDatabaseOnceReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_IsResourceLoadStatisticsEphemeralReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_IsVeryPrevalentResourceReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_LogUserInteractionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_MarkAttributedPrivateClickMeasurementsAsExpiredForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_MarkPrivateClickMeasurementsAsExpiredForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_MergeStatisticForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_PauseBackgroundFetchReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_PrepareToSuspendReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ProcessNotificationEventReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ProcessPushMessageReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_RemovePushSubscriptionsForOriginReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_RenameOriginInWebsiteDataReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ResetCacheMaxAgeCapForPrevalentResourcesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ResetCrossSiteLoadsWithLinkDecorationForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ResetParametersToDefaultValuesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ResetQuotaReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ResetStoragePersistedStateReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ResumeBackgroundFetchReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ScheduleClearInMemoryAndPersistentReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ScheduleCookieBlockingUpdateReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_ScheduleStatisticsAndDataRecordsProcessingReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(APP_BOUND_DOMAINS)
    { "NetworkProcess_SetAppBoundDomainsForResourceLoadStatisticsReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "NetworkProcess_SetBackupExclusionPeriodForTestingReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkProcess_SetCacheMaxAgeCapForPrevalentResourcesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetCrossSiteLoadWithLinkDecorationForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetFirstPartyHostCNAMEDomainForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetFirstPartyWebsiteDataRemovalModeForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetGrandfatheredReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetGrandfatheringTimeReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetIsRunningResourceLoadStatisticsTestReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetLastSeenReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(MANAGED_DOMAINS)
    { "NetworkProcess_SetManagedDomainsForResourceLoadStatisticsReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkProcess_SetMaxStatisticsEntriesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetMinimumTimeBetweenDataRecordsRemovalReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetNotifyPagesWhenDataRecordsWereScannedReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetOriginQuotaRatioEnabledForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPCMFraudPreventionValuesForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPrevalentResourceForDebugModeReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPrevalentResourceReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementAppBundleIDForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementAttributionReportURLsForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementEphemeralMeasurementForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementOverrideTimerForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementTokenPublicKeyURLForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPrivateClickMeasurementTokenSignatureURLForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPruneEntriesDownToReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetPushAndNotificationsEnabledForOriginReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetResourceLoadStatisticsDebugModeReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetResourceLoadStatisticsTimeAdvanceForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetShouldClassifyResourcesBeforeDataRecordsRemovalReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetShouldDowngradeReferrerForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetShouldEnbleSameSiteStrictEnforcementForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetSubframeUnderTopFrameDomainReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetSubresourceUnderTopFrameDomainReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetSubresourceUniqueRedirectFromReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetSubresourceUniqueRedirectToReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetThirdPartyCNAMEDomainForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetThirdPartyCookieBlockingModeReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetTimeToLiveUserInteractionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetToSameSiteStrictCookiesForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetTopFrameUniqueRedirectFromReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetTopFrameUniqueRedirectToReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SetVeryPrevalentResourceReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SharedPreferencesForWebProcessDidChangeReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SimulatePrivateClickMeasurementSessionRestartReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_StatisticsDatabaseHasAllTablesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_StoreServiceWorkerRegistrationsReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_SyncLocalStorageReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_UpdateBundleIdentifierReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_UpdatePrevalentDomainsToBlockCookiesForReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkProcess_WebsiteDataOriginDirectoryForTestingReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA)
    { "NetworkRTCProvider_GetInterfaceNameReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "NetworkSocketChannel_SendDataReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkSocketChannel_SendStringReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_CacheStorageAllCachesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_CacheStorageClearMemoryRepresentationReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_CacheStorageOpenCacheReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_CacheStoragePutRecordsReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_CacheStorageRemoveCacheReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_CacheStorageRemoveRecordsReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_CacheStorageRepresentationReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_CacheStorageRetrieveRecordsReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_ClearReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_CloseSyncAccessHandleReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_ConnectToStorageAreaReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_CreateSyncAccessHandleReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_EstimateReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_FileSystemGetDirectoryReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_GetDirectoryHandleReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_GetFileHandleReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_GetFileReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_GetHandleNamesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_GetHandleReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_IsSameEntryReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_MoveReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_PersistReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_PersistedReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_RemoveEntryReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_RemoveItemReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_RequestNewCapacityForSyncAccessHandleReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_ResolveReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkStorageManager_SetItemReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkTransportSession_CreateBidirectionalStreamReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkTransportSession_CreateOutgoingUnidirectionalStreamReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkTransportSession_SendDatagramReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkTransportSession_SendStreamSendBytesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NetworkTransportSession_StreamSendBytesReply"_s, ReceiverName::AsyncReply, false, false },
    { "NotificationManagerMessageHandler_GetPermissionStateReply"_s, ReceiverName::AsyncReply, false, false },
    { "NotificationManagerMessageHandler_RequestPermissionReply"_s, ReceiverName::AsyncReply, false, false },
    { "NotificationManagerMessageHandler_ShowNotificationReply"_s, ReceiverName::AsyncReply, false, false },
    { "PlatformXRSystem_EnumerateImmersiveXRDevicesReply"_s, ReceiverName::AsyncReply, false, false },
    { "PlatformXRSystem_RequestFrameReply"_s, ReceiverName::AsyncReply, false, false },
    { "PlatformXRSystem_RequestPermissionOnSessionFeaturesReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_GetAppBadgeForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_GetNotificationsReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_GetPendingPushMessageReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_GetPendingPushMessagesReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_GetPushPermissionStateReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_GetPushSubscriptionReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_GetPushTopicsForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_IncrementSilentPushCountReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_InjectEncryptedPushMessageForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_InjectPushMessageForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_RemoveAllPushSubscriptionsReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_RemovePushSubscriptionsForOriginReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_RequestPushPermissionReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_SetPublicTokenForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_SetPushAndNotificationsEnabledForOriginReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_ShowNotificationReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_SubscribeToPushServiceReply"_s, ReceiverName::AsyncReply, false, false },
    { "PushClientConnection_UnsubscribeFromPushServiceReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteAudioDestinationManager_StartAudioDestinationReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteAudioDestinationManager_StopAudioDestinationReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteAudioMediaStreamTrackRendererInternalUnitManager_CreateUnitReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteBarcodeDetector_DetectReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteBuffer_CopyReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteBuffer_MapAsyncReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteCDMInstanceProxy_InitializeWithConfigurationReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteCDMInstanceProxy_SetServerCertificateReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteCDMInstanceSessionProxy_CloseSessionReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteCDMInstanceSessionProxy_LoadSessionReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteCDMInstanceSessionProxy_RemoveSessionDataReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteCDMInstanceSessionProxy_RequestLicenseReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteCDMInstanceSessionProxy_UpdateLicenseReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteCDMProxy_GetSupportedConfigurationReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteDevice_CreateComputePipelineAsyncReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteDevice_CreateRenderPipelineAsyncReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteDevice_PopErrorScopeReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteDevice_ResolveDeviceLostPromiseReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteDevice_ResolveUncapturedErrorEventReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteFaceDetector_DetectReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaEngineConfigurationFactoryProxy_CreateDecodingConfigurationReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaEngineConfigurationFactoryProxy_CreateEncodingConfigurationReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaPlayerProxy_DidLoadingProgressReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(MEDIA_SOURCE)
    { "RemoteMediaPlayerProxy_LoadMediaSourceReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "RemoteMediaPlayerProxy_LoadReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaPlayerProxy_PerformTaskAtTimeReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaPlayerProxy_RequestHostingContextIDReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaRecorderManager_CreateRecorderReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaRecorder_FetchDataReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaRecorder_PauseReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaRecorder_ResumeReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaRecorder_StopRecordingReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaResourceManager_DataReceivedReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaResourceManager_RedirectReceivedReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaResourceManager_ResponseReceivedReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaSessionCoordinatorProxy_CoordinatePauseReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaSessionCoordinatorProxy_CoordinatePlayReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaSessionCoordinatorProxy_CoordinateSeekToReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaSessionCoordinatorProxy_CoordinateSetTrackReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaSessionCoordinatorProxy_JoinReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaSessionCoordinator_PauseSessionReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaSessionCoordinator_PlaySessionReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaSessionCoordinator_SeekSessionToTimeReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteMediaSessionCoordinator_SetSessionTrackReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteQueue_OnSubmittedWorkDoneReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteQueue_WriteBufferReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteQueue_WriteTextureReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteRenderingBackend_GetImageBufferResourceLimitsForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteRenderingBackend_GetRemoteBarcodeDetectorSupportedFormatsReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteSampleBufferDisplayLayerManager_CreateLayerReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteShaderModule_CompilationInfoReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteSourceBufferProxy_AppendReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteSourceBufferProxy_BufferedSamplesForTrackIdReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteSourceBufferProxy_ComputeSeekTimeReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteSourceBufferProxy_EnqueuedSamplesForTrackIDReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteSourceBufferProxy_RemoveCodedFramesReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteSourceBufferProxy_SetMaximumBufferSizeReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteTextDetector_DetectReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteWebInspectorUIProxy_LoadReply"_s, ReceiverName::AsyncReply, false, false },
    { "RemoteWebInspectorUIProxy_PickColorFromScreenReply"_s, ReceiverName::AsyncReply, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateBufferedChangedReply"_s, ReceiverName::AsyncReply, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateDidReceiveInitializationSegmentReply"_s, ReceiverName::AsyncReply, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateDurationChangedReply"_s, ReceiverName::AsyncReply, false, false },
    { "SourceBufferPrivateRemoteMessageReceiver_SourceBufferPrivateShuttingDownReply"_s, ReceiverName::AsyncReply, false, false },
    { "UserMediaCaptureManager_GetMediaStreamDevicesReply"_s, ReceiverName::AsyncReply, false, false },
    { "UserMediaCaptureManager_ValidateUserMediaRequestConstraintsReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_ComputeElementLayoutReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_DeleteCookieReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_GetComputedLabelReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_GetComputedRoleReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_GetCookiesForFrameReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_ResolveChildFrameWithNameReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_ResolveChildFrameWithNodeHandleReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_ResolveChildFrameWithOrdinalReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_ResolveParentFrameReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_SelectOptionElementReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_SetFilesForInputFileUploadReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebAutomationSessionProxy_SnapshotRectForScreenshotReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebBroadcastChannelRegistry_PostMessageToRemoteReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebCookieManager_DeleteAllCookiesModifiedSinceReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebCookieManager_DeleteAllCookiesReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebCookieManager_DeleteCookieReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebCookieManager_DeleteCookiesForHostnamesReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebCookieManager_GetAllCookiesReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebCookieManager_GetCookiesReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebCookieManager_GetHTTPCookieAcceptPolicyReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebCookieManager_GetHostnamesWithCookiesReply"_s, ReceiverName::AsyncReply, false, false },
#if USE(SOUP)
    { "WebCookieManager_ReplaceCookiesReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebCookieManager_SetCookieReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebCookieManager_SetCookiesReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebCookieManager_SetHTTPCookieAcceptPolicyReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebFullScreenManager_RequestRestoreFullScreenReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebInspectorUIProxy_LoadReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebInspectorUIProxy_PickColorFromScreenReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebLockRegistryProxy_AbortLockRequestReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebLockRegistryProxy_SnapshotReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(WRITING_TOOLS_UI)
    { "WebPageProxy_AddTextAnimationForAnimationIDReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if USE(SYSTEM_PREVIEW)
    { "WebPageProxy_BeginSystemPreviewReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPageProxy_DecidePolicyForNavigationActionAsyncReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_DecidePolicyForNewWindowActionReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_DecidePolicyForResponseReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_DocumentURLForConsoleLogReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(MEDIA_STREAM)
    { "WebPageProxy_EnumerateMediaDevicesForFrameReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPageProxy_FocusFromServiceWorkerReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_HandleMessageWithAsyncReplyReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(ARKIT_INLINE_PREVIEW)
    { "WebPageProxy_ModelElementAnimationCurrentTimeReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_ModelElementAnimationDurationReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
    { "WebPageProxy_ModelElementCreateRemotePreviewReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW)
    { "WebPageProxy_ModelElementGetCameraReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_ModelElementHasAudioReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_ModelElementIsLoopingAnimationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_ModelElementIsMutedReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_ModelElementIsPlayingAnimationReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
    { "WebPageProxy_ModelElementLoadRemotePreviewReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW)
    { "WebPageProxy_ModelElementSetAnimationCurrentTimeReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_ModelElementSetAnimationIsPlayingReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_ModelElementSetCameraReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_ModelElementSetIsLoopingAnimationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_ModelElementSetIsMutedReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
    { "WebPageProxy_ModelElementSizeDidChangeReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_ModelInlinePreviewUUIDsReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPageProxy_RequestCookieConsentReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_RequestNotificationPermissionReply"_s, ReceiverName::AsyncReply, false, false },
#if USE(QUICK_LOOK)
    { "WebPageProxy_RequestPasswordForQuickLookDocumentInMainFrameReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(IMAGE_ANALYSIS)
    { "WebPageProxy_RequestTextRecognitionReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPageProxy_SendMessageToWebViewWithReplyReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(DEVICE_ORIENTATION)
    { "WebPageProxy_ShouldAllowDeviceOrientationAndMotionAccessReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPageProxy_ShowContactPickerReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(GTK)
    { "WebPageProxy_ShowEmojiPickerReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(MEDIA_CONTROLS_CONTEXT_MENUS) && USE(UICONTEXTMENU)
    { "WebPageProxy_ShowMediaControlsContextMenuReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(PDF_PLUGIN) && PLATFORM(MAC)
    { "WebPageProxy_ShowPDFContextMenuReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPageProxy_ShowShareSheetReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(SPEECH_SYNTHESIS)
    { "WebPageProxy_SpeechSynthesisPauseReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_SpeechSynthesisResumeReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_SpeechSynthesisSetFinishedCallbackReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageProxy_SpeechSynthesisSpeakReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(APPLE_PAY_AMS_UI)
    { "WebPageProxy_StartApplePayAMSUISessionReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPageProxy_WillSubmitFormReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageTesting_IsLayerTreeFrozenReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageTesting_SetPageScaleFactorReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPageTesting_SetTopContentInsetReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA)
    { "WebPage_AddDictationAlternativeReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_AddLayerForFindOverlayReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_AdjustVisibilityForTargetedElementsReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ApplyAutocorrectionReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(MAC)
    { "WebPage_AttributedSubstringForCharacterRangeAsyncReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_BeginSelectionInDirectionReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_CharacterIndexForPointAsyncReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ClearServiceWorkerEntitlementOverrideReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_CompleteTextManipulationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_ComputePagesForPrintingDuringDOMPrintOperationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_ComputePagesForPrintingReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_CountStringMatchesReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(APP_HIGHLIGHTS)
    { "WebPage_CreateAppHighlightInSelectedRangeReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(MEDIA_SESSION_COORDINATOR)
    { "WebPage_CreateMediaSessionCoordinatorReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(WRITING_TOOLS_UI)
    { "WebPage_CreateTextIndicatorForTextAnimationIDReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(DATA_DETECTION)
    { "WebPage_DetectDataInAllFramesReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_DictationAlternativesAtSelectionReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_DidGetLoadDecisionForIconReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_DispatchWheelEventWithoutScrollingReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(DRAG_SUPPORT)
    { "WebPage_DragEndedReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(GTK)
    { "WebPage_DrawPagesForPrintingDuringDOMPrintOperationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_DrawPagesForPrintingReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_DrawPagesToPDFDuringDOMPrintOperationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_DrawPagesToPDFReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_DrawRectToImageDuringDOMPrintOperationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_DrawRectToImageReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if (PLATFORM(COCOA) && PLATFORM(IOS_FAMILY))
    { "WebPage_DrawToImageReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_DrawToPDFReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if (PLATFORM(COCOA) && PLATFORM(IOS_FAMILY))
    { "WebPage_DrawToPDFiOSReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_EndPrintingDuringDOMPrintOperationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_EndPrintingReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_ExecuteEditCommandWithCallbackReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ExtendSelectionForReplacementReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_ExtendSelectionReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_FindRectsForStringMatchesReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(IMAGE_ANALYSIS)
    { "WebPage_FindStringIncludingImagesReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_FindStringMatchesReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_FindStringReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_FindTextRangesForStringMatchesReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA)
    { "WebPage_FirstRectForCharacterRangeAsyncReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_FocusNextFocusedElementReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_FocusTextInputContextAndPlaceCaretReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_GetAccessibilityTreeDataReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(APPLICATION_MANIFEST)
    { "WebPage_GetApplicationManifestReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_GetBytecodeProfileReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA)
    { "WebPage_GetContentsAsAttributedStringReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(MHTML)
    { "WebPage_GetContentsAsMHTMLDataReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_GetContentsAsStringReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetFrameInfoReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetFrameTreeReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetLoadedSubresourceDomainsReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetMainResourceDataOfFrameReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA)
    { "WebPage_GetMarkedRangeAsyncReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_GetPDFFirstPageSizeReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetProcessDisplayNameReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_GetRectsAtSelectionOffsetWithTextReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetRectsForGranularityWithSelectionOffsetReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_GetRenderTreeExternalRepresentationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetResourceDataFromFrameReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetSamplingProfilerOutputReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA)
    { "WebPage_GetSelectedRangeAsyncReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_GetSelectionAsWebArchiveDataReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_GetSelectionContextReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_GetSelectionOrContentsAsStringReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetSourceForFrameReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetTextFragmentMatchReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetWebArchiveOfFrameReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_GetWebArchiveOfFrameWithFileNameReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_HandleWheelEventReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA)
    { "WebPage_HasMarkedTextReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(ATTACHMENT_ELEMENT)
    { "WebPage_InsertAttachmentReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
    { "WebPage_InsertDroppedImagePlaceholdersReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(COCOA)
    { "WebPage_InsertTextPlaceholderReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_LastNavigationWasAppInitiatedReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_LoadAndDecodeImageReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_MoveSelectionAtBoundaryWithDirectionReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_MoveSelectionByOffsetReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_NavigateServiceWorkerClientReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_NumberOfVisibilityAdjustmentRectsReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(PDF_PLUGIN) && PLATFORM(MAC)
    { "WebPage_OpenPDFWithPreviewReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
    { "WebPage_PauseAllAnimationsReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_PauseAllMediaPlaybackReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_PerformActionOnElementReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(GTK) && ENABLE(DRAG_SUPPORT) || !PLATFORM(GTK) && ENABLE(DRAG_SUPPORT)
    { "WebPage_PerformDragControllerActionReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if !PLATFORM(GTK) && ENABLE(DRAG_SUPPORT)
    { "WebPage_PerformDragOperationReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_PerformHitTestForMouseEventReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
    { "WebPage_PlayAllAnimationsReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if (PLATFORM(IOS_FAMILY) && ENABLE(REVEAL))
    { "WebPage_PrepareSelectionForContextMenuWithLocationInViewReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_RemoteDictionaryPopupInfoToRootViewReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_RemoteViewPointToRootViewReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_RemoteViewRectToRootViewReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(DATA_DETECTION)
    { "WebPage_RemoveDataDetectedLinksReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_RemoveLayerForFindOverlayReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA)
    { "WebPage_RemoveTextPlaceholderReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_ReplaceMatchesReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(COCOA)
    { "WebPage_RequestActiveNowPlayingSessionInfoReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_RequestAutocorrectionDataReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_RequestDocumentEditingContextReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_RequestEvasionRectsAboveSelectionReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_RequestFocusedElementInformationReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_RequestFontAttributesAtSelectionStartReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_RequestImageBitmapReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_RequestMediaPlaybackStateReply"_s, ReceiverName::AsyncReply, false, false },
#if (PLATFORM(IOS_FAMILY) && ENABLE(REVEAL))
    { "WebPage_RequestRVItemInCurrentSelectedRangeReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_RequestRectForFoundTextRangeReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_RequestTargetedElementReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_RequestTextExtractionReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_ResetVisibilityAdjustmentsForTargetedElementsReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_ResumeAllMediaPlaybackReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_ResumeReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_RunJavaScriptInFrameInScriptWorldReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(PDF_PLUGIN) && PLATFORM(MAC)
    { "WebPage_SavePDFReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SelectPositionAtBoundaryWithDirectionReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_SelectPositionAtPointReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_SelectTextWithGranularityAtPointReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_SelectWithGestureReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_SelectWithTwoTouchesReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebPage_SendMessageToWebProcessExtensionWithReplyReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_SetActivityStateReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_SetInitialFocusReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_SetMutedReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(IMAGE_ANALYSIS_ENHANCEMENTS)
    { "WebPage_ShouldAllowRemoveBackgroundReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_ShouldDismissKeyboardAfterTapAtPointReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_SimulateClickOverFirstMatchingTextInViewportWithUserInteractionReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(VIDEO_PRESENTATION_MODE)
    { "WebPage_StartPlayingPredominantVideoReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_StartTextManipulationsReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_StopMediaCaptureReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_SuspendAllMediaPlaybackReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_SuspendReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_TakeSnapshotForTargetedElementReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_TakeSnapshotReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_TextInputContextsInRectReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if !ENABLE(IOS_TOUCH_EVENTS) && ENABLE(TOUCH_EVENTS)
    { "WebPage_TouchEventReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_TryCloseReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_URLSchemeTaskWillPerformRedirectionReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(ATTACHMENT_ELEMENT)
    { "WebPage_UpdateAttachmentAttributesReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_UpdateRenderingWithForcedRepaintReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPage_UpdateSelectionWithDeltaReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_UpdateSelectionWithExtentPointAndBoundaryReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_UpdateSelectionWithExtentPointReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebPage_UpdateSelectionWithTouchesReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(WRITING_TOOLS_UI)
    { "WebPage_UpdateUnderlyingTextVisibilityForTextAnimationIDReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(IMAGE_ANALYSIS)
    { "WebPage_UpdateWithTextRecognitionResultReply"_s, ReceiverName::AsyncReply, false, false },
#endif
#if ENABLE(MEDIA_STREAM)
    { "WebPage_UserMediaAccessWasGrantedReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPage_ValidateCommandReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(WRITING_TOOLS)
    { "WebPage_WillBeginWritingToolsSessionReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebPermissionControllerProxy_QueryReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(GAMEPAD)
    { "WebProcessPool_PlayGamepadEffectReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcessPool_StopGamepadEffectsReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcessPool_StoppedUsingGamepadsReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebProcessProxy_GetNotificationsReply"_s, ReceiverName::AsyncReply, false, false },
#if PLATFORM(GTK) || PLATFORM(WPE)
    { "WebProcessProxy_SendMessageToWebContextWithReplyReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebProcess_ClearCachedPageReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_DeleteAllCookiesReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_DeleteWebsiteDataForOriginReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_DeleteWebsiteDataForOriginsReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_DeleteWebsiteDataReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_EstablishRemoteWorkerContextConnectionToNetworkProcessReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_FetchWebsiteDataReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_GetActivePagesOriginsForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_InitializeWebProcessReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_IsJITEnabledReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_PrepareToSuspendReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_ReleaseMemoryReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_ReloadExecutionContextsForOriginReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_SeedResourceLoadStatisticsForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_SendResourceLoadStatisticsDataImmediatelyReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_SetDomainsWithCrossPageStorageAccessReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_SetIsInProcessCacheReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebProcess_SetThirdPartyCookieBlockingModeReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebResourceLoader_WillSendRequestReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWClientConnection_FocusServiceWorkerClientReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWClientConnection_SetServiceWorkerClientIsControlledReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWContextManagerConnection_FireBackgroundFetchClickEventReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWContextManagerConnection_FireBackgroundFetchEventReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWContextManagerConnection_FireNotificationEventReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWContextManagerConnection_FirePushEventReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_AbortBackgroundFetchReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_BackgroundFetchIdentifiersReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_BackgroundFetchInformationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_DisableNavigationPreloadReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_EnableNavigationPreloadReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_GetNavigationPreloadStateReply"_s, ReceiverName::AsyncReply, false, false },
#if ENABLE(WEB_PUSH_NOTIFICATIONS)
    { "WebSWServerConnection_GetNotificationsReply"_s, ReceiverName::AsyncReply, false, false },
#endif
    { "WebSWServerConnection_GetPushPermissionStateReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_GetPushSubscriptionReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_GetRegistrationsReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_MatchBackgroundFetchReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_MatchRegistrationReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_RetrieveRecordResponseReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_ScheduleUnregisterJobInServerReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_SetNavigationPreloadHeaderValueReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_StartBackgroundFetchReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_StoreRegistrationsOnDiskReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_SubscribeToPushServiceReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_TerminateWorkerFromClientReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_UnsubscribeFromPushServiceReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_WhenRegistrationReadyReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerConnection_WhenServiceWorkerIsTerminatedForTestingReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerToContextConnection_ClaimReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerToContextConnection_FindClientByVisibleIdentifierReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerToContextConnection_FocusReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerToContextConnection_NavigateReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSWServerToContextConnection_OpenWindowReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebScreenOrientationManagerProxy_LockReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSharedWorkerContextManagerConnection_PostConnectEventReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebSharedWorkerObjectConnection_FetchScriptInClientReply"_s, ReceiverName::AsyncReply, false, false },
    { "WebUserContentControllerProxy_DidPostMessageReply"_s, ReceiverName::AsyncReply, false, false },
#if USE(AUDIO_SESSION)
    { "GPUConnectionToWebProcess_EnsureAudioSession"_s, ReceiverName::GPUConnectionToWebProcess, true, false },
#endif
    { "IPCConnectionTester_SyncMessage"_s, ReceiverName::IPCConnectionTester, true, false },
    { "IPCStreamTester_CheckAutoreleasePool"_s, ReceiverName::IPCStreamTester, true, false },
    { "IPCStreamTester_SyncCrashOnZero"_s, ReceiverName::IPCStreamTester, true, false },
    { "IPCStreamTester_SyncMessageEmptyReply"_s, ReceiverName::IPCStreamTester, true, false },
    { "IPCStreamTester_SyncMessageReturningSharedMemory1"_s, ReceiverName::IPCStreamTester, true, false },
    { "IPCTester_ReleaseConnectionTester"_s, ReceiverName::IPCTester, true, false },
    { "IPCTester_ReleaseStreamTester"_s, ReceiverName::IPCTester, true, false },
    { "IPCTester_StopMessageTesting"_s, ReceiverName::IPCTester, true, false },
    { "IPCTester_SyncPing"_s, ReceiverName::IPCTester, true, false },
    { "IPCTester_SyncPingEmptyReply"_s, ReceiverName::IPCTester, true, false },
    { "NetworkConnectionToWebProcess_BlobSize"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_BlobType"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_CookieRequestHeaderFieldValue"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_CookiesEnabledSync"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_CookiesForDOM"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_DomCookiesForHost"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_GetNetworkLoadInformationResponse"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_GetNetworkLoadIntermediateInformation"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_GetRawCookies"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_PerformSynchronousLoad"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_TakeNetworkLoadInformationMetrics"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkConnectionToWebProcess_TestProcessIncomingSyncMessagesWhenWaitingForSyncReply"_s, ReceiverName::NetworkConnectionToWebProcess, true, false },
    { "NetworkProcessProxy_TestProcessIncomingSyncMessagesWhenWaitingForSyncReply"_s, ReceiverName::NetworkProcessProxy, true, false },
    { "NetworkProcess_ProcessWillSuspendImminentlyForTestingSync"_s, ReceiverName::NetworkProcess, true, false },
    { "NetworkProcess_ResetServiceWorkerFetchTimeoutForTesting"_s, ReceiverName::NetworkProcess, true, false },
    { "NetworkProcess_SetCacheModelSynchronouslyForTesting"_s, ReceiverName::NetworkProcess, true, false },
    { "NetworkProcess_SetServiceWorkerFetchTimeoutForTesting"_s, ReceiverName::NetworkProcess, true, false },
    { "NetworkStorageManager_ConnectToStorageAreaSync"_s, ReceiverName::NetworkStorageManager, true, false },
    { "NotificationManagerMessageHandler_GetPermissionStateSync"_s, ReceiverName::NotificationManagerMessageHandler, true, false },
    { "RemoteAdapter_RequestDevice"_s, ReceiverName::RemoteAdapter, true, false },
    { "RemoteAudioSessionProxy_TryToSetActive"_s, ReceiverName::RemoteAudioSessionProxy, true, false },
    { "RemoteBuffer_GetMappedRange"_s, ReceiverName::RemoteBuffer, true, false },
    { "RemoteCDMFactoryProxy_CreateCDM"_s, ReceiverName::RemoteCDMFactoryProxy, true, false },
    { "RemoteCDMFactoryProxy_SupportsKeySystem"_s, ReceiverName::RemoteCDMFactoryProxy, true, false },
    { "RemoteCDMInstanceProxy_CreateSession"_s, ReceiverName::RemoteCDMInstanceProxy, true, false },
    { "RemoteCDMProxy_CreateInstance"_s, ReceiverName::RemoteCDMProxy, true, false },
    { "RemoteCompositorIntegration_PaintCompositedResultsToCanvas"_s, ReceiverName::RemoteCompositorIntegration, true, false },
    { "RemoteCompositorIntegration_PrepareForDisplay"_s, ReceiverName::RemoteCompositorIntegration, true, false },
#if PLATFORM(COCOA)
    { "RemoteCompositorIntegration_RecreateRenderBuffers"_s, ReceiverName::RemoteCompositorIntegration, true, false },
#endif
    { "RemoteGPU_IsValid"_s, ReceiverName::RemoteGPU, true, false },
    { "RemoteGPU_RequestAdapter"_s, ReceiverName::RemoteGPU, true, false },
#if ENABLE(WEBXR)
    { "RemoteGraphicsContextGL_AddFoveation"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_CheckFramebufferStatus"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ClientWaitSync"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(VIDEO) && PLATFORM(COCOA)
    { "RemoteGraphicsContextGL_CopyTextureFromVideoFrame"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_DrawSurfaceBufferToImageBuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(WEBXR)
    { "RemoteGraphicsContextGL_EnableRequiredWebXRExtensions"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_FenceSync"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetActiveAttrib"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetActiveUniform"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetActiveUniformBlockName"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetActiveUniformBlockiv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetActiveUniforms"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetAttribLocation"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetBooleanv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetBufferParameteri"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetBufferSubDataInline"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetBufferSubDataSharedMemory"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetErrors"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetFloatv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetFragDataLocation"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetFramebufferAttachmentParameteri"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetInteger64"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetInteger64EXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetInteger64i"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetIntegeri_v"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetIntegerv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetInternalformativ"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetProgramInfoLog"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetProgrami"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetQuery"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetQueryObjectiEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetQueryObjectui"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetQueryObjectui64EXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetQueryiEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetRenderbufferParameteri"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetSamplerParameterf"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetSamplerParameteri"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetShaderInfoLog"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetShaderPrecisionFormat"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetShaderSource"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetShaderi"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetString"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetSynci"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetTexParameterf"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetTexParameteri"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetTransformFeedbackVarying"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetTranslatedShaderSourceANGLE"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetUniformBlockIndex"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetUniformIndices"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetUniformLocation"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetUniformfv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetUniformiv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetUniformuiv"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_GetVertexAttribOffset"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsBuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsEnabled"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsFramebuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsProgram"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsQuery"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsQueryEXT"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsRenderbuffer"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsSampler"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsShader"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsSync"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsTexture"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsTransformFeedback"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_IsVertexArray"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if PLATFORM(COCOA) || USE(GRAPHICS_LAYER_WC) || USE(GBM) || !PLATFORM(COCOA) && !USE(GRAPHICS_LAYER_WC) && !USE(GBM)
    { "RemoteGraphicsContextGL_PrepareForDisplay"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
    { "RemoteGraphicsContextGL_ReadPixelsInline"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
    { "RemoteGraphicsContextGL_ReadPixelsSharedMemory"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#if ENABLE(MEDIA_STREAM) || ENABLE(WEB_CODECS)
    { "RemoteGraphicsContextGL_SurfaceBufferToVideoFrame"_s, ReceiverName::RemoteGraphicsContextGL, true, false },
#endif
#if ENABLE(RE_DYNAMIC_CONTENT_SCALING)
    { "RemoteImageBufferSet_DynamicContentScalingDisplayList"_s, ReceiverName::RemoteImageBufferSet, true, false },
    { "RemoteImageBuffer_DynamicContentScalingDisplayList"_s, ReceiverName::RemoteImageBuffer, true, false },
#endif
    { "RemoteImageBuffer_FilteredNativeImage"_s, ReceiverName::RemoteImageBuffer, true, false },
    { "RemoteImageBuffer_FlushContextSync"_s, ReceiverName::RemoteImageBuffer, true, false },
    { "RemoteImageBuffer_GetPixelBuffer"_s, ReceiverName::RemoteImageBuffer, true, false },
    { "RemoteImageBuffer_GetPixelBufferWithNewMemory"_s, ReceiverName::RemoteImageBuffer, true, false },
    { "RemoteImageBuffer_GetShareableBitmap"_s, ReceiverName::RemoteImageBuffer, true, false },
    { "RemoteLegacyCDMFactoryProxy_CreateCDM"_s, ReceiverName::RemoteLegacyCDMFactoryProxy, true, false },
    { "RemoteLegacyCDMFactoryProxy_SupportsKeySystem"_s, ReceiverName::RemoteLegacyCDMFactoryProxy, true, false },
    { "RemoteLegacyCDMProxy_CreateSession"_s, ReceiverName::RemoteLegacyCDMProxy, true, false },
    { "RemoteLegacyCDMProxy_SupportsMIMEType"_s, ReceiverName::RemoteLegacyCDMProxy, true, false },
    { "RemoteLegacyCDMSessionProxy_CachedKeyForKeyID"_s, ReceiverName::RemoteLegacyCDMSessionProxy, true, false },
    { "RemoteLegacyCDMSessionProxy_GenerateKeyRequest"_s, ReceiverName::RemoteLegacyCDMSessionProxy, true, false },
    { "RemoteLegacyCDMSessionProxy_Update"_s, ReceiverName::RemoteLegacyCDMSessionProxy, true, false },
    { "RemoteMediaPlayerManagerProxy_GetSupportedTypes"_s, ReceiverName::RemoteMediaPlayerManagerProxy, true, false },
    { "RemoteMediaPlayerManagerProxy_SupportsKeySystem"_s, ReceiverName::RemoteMediaPlayerManagerProxy, true, false },
    { "RemoteMediaPlayerManagerProxy_SupportsTypeAndCodecs"_s, ReceiverName::RemoteMediaPlayerManagerProxy, true, false },
#if PLATFORM(IOS_FAMILY)
    { "RemoteMediaPlayerProxy_AccessLog"_s, ReceiverName::RemoteMediaPlayerProxy, true, false },
#endif
#if PLATFORM(COCOA)
    { "RemoteMediaPlayerProxy_ColorSpace"_s, ReceiverName::RemoteMediaPlayerProxy, true, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "RemoteMediaPlayerProxy_ErrorLog"_s, ReceiverName::RemoteMediaPlayerProxy, true, false },
#endif
    { "RemoteMediaPlayerProxy_IsCrossOrigin"_s, ReceiverName::RemoteMediaPlayerProxy, true, false },
#if PLATFORM(COCOA)
    { "RemoteMediaPlayerProxy_NativeImageForCurrentTime"_s, ReceiverName::RemoteMediaPlayerProxy, true, false },
#endif
    { "RemoteMediaPlayerProxy_VideoFrameForCurrentTimeIfChanged"_s, ReceiverName::RemoteMediaPlayerProxy, true, false },
    { "RemoteMediaSourceProxy_AddSourceBuffer"_s, ReceiverName::RemoteMediaSourceProxy, true, false },
#if PLATFORM(COCOA)
    { "RemoteRenderingBackend_PrepareImageBufferSetsForDisplaySync"_s, ReceiverName::RemoteRenderingBackend, true, false },
#endif
    { "RemoteSourceBufferProxy_CanSwitchToType"_s, ReceiverName::RemoteSourceBufferProxy, true, false },
    { "RemoteSourceBufferProxy_EvictCodedFrames"_s, ReceiverName::RemoteSourceBufferProxy, true, false },
    { "RemoteSourceBufferProxy_MinimumUpcomingPresentationTimeForTrackID"_s, ReceiverName::RemoteSourceBufferProxy, true, false },
#if PLATFORM(COCOA)
    { "RemoteVideoFrameObjectHeap_ConvertFrameBuffer"_s, ReceiverName::RemoteVideoFrameObjectHeap, true, false },
    { "RemoteVideoFrameObjectHeap_PixelBuffer"_s, ReceiverName::RemoteVideoFrameObjectHeap, true, false },
#endif
    { "WebFullScreenManagerProxy_SupportsFullScreen"_s, ReceiverName::WebFullScreenManagerProxy, true, false },
    { "WebPageProxy_AccessibilityScreenToRootView"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_BackForwardGoToItem"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_BackForwardItemAtIndex"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_BackForwardListContainsItem"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_BackForwardListCounts"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_BindRemoteAccessibilityFrames"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_CanUndoRedo"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_CheckGrammarOfString"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_CheckSpellingOfString"_s, ReceiverName::WebPageProxy, true, false },
#if USE(UNIFIED_TEXT_CHECKING)
    { "WebPageProxy_CheckTextOfParagraph"_s, ReceiverName::WebPageProxy, true, false },
#endif
    { "WebPageProxy_CreateNewPage"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_DecidePolicyForNavigationActionSync"_s, ReceiverName::WebPageProxy, true, false },
#if USE(DICTATION_ALTERNATIVES)
    { "WebPageProxy_DictationAlternatives"_s, ReceiverName::WebPageProxy, true, false },
#endif
#if PLATFORM(MAC)
    { "WebPageProxy_DismissCorrectionPanelSoon"_s, ReceiverName::WebPageProxy, true, false },
#endif
    { "WebPageProxy_ExceededDatabaseQuota"_s, ReceiverName::WebPageProxy, true, false },
#if PLATFORM(COCOA)
    { "WebPageProxy_ExecuteSavedCommandBySelector"_s, ReceiverName::WebPageProxy, true, false },
#endif
    { "WebPageProxy_ExecuteUndoRedo"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_FrameTextForTesting"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_GetGuessesForWord"_s, ReceiverName::WebPageProxy, true, false },
#if PLATFORM(COCOA)
    { "WebPageProxy_GetIsSpeaking"_s, ReceiverName::WebPageProxy, true, false },
#endif
    { "WebPageProxy_GetMenuBarIsVisible"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_GetStatusBarIsVisible"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_GetToolbarsAreVisible"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_GetWindowFrame"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_HandleSynchronousMessage"_s, ReceiverName::WebPageProxy, true, false },
#if PLATFORM(IOS_FAMILY)
    { "WebPageProxy_InterpretKeyEvent"_s, ReceiverName::WebPageProxy, true, false },
#endif
    { "WebPageProxy_LoadRecentSearches"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_LoadSynchronousURLSchemeTask"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_PrintFrame"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_RenderTreeAsTextForTesting"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_RequestDOMPasteAccess"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_RootViewToAccessibilityScreen"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_RootViewToScreen"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_RunBeforeUnloadConfirmPanel"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_RunJavaScriptAlert"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_RunJavaScriptConfirm"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_RunJavaScriptPrompt"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_ScreenToRootView"_s, ReceiverName::WebPageProxy, true, false },
#if ENABLE(ATTACHMENT_ELEMENT)
    { "WebPageProxy_SerializedAttachmentDataForIdentifiers"_s, ReceiverName::WebPageProxy, true, false },
#endif
#if ENABLE(SPEECH_SYNTHESIS)
    { "WebPageProxy_SpeechSynthesisVoiceList"_s, ReceiverName::WebPageProxy, true, false },
#endif
    { "WebPageProxy_SpellingUIIsShowing"_s, ReceiverName::WebPageProxy, true, false },
#if USE(APPKIT)
    { "WebPageProxy_SubstitutionsPanelIsShowing"_s, ReceiverName::WebPageProxy, true, false },
#endif
    { "WebPageProxy_UnwrapCryptoKey"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageProxy_WrapCryptoKey"_s, ReceiverName::WebPageProxy, true, false },
    { "WebPageTesting_IsEditingCommandEnabled"_s, ReceiverName::WebPageTesting, true, false },
    { "WebPage_BindRemoteAccessibilityFrames"_s, ReceiverName::WebPage, true, false },
#if (PLATFORM(COCOA) && PLATFORM(IOS_FAMILY))
    { "WebPage_ComputePagesForPrintingiOS"_s, ReceiverName::WebPage, true, false },
#endif
    { "WebPage_FrameTextForTesting"_s, ReceiverName::WebPage, true, false },
#if PLATFORM(COCOA)
    { "WebPage_GetDataSelectionForPasteboard"_s, ReceiverName::WebPage, true, false },
    { "WebPage_GetStringSelectionForPasteboard"_s, ReceiverName::WebPage, true, false },
    { "WebPage_ReadSelectionFromPasteboard"_s, ReceiverName::WebPage, true, false },
#endif
    { "WebPage_RenderTreeAsTextForTesting"_s, ReceiverName::WebPage, true, false },
#if PLATFORM(COCOA)
    { "WebPage_ShouldDelayWindowOrderingEvent"_s, ReceiverName::WebPage, true, false },
#endif
#if PLATFORM(IOS_FAMILY)
    { "WebPage_SyncApplyAutocorrection"_s, ReceiverName::WebPage, true, false },
#endif
    { "WebPage_TestProcessIncomingSyncMessagesWhenWaitingForSyncReply"_s, ReceiverName::WebPage, true, true },
#if PLATFORM(COCOA)
    { "WebPasteboardProxy_AddPasteboardTypes"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
    { "WebPasteboardProxy_AllPasteboardItemInfo"_s, ReceiverName::WebPasteboardProxy, true, false },
    { "WebPasteboardProxy_ContainsStringSafeForDOMToReadForType"_s, ReceiverName::WebPasteboardProxy, true, false },
#if PLATFORM(COCOA)
    { "WebPasteboardProxy_ContainsURLStringSuitableForLoading"_s, ReceiverName::WebPasteboardProxy, true, false },
    { "WebPasteboardProxy_GetNumberOfFiles"_s, ReceiverName::WebPasteboardProxy, true, false },
    { "WebPasteboardProxy_GetPasteboardBufferForType"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
#if PLATFORM(COCOA) || PLATFORM(GTK)
    { "WebPasteboardProxy_GetPasteboardChangeCount"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
#if PLATFORM(COCOA)
    { "WebPasteboardProxy_GetPasteboardColor"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
    { "WebPasteboardProxy_GetPasteboardItemsCount"_s, ReceiverName::WebPasteboardProxy, true, false },
#if PLATFORM(COCOA)
    { "WebPasteboardProxy_GetPasteboardPathnamesForType"_s, ReceiverName::WebPasteboardProxy, true, false },
    { "WebPasteboardProxy_GetPasteboardStringForType"_s, ReceiverName::WebPasteboardProxy, true, false },
    { "WebPasteboardProxy_GetPasteboardStringsForType"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
#if PLATFORM(COCOA) || USE(LIBWPE)
    { "WebPasteboardProxy_GetPasteboardTypes"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
#if PLATFORM(COCOA)
    { "WebPasteboardProxy_GetPasteboardURL"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
#if PLATFORM(GTK)
    { "WebPasteboardProxy_GetTypes"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
    { "WebPasteboardProxy_InformationForItemAtIndex"_s, ReceiverName::WebPasteboardProxy, true, false },
#if PLATFORM(GTK)
    { "WebPasteboardProxy_ReadBuffer"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
    { "WebPasteboardProxy_ReadBufferFromPasteboard"_s, ReceiverName::WebPasteboardProxy, true, false },
#if PLATFORM(GTK)
    { "WebPasteboardProxy_ReadFilePaths"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
    { "WebPasteboardProxy_ReadStringFromPasteboard"_s, ReceiverName::WebPasteboardProxy, true, false },
#if PLATFORM(GTK)
    { "WebPasteboardProxy_ReadText"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
    { "WebPasteboardProxy_ReadURLFromPasteboard"_s, ReceiverName::WebPasteboardProxy, true, false },
#if PLATFORM(COCOA)
    { "WebPasteboardProxy_SetPasteboardBufferForType"_s, ReceiverName::WebPasteboardProxy, true, false },
    { "WebPasteboardProxy_SetPasteboardColor"_s, ReceiverName::WebPasteboardProxy, true, false },
    { "WebPasteboardProxy_SetPasteboardStringForType"_s, ReceiverName::WebPasteboardProxy, true, false },
    { "WebPasteboardProxy_SetPasteboardTypes"_s, ReceiverName::WebPasteboardProxy, true, false },
    { "WebPasteboardProxy_SetPasteboardURL"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
#if (PLATFORM(COCOA) && ENABLE(IPC_TESTING_API))
    { "WebPasteboardProxy_TestIPCSharedMemory"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
    { "WebPasteboardProxy_TypesSafeForDOMToReadAndWrite"_s, ReceiverName::WebPasteboardProxy, true, false },
#if PLATFORM(COCOA)
    { "WebPasteboardProxy_URLStringSuitableForLoading"_s, ReceiverName::WebPasteboardProxy, true, false },
#endif
    { "WebPasteboardProxy_WriteCustomData"_s, ReceiverName::WebPasteboardProxy, true, false },
    { "WebProcessPool_HandleSynchronousMessage"_s, ReceiverName::WebProcessPool, true, false },
    { "WebProcessProxy_GetNetworkProcessConnection"_s, ReceiverName::WebProcessProxy, true, false },
#if PLATFORM(MAC) || PLATFORM(MACCATALYST)
    { "WebProcessProxy_IsAXAuthenticated"_s, ReceiverName::WebProcessProxy, true, false },
#endif
    { "WebProcessProxy_ShouldTerminate"_s, ReceiverName::WebProcessProxy, true, false },
    { "WebProcessProxy_UnwrapCryptoKey"_s, ReceiverName::WebProcessProxy, true, false },
    { "WebProcessProxy_WaitForSharedPreferencesForWebProcessToSync"_s, ReceiverName::WebProcessProxy, true, false },
    { "WebProcessProxy_WrapCryptoKey"_s, ReceiverName::WebProcessProxy, true, false },
    { "WebScreenOrientationManagerProxy_CurrentOrientation"_s, ReceiverName::WebScreenOrientationManagerProxy, true, false },
    { "WrappedAsyncMessageForTesting"_s, ReceiverName::IPC, true, false },
    { "<invalid message name>"_s, ReceiverName::Invalid, false, false }
};

} // namespace IPC::Detail
